c$$$  main program documentation block                                  
c                .      .    .                                       .  
c main program:  sstbias     corrects satellite sst biases              
c   prgmmr: reynolds         org: np24        date: 1993-01-07            
c                                                                       
c abstract: reads in the satellite and in situ (ship and buoy) data     
c  and computes three sst analyses on a 4 degree grid: in situ,         
c  daytime satellite and nighttime satellite.  the date are converted   
c  to increments using the oi first guess. using the poisson            
c  technique of reynolds (1988, j. of climate), a bias adjustment       
c  is computed using for both daytime and nighttime satellite data.     
c  the bias adjustment field has a spatial resolution of about 10       
c  degrees.  the daytime and nighttime adjustment is interpolated       
c  to the location of the individual day and night satellite            
c  observation (as appropriate) and added to the observations.          
c  these corrected satellite observations are written out for use       
c  in the oi step which follows.                                        
c                                                                       
c program history log:                                                  
c   1991-07-22  richard w. reynolds                                       
c   1991-09-14  richard w. reynolds - updated to screen in situ and       
c               satellite data before being used to compute bias          
c               adjustment                                                
c   1996-08-07  diane c. stokes - minor modifications to run on cray
c   1996-09-24  diane c. stokes - added output of correction fields to 
c               be archived in wbiasarc
c   1997-05-15  diane c. stokes 
c                - commented out cosine weighting for bias correction  
c                - land sea mask changed to weights (0.,.25,.5,.75,1.)
c                - masked out Caspian Sea and Hudson Bay for bias corr
c                - set polar limits to ice for blend 
c   1997-08-21  diane c. stokes 
c                - set satellite forcing to 0
c                - add count of superobs by type            
c                - no correction if num 4-deg insitu boxes w/data <500
c                - output intermediate fields for viewing with GrADS
c                - fixed limits of do_loop 114.  was overwriting
c                  new (97/5/15) hard ice limits with sat/insitu.
c   1999-12-09  diane c. stokes 
c                - changed denominator in intrp1 to
c                -           bot = ac + bc + ad + bd
c                - fixed if check for mask
c                - moved mask set-up forward in code and applying to
c                  input data before fields are generated.  previously
c                  only used for application of correction.
c                  Not sure yet if this will work well
c                - mask Gulf of California when generating fields.
c                                                                       
c usage:                                                                
c   input files:                                                        
c     fort.11  -  ship super observations: buoy.grid                   
c     fort.12  -  buoy super observations: ship.grid                   
c     fort.13  -  ice super observations: ice.grid                     
c     fort.14  -  daytime satellite super observations: dsat.grid      
c     fort.15  -  nighttime satellite super observations: nsat.grid    
c     fort.31  -  1 degree landsea tags                                 
c     fort.41  -  oi first guess                                        
c                                                                       
c   output files:  (including scratch files)                            
c     fort.06  -  printout                                              
c     fort.50  -  correction fields : 1-degree grid day&night
c     fort.51  -  daytime satellite super observations: dsat.cgrid
c              -  bias corrected                                        
c     fort.52  -  nighttime satellite super observations: nsat.cgrid
c              -  bias corrected                                        
c     fort.87  -  scratch file for subroutine weaver                    
c                                                                       
c   subprograms called:                                                 
c     unique:    - conv14, anal4 , forfcn, relax,  expand, minmax,      
c                - biascr, gengrd, tukey,  tukeyf, extrap,              
c                - intrp ,intrp1                                        
c     library:                                                          
c       common   - weaver                                               
c                                                                       
c   exit states:                                                        
c     cond =   0 - successful run                                       
c          =  02 - premature end of input data (units 11-15)            
c          =  10 - premature end of first guess oi (unit 10)            
c          =  03 - premature end of input data (units 14-15)            
c                  in subr. biascr                                      
c                                                                       
c remarks: list caveats, other helpful hints or information             
c                                                                       
c attributes:                                                           
c   language: fortran 77 
c   machine:  cray                                                       
c                                                                       
c$$$                                                                    
c                                                                       
c     for sst4                                                          
c                                                                       
c   long:   1,  21,  41,  61,  81, 101, 121, 141, 161,-179,             
c   index:  1,   6,  11,  16,  21,  26,  31,  36,  41,  46,             
c                                                                       
c   long:   -159,-139,-119, -99, -79, -59, -39, -19,  -3.               
c   index:    51,  56,  61,  66,  71,  76,  81,  86,  90.               
c                                                                       
c   lat:   -87,-67,-47,-27, -7, 13, 33, 53, 73, 89.                     
c   index:   1,  6, 11, 16, 21, 26, 31, 35, 41, 45.                     
c                                                                       
c      for sst1                                                         
c                                                                       
c   long:    0.5,  20.5,  40.5,  60.5,  80.5, 100.5, 120.5, 140.5,      
c   index:     1,    21,    41,    61,    81,   101,   121,   141,      
c                                                                       
c   long:  160.5,-179.5,-159.5,-139.5,-119.5, -99.5, -79.5, -59.5,      
c   index:   161,   181,   201,   221,   241,   261,   281,   301,      
c                                                                       
c   long:  -39.5, -19.5,  -0.5.                                         
c   index:   321,   341,   360.                                         
c                                                                       
c   lat :  -89.5, -69.5, -49.5, -29.5,  -9.5,  10.5,  30.5,  50.5,      
c   index:     1,    21,    41,    61,    81,   101,   121,   141,      
c                                                                       
c   lat :   70.5,  89.5.                                                
c   index:   161,   180.                                                
c                                                                       
c      include 'indexb.h'
      dimension sstoi(380,180)                                          
      dimension ship1(360,180),ship4(90,45)                             
      dimension satd1(360,180),satd4(90,45)                             
      dimension bldd4(90,45),bldn4(90,45),force(90,45)                  
      dimension wkt(100,55),wkw(90,45)
      dimension wk1(380,180),weav(1000),wk4(90,45)                     
      dimension satn1(360,180),satn4(90,45)                             
      dimension nship1(360,180),nship4(90,45)                           
      dimension npbo1(360,180),npbo4(90,45)                           
      dimension nsatd1(360,180),nsatd4(90,45)                           
      dimension nsatn1(360,180),nsatn4(90,45)                           
      dimension fx(360),fy(180),cx(92),cy(47)                           
      dimension cosl(44),cosc(45)                                       
      dimension mask(360,180),amask(360,180) 
*     dimension yt(180),xt(380),ls(380,180),cosyt(180)                  
      dimension yt(180),xt(380),wt(380,180)
      dimension ictr(2000),rsst(2000),rlon(2000),rlat(2000)             
      dimension rstd(2000)                                              
      dimension big(5)                                                  
      integer bindex                                                  
      character*8 type                                                  
      character*10 title(5)                                             
      data big /3.,3.,3.,10.,10./                                       
      data title /'ship sst  ','buoy sst  ','ice sst  ',                
     1 'daysat sst','ntesat sst'/                                       
      read (5,*) bindex                                      
c    read in first guess
      read (41,end=2000) jyrst,jmst,jdst,jyrend,jmend,jdend,jdays        
      read (41,end=2000) ((sstoi(i,j),i=191,370),(sstoi(i,j),i=11,190),
     1   j=1,180)                                          
      if(jyrst.lt.100)then
        if(jyrst.lt.21)then
          jyrst=jyrst+2000
        else
          jyrst=jyrst+1900
        endif
        print*,'JYRST MODIFIED TO 4-DIGITS:',jyrst
      endif
      if(jyrend.lt.100)then
        if(jyrend.lt.21)then
          jyrend=jyrend+2000
        else
          jyrend=jyrend+1900
        endif
        print*,'JYREND MODIFIED TO 4-DIGITS:',jyrend
      endif

c  set missing values at south pole (and north if necessary)
c to ice temperature for weaver boundary condition.
      do i=11,370
        if(sstoi(i,1).lt.-1.8)sstoi(i,1)=-1.8
        if(sstoi(i,180).lt.-1.8)sstoi(i,180)=-1.8
      enddo
      
      do j=1,180
        do i=11,370
          if(sstoi(i,j).lt.-1.8)then
            wk1(i,j)=0.
          else
            wk1(i,j)=1.
          endif
        enddo
      enddo
     
c fill in land ssts over antartica for stability in weaver
c  (taken from OI code)
      do j=1,30
        do i=11,370
          if(sstoi(i,j).lt.-1.8)then
            sstoi(i,j)=-1.8
            wk1(i,j)=1.
          endif
        enddo
      enddo
      
      do j=1,180
        do i=1,10
          sstoi(i,j) = sstoi(i+360,j)
          sstoi(i+370,j) = sstoi(i+10,j)
        enddo
      enddo

      call weaver (weav,0,sstoi,wk1,380,180,-189.5,189.5,-89.5,89.5,
     1 5.,0,0,0,2,0,-1.,ichk)
      if(ichk.ne.0)then
        print*,' WEAVER DID NOT FILL GRID.  EXIT 77.'
        call exit (77)
      endif
      do j=1,180
        do i=1,10
          sstoi(i,j) = sstoi(i+360,j)
          sstoi(i+370,j) = sstoi(i+10,j)
        enddo
      enddo

      write (50)  jyrst,jmst,jdst,jyrend,jmend,jdend,jdays,bindex  
      print 312,jyrst,jmst,jdst,bindex                                          
  312 format (//'oi: jyrst =',i4,3x,'jmst =',i3,3x,'jdst =',i3,
     1 'bindex = ',i10)          
      iship = 5                                                         
*     isat = 1                                                          
      isat = 0   
      iice = 5                                                          
      sstice=-2.
      nprt = 0                                                          
      iice2 = iice*2                                                    
c note: ice is added to sat and ship fields                             
c if nobs lt isat forcing set to 0.                                     
c if nobs ge iship internal/external bc; no relaxation                  
c in ice coveraged regions nobs increased by iice for ships and         
c    by iice2 for satellite                                             
c                                                                       
c nprt = 0 will print arrays                                            
      print 55,iship,isat,iice,nprt                                     
   55 format (/'iship =',i3,3x,'isat =',i3,3x,'iice =',i3,3x,'nprt =',  
     1 i3)                                                              
      rad = 3.14159/180.                                                
      do 10 j=1,45                                                      
      xlat = ((j-1)*4. - 87.)*rad                                       
      cosc(j) = cos(xlat)                                               
   10 continue                                                          
      do 12 j=1,44                                                      
      xlat = ((j-1)*4. - 85.)*rad                                       
      cosl(j) = cos(xlat)                                               
   12 continue                                                          
      do 105 ii=1,360                                                   
      do 105 jj=1,180                                                   
      ship1(ii,jj) = 0.                                                 
      satd1(ii,jj) = 0.                                                 
      satn1(ii,jj) = 0.                                                 
      nship1(ii,jj) = 0                                                 
      npbo1(ii,jj) = 0                                                 
      nsatd1(ii,jj) = 0                                                 
      nsatn1(ii,jj) = 0                                                 
  105 continue                                                          

        open(31,form='unformatted',status='old',access='direct',
     1     recl=360*180*4)
        read (31,rec=1) amask                                                 
        mask=nint(amask)

        do 32 i=1,360                                                     
        ip = i + 10                                                       
        ix = i + 180                                                      
        if (ix.gt.360) ix = ix - 360                                      
        do 32 j=1,180                                                     
*       ival = 1                                                          
*       if (mask(ix,j).ge.3) ival = 0                                     
*       ls(ip,j) = ival                                                   
        if (mask(ix,j).eq.0) then
          wt(ip,j)=0.
        else
          wt(ip,j)=1.
        endif
   32   continue                                                          

c set Caspian Sea to land    (46E-55E, 36N-48N)
*       do i=47,55
        do i=227,235
          ip=i+10
          do j=127,138
            wt(ip,j)=0.
          enddo
        enddo
    
c set Hudson Bay to land    
c  Main area              (95W-73W, 51N-67N)
*       do i=266,287
        do i=86,107
          ip=i+10
          do j=142,157
            wt(ip,j)=0.
          enddo
        enddo
c  Fox Basin              (83W-72W, 67N-70N)
*       do i=278,288
        do i=98,108
          ip=i+10
          do j=158,160
            wt(ip,j)=0.
          enddo
        enddo
c  Feather mask along Hudson Strait (73W-70W, 61N-65N)
*       i=288
        i=108
        ip=i+10
        do j=152,155
          wt(ip,j)=0.25
        enddo
*       i=289
        i=109
        ip=i+10
        do j=152,154
          wt(ip,j)=0.5
        enddo
*       i=290
        i=110
        ip=i+10
        do j=152,153
          wt(ip,j)=0.75
        enddo

c  Set Gulf of California to land
        wt(247-180+10,121)=0.
        wt(246-180+10,121)=0.
        wt(246-180+10,122)=0.
        wt(247-180+10,122)=0.
        wt(247-180+10,120)=0.
        wt(248-180+10,120)=0.
        wt(248-180+10,119)=0.
        wt(249-180+10,119)=0.
        wt(248-180+10,118)=0.
        wt(249-180+10,118)=0.
        wt(250-180+10,118)=0.
        wt(249-180+10,117)=0.
        wt(250-180+10,117)=0.
        wt(251-180+10,117)=0.
        wt(250-180+10,116)=0.
        wt(251-180+10,116)=0.
        wt(252-180+10,116)=0.
        wt(250-180+10,115)=0.
        wt(251-180+10,115)=0.
        wt(252-180+10,115)=0.
        wt(253-180+10,115)=0.
c write mask to grads for checkout
      open(77,form='unformatted',access='direct',recl=380*180*4)
      write(77,rec=1)((wt(i,j),i=11,370),j=1,180)

      
c overlap mask
        do 33 j=1,180                                                     
        do 33 i=1,10                                  
*         ls(i,j) = ls(i+360,j)                      
*         ls(i+370,j) = ls(i+10,j)                        
          wt(i,j) = wt(i+360,j)                          
          wt(i+370,j) = wt(i+10,j)                                  
   33   continue                                                          

      do 100 iu=11,15                                                   
      iii = iu - 10                                                     
      irj = 0                                                           
      read (iu,end=101) iyrst,imst,idst,iyrend,imend,idend,ndays        
      if(iyrst.lt.100)then
        if(iyrst.lt.21)then
          iyrst=iyrst+2000
        else
          iyrst=iyrst+1900
        endif
        print*,'IYRST MODIFIED TO 4-DIGITS:',iyrst
      endif
      if(iyrend.lt.100)then
        if(iyrend.lt.21)then
          iyrend=iyrend+2000
        else
          iyrend=iyrend+1900
        endif
        print*,'IYREND MODIFIED TO 4-DIGITS:',iyrend
      endif

      print 260,title(iu-10),iyrst,imst,idst,iyrend,imend,idend,ndays   
  260 format (/,a10,3x,'start date = ',i4,2i3,3x,' end date = ',i4,2i3,
     1    3x,' num days = ',i3)  
   50 read (iu,end=1000) type,ict                                       
c     print 40,type,ict                                                 
c  40 format (' read',a10,3x,'numb super obs =',i10)                    
      if (ict.eq.0) go to 101                                           
      read (iu,end=1000) (ictr(ii),ii=1,ict)                            
      read (iu,end=1000) (rsst(ii),ii=1,ict)                            
      read (iu,end=1000) (rlat(ii),ii=1,ict)                            
      read (iu,end=1000) (rlon(ii),ii=1,ict)                            
*     read (iu,end=1000) (rstd(ii),ii=1,ict)                            
      read (iu,end=1000) 
      if (iu.lt.13) then                                                
      do 331 l=1,ict                                                    
      jj = rlat(l) + 91.                                                
      if (jj.eq.181) jj = 180                                           
      ii = rlon(l) + 1.                                                 
      if (ii.eq.361) ii = 1                                             
      if (jj.lt.1.or.jj.gt.180) go to 331                               
      if (ii.lt.1.or.ii.gt.360) go to 331                               
      kk = ii + 190                                                     
      if (kk.gt.370) kk = kk - 360                                      
c discarding obs not included in 1-degree land mask
c  so that obs in small lakes near coasts don't affect
c  fields and thus correction.  this will be mainly satellite obs 
      if(wt(kk,jj).eq.0)go to 331
      dif = rsst(l)-sstoi(kk,jj)                                        
      if (abs(dif).gt.big(iii)) then                                    
      irj = irj + 1                                                     
c     if (iii.eq.1) go to 331                                           
c     xlon = rlon(l)                                                    
c     if (xlon.gt.180.) xlon = xlon - 360.                              
c     print 937,irj,title(iii),type,rlat(l),xlon,rsst(l),sstoi(kk,jj),  
c    1 dif                                                              
c 937 format (1x,i5,2x,a10,2x,a8,3x,'lat =',f6.1,3x,'lon =',f7.1,3x,    
c    1 'sst sobs =',f6.1,3x,'guess =',f6.1,3x,'dif =',f6.1)             
      go to 331                                                         
      endif                                                             
      val = rsst(l)*float(ictr(l))                                      
      nship1(ii,jj) = nship1(ii,jj) + ictr(l)                           
      npbo1(ii,jj) = npbo1(ii,jj) + ictr(l) ! separate cnt with no ice                          
      ship1(ii,jj) = ship1(ii,jj) + val                                 
  331 continue                                                          
      go to 50                                                          
      endif                                                             
      if (iu.eq.13) then                                                
      do 330 l=1,ict                                                    
      jj = rlat(l) + 91.                                                
      if (jj.eq.181) jj = 180                                           
      ii = rlon(l) + 1.                                                 
      if (ii.eq.361) ii = 1                                             
      if (jj.lt.1.or.jj.gt.180) go to 330                               
      if (ii.lt.1.or.ii.gt.360) go to 330                               
      kk = ii + 190                                                     
      if (kk.gt.370) kk = kk - 360                                      
      if(wt(kk,jj).eq.0)go to 330
      dif = rsst(l)-sstoi(kk,jj)                                        
      if (abs(dif).gt.big(iii)) then                                    
      irj = irj + 1                                                     
c     xlon = rlon(l)                                                    
c     if (xlon.gt.180.) xlon = xlon - 360.                              
c     print 937,irj,title(iii),type,rlat(l),xlon,rsst(l),sstoi(kk,jj),  
c    1 dif                                                              
      go to 330                                                         
      endif                                                             
      val = rsst(l)*float(iice)                                         
      val2 = rsst(l)*float(iice2)                                       
      nsatd1(ii,jj) = nsatd1(ii,jj) + iice2                             
      satd1(ii,jj) = satd1(ii,jj) + val2                                
      nsatn1(ii,jj) = nsatn1(ii,jj) + iice2                             
      satn1(ii,jj) = satn1(ii,jj) + val2                                
      nship1(ii,jj) = nship1(ii,jj) + iice                              
      ship1(ii,jj) = ship1(ii,jj) + val                                 
  330 continue                                                          
      go to 50                                                          
      endif                                                             
      if (iu.eq.14) then                                                
      do 332 l=1,ict                                                    
      jj = rlat(l) + 91.                                                
      if (jj.eq.181) jj = 180                                           
      ii = rlon(l) + 1.                                                 
      if (ii.eq.361) ii = 1                                             
      if (jj.lt.1.or.jj.gt.180) go to 332                               
      if (ii.lt.1.or.ii.gt.360) go to 332                               
      kk = ii + 190                                                     
      if (kk.gt.370) kk = kk - 360                                      
      if(wt(kk,jj).eq.0)go to 332
      dif = rsst(l)-sstoi(kk,jj)                                        
      if (abs(dif).gt.big(iii)) then                                    
      irj = irj + 1                                                     
c     xlon = rlon(l)                                                    
c     if (xlon.gt.180.) xlon = xlon - 360.                              
c     print 937,irj,title(iii),type,rlat(l),xlon,rsst(l),sstoi(kk,jj),  
c    1 dif                                                              
      go to 332                                                         
      endif                                                             
      val = rsst(l)*float(ictr(l))                                      
      nsatd1(ii,jj) = nsatd1(ii,jj) + ictr(l)                           
      satd1(ii,jj) = satd1(ii,jj) + val                                 
  332 continue                                                          
      go to 50                                                          
      endif                                                             
      if (iu.eq.15) then                                                
      do 333 l=1,ict                                                    
      jj = rlat(l) + 91.                                                
      if (jj.eq.181) jj = 180                                           
      ii = rlon(l) + 1.                                                 
      if (ii.eq.361) ii = 1                                             
      if (jj.lt.1.or.jj.gt.180) go to 333                               
      if (ii.lt.1.or.ii.gt.360) go to 333                               
      kk = ii + 190                                                     
      if (kk.gt.370) kk = kk - 360                                      
      if(wt(kk,jj).eq.0)go to 333
      dif = rsst(l)-sstoi(kk,jj)                                        
      if (abs(dif).gt.big(iii)) then                                    
      irj = irj + 1                                                     
c     xlon = rlon(l)                                                    
c     if (xlon.gt.180.) xlon = xlon - 360.                              
c     print 937,irj,title(iii),type,rlat(l),xlon,rsst(l),sstoi(kk,jj),  
c    1 dif                                                              
      go to 333                                                         
      endif                                                             
      val = rsst(l)*float(ictr(l))                                      
      nsatn1(ii,jj) = nsatn1(ii,jj) + ictr(l)                           
      satn1(ii,jj) = satn1(ii,jj) + val                                 
  333 continue                                                          
      endif                                                             
      go to 50                                                          
  101 print 938,irj,title(iii),big(iii)                                 
  938 format (1x,i5,2x,a10,3x,'obs rejected',3x,'max incr (big) =',     
     1 f7.2)                                                            
  100 continue                                                          
*     do 110 i=1,360                                                    
*     do 110 j=1,180                                                    
*     if (nship1(i,j).gt.0) ship1(i,j) = ship1(i,j)/float(nship1(i,j))  
*     if (nsatd1(i,j).gt.0) satd1(i,j) = satd1(i,j)/float(nsatd1(i,j))  
*     if (nsatn1(i,j).gt.0) satn1(i,j) = satn1(i,j)/float(nsatn1(i,j))  
* 110 continue                                                          
      do 118 i=1,360                                                    
c  set southern limit to ice 
        do 110 j=1,9     ! cover first 2 4-degree boxes
          nship1(i,j)=iice
          ship1(i,j)=sstice
          nsatd1(i,j)=iice2
          satd1(i,j)=sstice
          nsatn1(i,j)=iice2
          satn1(i,j)=sstice
  110   continue                                                          
        do 114 j=10,177     
          if (nship1(i,j).gt.0) ship1(i,j)=ship1(i,j)/float(nship1(i,j))  
          if (nsatd1(i,j).gt.0) satd1(i,j)=satd1(i,j)/float(nsatd1(i,j))  
          if (nsatn1(i,j).gt.0) satn1(i,j)=satn1(i,j)/float(nsatn1(i,j))  
  114   continue                                                          
c  set northern limit to ice
        do 116 j=178,180   ! cover last 4-degree partial box
          nship1(i,j)=iice
          ship1(i,j)=sstice
          nsatd1(i,j)=iice2
          satd1(i,j)=sstice
          nsatn1(i,j)=iice2
          satn1(i,j)=sstice
  116   continue                                                          
  118 continue                                                          
c supplement satellite fields with ice north of 80n and south of 80s    
c     do 115 i=1,360                                                    
c     do 115 j=1,10                                                     
c     k = 181 - j                                                       
c     satd1(i,j) = ship1(i,j)                                           
c     nsatd1(i,j) = nship1(i,j)                                         
c     satn1(i,j) = ship1(i,j)                                           
c     nsatn1(i,j) = nship1(i,j)                                         
c     satd1(i,k) = ship1(i,k)                                           
c     nsatd1(i,k) = nship1(i,k)                                         
c     satn1(i,k) = ship1(i,k)                                           
c     nsatn1(i,k) = nship1(i,k)                                         
c 115 continue                                                          
c   set up the lat/lon indices:  cx & cy: 2 degrees; fx & fy: 1 degree  
      print 271                                                         
  271 format (/)                                                      
      do 270 jlon=1,360                                                 
      fx(jlon) = 0.5 + float(jlon-1)                                    
  270 continue                                                          
      do 275 jlat=1,180                                                 
      fy(jlat) = -89.5 + float(jlat-1)                                  
  275 continue                                                          
      do 280 jlon=1,92                                                  
      cx(jlon) = 4.*float(jlon-1) - 3.                                  
  280 continue                                                          
      do 285 jlat=1,47                                                  
      cy(jlat) = 4.*float(jlat-1) - 91.                                 
  285 continue                                                          
c     print 281,fx                                                      
c 281 format (' fx =',12f10.1)                                          
c     print 282,fy                                                      
c 282 format (' fy =',12f10.1)                                          
c     print 283,cx                                                      
  283 format (' cx =',12f10.1)                                          
c     print 284,cy                                                      
  284 format (' cy =',12f10.1)                                          
      isum = iyrst*10000 + imst*100 + idst                              
      jsum = jyrst*10000 + jmst*100 + jdst                              
      if (isum.ne.jsum) print 313                                       
  313 format (/'*** warning oi and data dates do not match'//)          

c convert to increments
      do 210 i=1,360                                                    
      ix = i + 190                                                      
      if (ix.gt.370) ix = ix - 360                                      
      do 210 j=1,180                                                    
      if (nship1(i,j).gt.0) ship1(i,j) = ship1(i,j) - sstoi(ix,j)       
      if (nsatd1(i,j).gt.0) satd1(i,j) = satd1(i,j) - sstoi(ix,j)       
      if (nsatn1(i,j).gt.0) satn1(i,j) = satn1(i,j) - sstoi(ix,j)       
  210 continue                                                          
c  interpolate ship&buoy_only count first, then recompute ship4&nship4
      call conv14 (ship1,npbo1,ship4,npbo4)                           
      call conv14 (ship1,nship1,ship4,nship4)                           
      call conv14 (satd1,nsatd1,satd4,nsatd4)                           
      call conv14 (satn1,nsatn1,satn4,nsatn4)                           

c     call minmax (ship1,'anom sst ship 1-deg           ',360,180)      
c     call minmax (satd1,'anom sst sat day 1-deg        ',360,180)      
c     call minmax (satn1,'anom sst sat nte 1-deg        ',360,180)      
c     call minmax (ship4,'anom sst ship 4-deg           ',45,90)        
c     call minmax (satd4,'anom sst sat day 4-deg        ',45,90)        
c     call minmax (satn4,'anom sst sat nte 4-deg        ',45,90)        
      call anal4 (ship4,nship4,wkw,wkt,90,45,100,55)                    
c     call minmax (ship4,'anom analy sst ship 4-deg     ',45,90)        
      call anal4 (satd4,nsatd4,wkw,wkt,90,45,100,55)                    
c     call minmax (satd4,'anom analy sst sat day 4-deg  ',45,90)        
      call anal4 (satn4,nsatn4,wkw,wkt,90,45,100,55)                    
c     call minmax (satn4,'anom analy sst sat nte 4-deg  ',45,90)        
*     print*
*     print*,' SHIP4 FROM ANAL4'
*     print '(20f5.1)',ship4
*     print*
*     print*,' SATD4 FROM ANAL4'
*     print '(20f5.1)',satd4
*     print*
*     print*,' SATN4 FROM ANAL4'
*     print '(20f5.1)',satn4

      npbocrit=0
      npcrit=0
      ndcrit=0
      nncrit=0
      do j=1,45
        do i=1,90
          if(npbo4(i,j).ge.iship)npbocrit=npbocrit+1
          if(nship4(i,j).ge.iship)npcrit=npcrit+1
          if(nsatd4(i,j).ge.isat)ndcrit=ndcrit+1
          if(nsatn4(i,j).ge.isat)nncrit=nncrit+1
        enddo
      enddo
      print*,' pbo is ship and buoy only.  ship is ship/buoy/ice'
      print*,'num 4-deg  pbo squares with at least iship obs:',npbocrit
      print*,'num 4-deg ship squares with at least iship obs:',npcrit
      print*,'num 4-deg dsat squares with at least isat obs: ',ndcrit
      print*,'num 4-deg nsat squares with at least isat obs: ',nncrit
      print*

      if(npbocrit.ge.500)then
        do 400 i=1,90                                                     
        do 400 j=1,45                                                     
        bldd4(i,j) = ship4(i,j)                                           
        bldn4(i,j) = ship4(i,j)                                           
  400   continue                                                          
        call forfcn (satd4,nsatd4,force,cosc,cosl,isat)                   
*       print*
*       print*,' FORCE FOR SATD4'
*       print '(20f5.1)',force
        call relax (bldd4,nship4,force,cosc,cosl,iship) 
*       print*
*       print*,' BLDD4 FROM RELAX'
*       print '(20f5.1)',bldd4
        call forfcn (satn4,nsatn4,force,cosc,cosl,isat)                   
*       print*
*       print*,' FORCE FOR SATN4'
*       print '(20f5.1)',force
        call relax (bldn4,nship4,force,cosc,cosl,iship)                   
*       print*
*       print*,' BLDN4 FROM RELAX'
*       print '(20f5.1)',bldn4
        do 410 i=1,90                                                     
        do 410 j=1,45                                                     
        bldd4(i,j) = bldd4(i,j) - satd4(i,j)                              
        bldn4(i,j) = bldn4(i,j) - satn4(i,j)                              
  410   continue                                                          
c - generating the 1 degree global grid centered on 1/2-degree nodes    
*       call gengrd (380,180,xt,yt,cosyt)                                 
        call gengrd (380,180,xt,yt)                                 
c       print 111,xt                                                      
c 111   format (' xt      =',14f8.1)                                      
c       print 112,yt                                                      
c 112   format (' yt      =',14f8.1)                                      
c       print 113,cosyt                                                   
c 113   format (' cosyt   =',14f8.3)                                      
*       print*
*       print*,' BLDD4 BEFORE EXPAND'
*       print '(20f5.1)',bldd4
        call expand (bldd4,cx,cy,fx,fy,sstoi)                             
        write(50)((sstoi(i,j),i=191,370),(sstoi(i,j),i=11,190),j=1,180)
        call minmax (sstoi,'sat day bias (bld-sat)  1-deg ',380,180)      
        iu = 14                                                           
        iw = 51                                                           
*       call biascr (sstoi,iu,iw,'sst daysat',ls,xt,yt,cosyt)             
        call biascr (sstoi,iu,iw,'sst daysat',wt,xt,yt)             
        call expand (bldn4,cx,cy,fx,fy,sstoi)                             
        write(50)((sstoi(i,j),i=191,370),(sstoi(i,j),i=11,190),j=1,180)
        call minmax (sstoi,'sat nte bias (bld-sat)  1-deg ',380,180)      
        iu = 15                                                           
        iw = 52                                                           
*       call biascr (sstoi,iu,iw,'sst ntesat',ls,xt,yt,cosyt)             
        call biascr (sstoi,iu,iw,'sst ntesat',wt,xt,yt)             
      else
        print 805
  805   format('****       NO BIAS CORRECTION DONE      ****'/
     1         '** FILL SAT FILES WITH ORIGINAL SAT DATA  **'/
     2         '**** FILL OUTPUT BIAS FILES WITH ZEROES ****')
        do 850 iu=14,15
          iw=iu+37
          rewind iu 
          read (iu,end=1000) iyrst,imst,idst,iyrend,imend,idend,ndays   
      if(iyrst.lt.100)then
        if(iyrst.lt.21)then
          iyrst=iyrst+2000
        else
          iyrst=iyrst+1900
        endif
        print*,'IYRST MODIFIED TO 4-DIGITS:',iyrst
      endif
      if(iyrend.lt.100)then
        if(iyrend.lt.21)then
          iyrend=iyrend+2000
        else
          iyrend=iyrend+1900
        endif
        print*,'IYREND MODIFIED TO 4-DIGITS:',iyrend
      endif

          write (iw)         iyrst,imst,idst,iyrend,imend,idend,ndays   
  810     read (iu,end=1000) type,ict                 
****NO*** type(8:8) = 'c'                                 
          write (iw) type,ict                           
          if (ict.eq.0) go to 850                      
          read (iu,end=1000) (ictr(ii),ii=1,ict)      
          read (iu,end=1000) (rsst(ii),ii=1,ict)     
          read (iu,end=1000) (rlat(ii),ii=1,ict)    
          read (iu,end=1000) (rlon(ii),ii=1,ict)   
*         read (iu,end=1000) (rstd(ii),ii=1,ict)  
          read (iu,end=1000) 
          write (iw) (ictr(ii),ii=1,ict)         
          write (iw) (rsst(ii),ii=1,ict)        
          write (iw) (rlat(ii),ii=1,ict)       
          write (iw) (rlon(ii),ii=1,ict)      
*         write (iw) (rstd(ii),ii=1,ict)     
          write (iw) 
          go to 810                        
  850   continue                                  
      endif
      call exit (1)
 1000 print 1001                                                        
 1001 format (' premature end of input data: exit 2')                   
      endfile 21                                                        
      endfile 22                                                        
      rewind 21                                                         
      rewind 22                                                         
      call exit (2)
 2000 print 2001                                                        
 2001 format (' premature end of input oi field: exit 10')              
      endfile 21                                                        
      endfile 22                                                        
      rewind 21                                                         
      rewind 22                                                         
      call exit (10)
      end                                                               
      subroutine conv14 (sst1,nsst1,sst4,nsst4)                         
c$$$  subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    conv14      interpolate 1-deg field to 4-deg field     
c   prgmmr: r. w. reynolds    org: w/np24      date: 1991-07-22           
c                                                                       
c abstract: interpolates 1 degree resolution sst superobs in sst1       
c   and number of superobs in nsst1 to 4 degree sst superobs in sst4    
c   and number of superobs in nsst4.                                    
c                                                                       
c program history log:                                                  
c   1991-07-22  richard w. reynolds                                       
c                                                                       
c usage:    call conv14 (sst1,nsst1,sst4,nsst4)                         
c                                                                       
c   input argument list:                                                
c     sst1   -   1-deg resolution sst superobs in deg c                 
c     nsst1  -   1-deg resolution number of obs in used in sst1         
c                                                                       
c   output argument list:      (including work arrays)                  
c     sst1   -   4-deg resolution sst superobs in deg c                 
c     nsst1  -   4-deg resolution number of obs in used in sst4         
c                                                                       
c remarks:  array indices correspond to locations as follows:           
c                                                                       
c     for sst4 and nsst4                                                
c                                                                       
c   long:   1,  21,  41,  61,  81, 101, 121, 141, 161,-179,             
c   index:  1,   6,  11,  16,  21,  26,  31,  36,  41,  46,             
c                                                                       
c   long:   -159,-139,-119, -99, -79, -59, -39, -19,  -3.               
c   index:    51,  56,  61,  66,  71,  76,  81,  86,  90.               
c                                                                       
c   lat:   -87,-67,-47,-27, -7, 13, 33, 53, 73, 89.                     
c   index:   1,  6, 11, 16, 21, 26, 31, 35, 41, 45.                     
c                                                                       
c      for sst1 and nsst1                                               
c                                                                       
c   long:    0.5,  20.5,  40.5,  60.5,  80.5, 100.5, 120.5, 140.5,      
c   index:     1,    21,    41,    61,    81,   101,   121,   141,      
c                                                                       
c   long:  160.5,-179.5,-159.5,-139.5,-119.5, -99.5, -79.5, -59.5,      
c   index:   161,   181,   201,   221,   241,   261,   281,   301,      
c                                                                       
c   long:  -39.5, -19.5,  -0.5.                                         
c   index:   321,   341,   360.                                         
c                                                                       
c   lat :  -89.5, -69.5, -49.5, -29.5,  -9.5,  10.5,  30.5,  50.5,      
c   index:     1,    21,    41,    61,    81,   101,   121,   141,      
c                                                                       
c   lat :   70.5,  89.5.                                                
c   index:   161,   180.                                                
c                                                                       
c                                                                       
c attributes:                                                           
c   language: fortran 77 
c   machine:  cray                                                       
c                                                                       
c$$$                                                                    
      dimension sst1(360,180),sst4(90,45)                               
      dimension nsst1(360,180),nsst4(90,45)                             
      do 10 i=1,90                                                      
      is = i*4 - 4                                                      
      ie = is + 3                                                       
      do 10 j=1,45                                                      
      js = j*4 - 2                                                      
      je = js + 3                                                       
      if (je.gt.180) je = 180                                           
      sval = 0.                                                         
      aval = 0.                                                         
      do 20 ii=is,ie                                                    
      ll = ii                                                           
      if (ll.eq.0) ll = 360                                             
      do 20 jj = js,je                                                  
      aa = nsst1(ll,jj)                                                 
      sval = sval + aa*sst1(ll,jj)                                      
      aval = aval + aa                                                  
   20 continue                                                          
      sst4(i,j) = 0.                                                    
      if (aval.gt.0.) sst4(i,j) = sval/aval                             
      nsst4(i,j) = aval                                                 
   10 continue                                                          
      return                                                            
      end                                                               
      subroutine minmax (sst,name,ii,jj)                                
c$$$  subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    minmax      print minimum and maximum of array         
c   prgmmr: r. w. reynolds    org: w/np24      date: 1991-07-22           
c                                                                       
c abstract: prints the minimum and maximum of sst array of size         
c  ii by jj.  also prints title.                                        
c                                                                       
c program history log:                                                  
c   1991-07-22  richard w. reynolds                                       
c                                                                       
c usage:    call minmax (sst,name,ii,jj)                                
c                                                                       
c   input argument list:                                                
c     sst   -   array to search for minimum and maximum                 
c     name  -   character*30 descriptive title of sst array             
c     ii    -   first dimension of sst                                  
c     jj    -   second dimension of sst                                 
c                                                                       
c   output files:                                                       
c     fort.06  - printout                                               
c                                                                       
c attributes:                                                           
c   language: fortran 77 
c   machine:  cray                                                       
c                                                                       
c$$$                                                                    
      dimension sst(ii,jj)                                              
      character*30 name                                                 
      amin = +1.e30                                                     
      amax = -amin                                                      
      do 10 i=1,ii                                                      
      do 10 j=1,jj                                                      
      if (amax.lt.sst(i,j)) amax = sst(i,j)                             
      if (amin.gt.sst(i,j)) amin = sst(i,j)                             
   10 continue                                                          
      print 20,name,amax,amin                                           
   20 format (/,a30,5x,'max =',f8.2,3x,'min =',f8.2)                  
      return                                                            
      end                                                               
      subroutine relax (ship,nship,force,cosc,cosl,iship)               
c$$$  subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    relax       finds solution to poisson equation.        
c   prgmmr: r. w. reynolds    org: w/np24      date: 1991-07-22           
c                                                                       
c abstract: solves poisson equation in spherical coordinates.           
c   the array ship is both input and output (see below).                
c                                                                       
c program history log:                                                  
c   1991-07-22  richard w. reynolds                                       
c                                                                       
c usage:    call relax (ship,nship,force,cosc,cosl,iship)               
c                                                                       
c   input argument list:                                                
c     ship   -   4-deg resolution in situ analyzed sst field in deg c   
c     nship  -   4-deg resolution number of ship obs used in ship       
c     force  -   4-deg resolution laplacian of satellite analysis       
c            -   may be either daytime or nighttime analysis            
c     cosc   -   cosine of latitude of center of grid box               
c     cosl   -   cosine of latitude of bottom of grid box               
c     iship  -   if nship(i,j) > or = iship, ship(i,j) is internal or   
c                external boundary condition and is not changed in      
c                relax                                                  
c                                                                       
c   output argument list:      (including work arrays)                  
c     ship   -   4-deg resolution blended (poisson solution)            
c                determined by solution of poisson equation where       
c                grid point is not fixed by internal or external        
c                boundary condition                                     
c                                                                       
c remarks:  array indices correspond to locations as follows:           
c                                                                       
c     for ship and nship                                                
c                                                                       
c   long:   1,  21,  41,  61,  81, 101, 121, 141, 161,-179,             
c   index:  1,   6,  11,  16,  21,  26,  31,  36,  41,  46,             
c                                                                       
c   long:   -159,-139,-119, -99, -79, -59, -39, -19,  -3.               
c   index:    51,  56,  61,  66,  71,  76,  81,  86,  90.               
c                                                                       
c   lat:   -87,-67,-47,-27, -7, 13, 33, 53, 73, 89.                     
c   index:   1,  6, 11, 16, 21, 26, 31, 35, 41, 45.                     
c                                                                       
c attributes:                                                           
c   language: fortran 77 
c   machine:  cray                                                       
c                                                                       
c$$$                                                                    
      dimension force(90,45),ship(90,45),cosc(45),cosl(44)              
      dimension nship(90,45)                                            
      w = 1.6                                                           
      e = 0.001                                                         
c     print 97,w,e                                                      
c  97 format (/'relax coef =',f10.4,3x,'max error =',f10.4/)            
      do 50 k=1,1000                                                    
      deltm = 0.0                                                       
      do 15 i=1,90                                                      
      ip = i + 1                                                        
      im = i - 1                                                        
      if (i.eq.1) im = 90                                               
      if (i.eq.90) ip = 1                                               
      do 20 j=2,44                                                      
      if (nship(i,j).ge.iship) go to 20                                 
      tll = ship(ip,j) + ship(im,j) - 2.*ship(i,j)                      
      txx = tll/(cosc(j)**2)                                            
      tpp = (ship(i,j+1)-ship(i,j))*cosl(j)                             
      tpm = (ship(i,j)-ship(i,j-1))*cosl(j-1)                           
      tyyc = (tpp - tpm)/cosc(j)                                        
      d2 = txx + tyyc                                                   
      r = d2 - force(i,j)                                               
      delt = r*w*cosc(j)**2/(2.*cosc(j)**2 + 2.)                        
      ship(i,j) = ship(i,j) + delt                                      
      deltm = amax1(abs(delt),deltm)                                    
   20 continue                                                          
   15 continue                                                          
      if (deltm.lt.e) then                                            
        print 66,k,deltm                                                  
   66   format('numb inter  = ',i4,3x,'deltm = ',f14.5)                  
        return                                                            
      endif
   50 continue                                                          
        print*,' 1000 reached'
        print 66,k,deltm                                                  
      return                                                            
      end                                                               
      subroutine forfcn (sat,nsat,force,cosc,cosl,isat)                 
c$$$  subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    forfcn       finds laplacian of satellite sst.         
c   prgmmr: r. w. reynolds    org: w/np24      date: 1991-07-22           
c                                                                       
c abstract: finds laplacian of satellite sst in spherical coordinates.  
c   the laplacian is set to zero if number of satellite obs is small    
c   (see below).                                                        
c                                                                       
c program history log:                                                  
c   1991-07-22  richard w. reynolds                                       
c                                                                       
c usage:    call forfcn (sat,nsat,force,cosc,cosl,isat)                 
c                                                                       
c   input argument list:                                                
c     sat    -   4-deg resolution satellite analyzed sst field          
c            -   in deg c                                               
c            -   may be either daytime or nighttime analysis            
c     nsat   -   4-deg resolution number of sat obs used in sat         
c     isat   -   if nsat(i,j) > or = isat, force(i,j) is set to 0       
c            -   external boundary condition and is not changed by      
c            -   relax                                                  
c                                                                       
c   output argument list:      (including work arrays)                  
c     force  -   4-deg resolution laplacian of satellite analysis       
c            -   set to 0 by relation between isat and nsat             
c     cosc   -   cosine of latitude of center of grid box               
c     cosl   -   cosine of latitude of bottom of grid box               
c                                                                       
c remarks:  array indices correspond to locations as follows:           
c                                                                       
c     for sat and nsat                                                  
c                                                                       
c   long:   1,  21,  41,  61,  81, 101, 121, 141, 161,-179,             
c   index:  1,   6,  11,  16,  21,  26,  31,  36,  41,  46,             
c                                                                       
c   long:   -159,-139,-119, -99, -79, -59, -39, -19,  -3.               
c   index:    51,  56,  61,  66,  71,  76,  81,  86,  90.               
c                                                                       
c   lat:   -87,-67,-47,-27, -7, 13, 33, 53, 73, 89.                     
c   index:   1,  6, 11, 16, 21, 26, 31, 35, 41, 45.                     
c                                                                       
c attributes:                                                           
c   language: fortran 77 
c   machine:  cray                                                       
c                                                                       
c$$$                                                                    
      dimension force(90,45),sat(90,45),cosc(45),cosl(44)               
      dimension nsat(90,45)                                             
      do 1 i=1,90                                                       
      do 1 j=1,45                                                       
      force(i,j) = 0.0                                                  
    1 continue                                                          
      l = 0                                                             
      do 15 i=1,90                                                      
      ip = i + 1                                                        
      im = i - 1                                                        
      if (i.eq.90) ip = 1                                               
      if (i.eq.1) im = 90                                               
c ignore satellite forcing in part of field filled with ice (>80n&,<80s)
      do 20 j=4,41                                                      
      if (nsat(i,j).lt.isat) go to 20                                   
      l = l + 1                                                         
      tll = sat(ip,j) + sat(im,j) - 2.*sat(i,j)                         
      txx = tll/(cosc(j)**2)                                            
      tpp = (sat(i,j+1)-sat(i,j))*cosl(j)                               
      tpm = (sat(i,j)-sat(i,j-1))*cosl(j-1)                             
      tyyc = (tpp - tpm)/cosc(j)                                        
      force(i,j) = txx + tyyc                                           
   20 continue                                                          
   15 continue                                                          
c     print 66,l                                                        
c  66 format (//'points forced by satellite =',i10)                      
      return                                                            
      end                                                               
      subroutine tukey (a,n,m)                                          
c$$$  subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    tukey       performs medium filtering of array a       
c   prgmmr: r. w. reynolds    org: w/np24      date: 1991-07-22           
c                                                                       
c abstract: provides multiple pass median filtering of array a.         
c   see appendix b of reynolds (1988, j. of climate) for details.       
c   array a is both input and output.                                   
c   filter is applied first to rows (east/west) then columns.           
c                                                                       
c program history log:                                                  
c   1991-07-22  richard w. reynolds                                       
c                                                                       
c usage:    call tukey (a,n,m)                                          
c                                                                       
c   input argument list:                                                
c     a      -   4-deg resolution input array to be filtered            
c     n      -   first dimension of a                                   
c     m      -   second dimension of a                                  
c                                                                       
c   output argument list:      (including work arrays)                  
c     a      -   4-deg resolution output array after filtering          
c                                                                       
c   subprograms called:                                                 
c     unique:    - tukeyf                                               
c                                                                       
c attributes:                                                           
c   language: fortran 77 
c   machine:  cray                                                       
c                                                                       
c$$$                                                                    
      dimension a(n,m)                                                  
      dimension  dy(200),sdy(200),y(200),                               
     1 y1(200),y2(200),y3(200),sy(200)                                  
      np = n+1                                                          
      mp = m+1                                                          
      nm4 = n-4                                                         
      mm4 = m-4                                                         
      do 150 lat=1,m                                                    
      do 100 i=1,n                                                      
      y(i) = a(i,lat)                                                   
  100 continue                                                          
      call tukeyf (y,y1,y2,y3,sy,n)                                     
      do 120 i=1,n                                                      
      dy(i) = y(i) - sy(i)                                              
  120 continue                                                          
      do 130 i=1,4                                                      
      dy(i) = 0.                                                        
      dy(np-i) = 0.                                                     
  130 continue                                                          
      call tukeyf (dy,y1,y2,y3,sdy,n)                                   
      do 140 i=5,nm4                                                    
      a(i,lat) = sy(i) +  sdy(i)                                        
  140 continue                                                          
  150 continue                                                          
      do 155 long=5,nm4                                                 
      do 105 i=1,m                                                      
      y(i) = a(long,i)                                                  
  105 continue                                                          
      call tukeyf (y,y1,y2,y3,sy,m)                                     
      do 125 i=1,m                                                      
      dy(i) = y(i) - sy(i)                                              
  125 continue                                                          
      do 135 i=1,4                                                      
      dy(i) = 0.                                                        
      dy(mp-i) = 0.                                                     
  135 continue                                                          
      call tukeyf (dy,y1,y2,y3,sdy,m)                                   
      do 145 i=5,mm4                                                    
      a(long,i) = sy(i) +  sdy(i)                                       
  145 continue                                                          
  155 continue                                                          
      return                                                            
      end                                                               
      subroutine tukeyf(y,y1,y2,y3,sy,n)                                
c$$$  subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    tukeyf      performs medium filter required by tukey   
c   prgmmr: r. w. reynolds    org: w/np24      date: 1991-07-22           
c                                                                       
c abstract: computes multiple pass median filtering of array y.         
c   called by subroutine tukey.                                         
c   see appendix b of reynolds (1988, j. of climate) for details.       
c                                                                       
c program history log:                                                  
c   1991-07-22  richard w. reynolds                                       
c                                                                       
c usage:    call tukeyf(y,y1,y2,y3,sy,n)                                
c                                                                       
c   input argument list:                                                
c     y      -   array to be filtered                                   
c     n      -   size of y and output arrays                            
c                                                                       
c   output argument list:      (including work arrays)                  
c     y1      -   array of first pass of filter                         
c     y2      -   array of second of filter                             
c     y1      -   array of third of filter                              
c     y1      -   array of fourth pass of filter                        
c                                                                       
c attributes:                                                           
c   language: fortran 77 
c   machine:  cray                                                       
c                                                                       
c$$$                                                                    
      dimension y(n),y1(n),y2(n),y3(n),sy(n)                            
c                                                                       
c     functions                                                         
c                                                                       
      amed2(xx,yy) = (xx + yy)*.5                                       
      amed3(xx,yy,zz) = (xx + yy + zz                                   
     1 - amax1(xx,yy,zz) - amin1(xx,yy,zz))                             
      amed4(xx,yy,zz,ww) = (xx + yy + zz + ww                           
     1 - amax1(xx,yy,zz,ww) - amin1(xx,yy,zz,ww))*.5                    
c                                                                       
      do 5 i=1,4                                                        
      y1(i)=0.0                                                         
      y2(i)=0.0                                                         
      y3(i)=0.0                                                         
      sy(i)=0.0                                                         
      in=n-i+1                                                          
      y1(in)=0.0                                                        
      y2(in)=0.0                                                        
      y3(in)=0.0                                                        
    5 sy(in)=0.0                                                        
c                                                                       
      n3=n-3                                                            
      do 10 i=1,n3                                                      
   10 y1(i+3/2)=amed4(y(i),y(i+1),y(i+2),y(i+3))                        
c                                                                       
      n4=n-4                                                            
      do 20 i=2,n4                                                      
   20 y2(i+3/2)=amed3(y1(i+1/2),y1(i+3/2),y1(i+5/2))                    
c                                                                       
      do 30 i=3,n4                                                      
   30 y3(i+1)=amed2(y2(i+1/2),y2(i+3/2))                                
c                                                                       
      n5=n-5                                                            
      do 40 i=4,n5                                                      
   40 sy(i+1)=amed3(y3(i),y3(i+1),y3(i+2))                              
      return                                                            
      end                                                               
      subroutine anal4 (sst4,nsst4,wkw,wkt,n,m,n10,m10)                 
c$$$  subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    anal4      analyzes sst on 4-deg grid                  
c   prgmmr: r. w. reynolds    org: w/np24      date: 1991-07-22           
c                                                                       
c abstract: takes super observations of sst of 4-degree grid and        
c           fills in internal values by interpolation and fills in      
c           external values by extrapolation.  the interpolation is     
c           done by subroutine weaver, the extrapolation by subroutine  
c           extrap.  the filled in array is then smoothed by the        
c           median filter of subroutine tukey.                          
c                                                                       
c program history log:                                                  
c   1991-07-22  richard w. reynolds                                       
c                                                                       
c usage:    call anal4 (sst4,nsst4,wkw,wkt,n,m,n10,m10)                 
c                                                                       
c   input argument list:                                                
c     sst4   -   4-deg resolution sst superobs in deg c                 
c            -   to be filled and smoothed                              
c     nsst4  -   4-deg resolution number of obs used in sst4            
c     wkw    -   4-deg resolution work array                            
c     wkt    -   4-deg resolution work array                            
c            -   this array is bigger than the other 4-deg fields       
c            -   to avoid end effects of median filter (subr tukey)     
c     n      -   first dimension of sst4, nsst4 and wkw                 
c                (90 in calling routine)                                
c     m      -   second dimension of sst4, nsst4 and wkw                
c                (45 in calling routine)                                
c     n10    -   first dimension of wkt                                 
c                (100 in calling routine)                               
c     m10    -   first dimension of wkt                                 
c                (55 in calling routine)                                
c                                                                       
c   output argument list:      (including work arrays)                  
c     sst4   -   4-deg resolution sst analyzed field in deg c           
c            -   filled and smoothed                                    
c                                                                       
c remarks:  array indices correspond to locations as follows:           
c                                                                       
c     for sst4 and nsst4 and wkw                                        
c                                                                       
c   long:   1,  21,  41,  61,  81, 101, 121, 141, 161,-179,             
c   index:  1,   6,  11,  16,  21,  26,  31,  36,  41,  46,             
c                                                                       
c   long:   -159,-139,-119, -99, -79, -59, -39, -19,  -3.               
c   index:    51,  56,  61,  66,  71,  76,  81,  86,  90.               
c                                                                       
c   lat:   -87,-67,-47,-27, -7, 13, 33, 53, 73, 89.                     
c   index:   1,  6, 11, 16, 21, 26, 31, 35, 41, 45.                     
c                                                                       
c attributes:                                                           
c   language: fortran 77 
c   machine:  cray                                                       
c                                                                       
c$$$                                                                    
      dimension sst4(n,m),nsst4(n,m),wkt(n10,m10),wkw(n,m)              
      dimension val(500)                                                
      n5 = n+5                                                          
      m5 = m+5                                                          
c     print 5,n,n5,n10,m,m5,m10                                         
c   5 format (/'n =',i3,3x,'n5 =',i3,3x,'n10 =',i3,3x,'m =',i3,3x,      
c    1        'm5 =',i3,3x,'m10 =',i3)                                  
      do 10 i=1,n                                                       
      do 10 j=1,m                                                       
      wkw(i,j) = 0.                                                     
      if (nsst4(i,j).gt.1) wkw(i,j) = 1.                                
   10 continue                                                          
      call weaver (val,0,sst4,wkw,n,m,1.,357.,-87.,89.,                 
     1 5.,0,0,0,2,0,-1.,ichk)                                           
c     if (ichk.ne.0) call exit (1004)
      call extrap (sst4,ichk,n,n,m)                                     
c     if (ichk.ne.0) call exit (1005)
      do 20 i=6,n5                                                      
      im = i - 5                                                        
      do 20 j=6,m5                                                      
      jm = j - 5                                                        
      wkt(i,j) = sst4(im,jm)                                            
   20 continue                                                          
      do 30 i=1,5                                                       
      do 30 j=1,m10                                                     
      wkt(i,j) = wkt(i+n,j)                                             
      wkt(i+n5,j) = wkt(i+5,j)                                          
   30 continue                                                          
      do 40 i=1,n10                                                     
      do 40 j=1,5                                                       
      wkt(i,j) = wkt(i,6)                                               
      wkt(i,j+m5) = wkt(i,m5)                                           
   40 continue                                                          
      call tukey (wkt,n10,m10)                                          
      do 50 i=6,n5                                                      
      im = i - 5                                                        
      do 50 j=6,m5                                                      
      jm = j - 5                                                        
      sst4(im,jm) = wkt(i,j)                                            
   50 continue                                                          
      return                                                            
      end                                                               
      subroutine expand (sst4,cx,cy,fx,fy,sstoi)                        
c$$$  subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    expand      interpolates 4-deg to oi 1-deg grid        
c   prgmmr: r. w. reynolds    org: w/np24      date: 1991-07-22           
c                                                                       
c abstract: interpolates 4-deg analyzed field or bias correction        
c   field to 1-deg field defined by oi grid.                            
c                                                                       
c program history log:                                                  
c   1991-07-22  richard w. reynolds                                       
c                                                                       
c usage:    call expand (sst4,cx,cy,fx,fy,sstoi)                        
c                                                                       
c   input argument list:                                                
c     sst4   -   4-deg resolution analyzed sst field or analyzed        
c                bias correction field in deg c                         
c     cx       - distribution of grid points in sst4 (first index)      
c     cy       - distribution of grid points in sst4 (second index)     
c     fx       - distribution of grid points in sstoi (first index)     
c     fy       - distribution of grid points in sstoi (second index)    
c                                                                       
c   output argument list:      (including work arrays)                  
c     sstoi    -   1-deg resolution sst field in deg c                  
c              -   location of center of grid given by                  
c              -   xt (first index) and yt (second index)               
c              -   which are defined by subroutine gengrd               
c                                                                       
c attributes:                                                           
c   language: fortran 77 
c   machine:  cray                                                       
c                                                                       
c$$$                                                                    
c     for sst4b                                                         
c                                                                       
c   long:   -3,   1,  21,  41,  61,  81, 101, 121, 141, 161,-179,       
c   index:   1,   2,   7,  12,  17,  22,  27,  32,  37,  42,  47,       
c                                                                       
c   long:   -159,-139,-119, -99, -79, -59, -39, -19,  -3,   1.          
c   index:    52,  57,  62,  67,  72,  77,  82,  87,  88,  92.          
c                                                                       
c   lat:   -91,-87,-67,-47,-27, -7, 13, 33, 53, 73, 89, 93.             
c   index:   1,  2,  7, 12, 17, 22, 27, 32, 37, 42, 46, 47.             
c                                                                       
      dimension sst4(90,45),sst4b(92,47)                                
      dimension sstoi(380,180),sst1(360,180)                            
      dimension fx(360),fy(180),cx(92),cy(47)                           

c dcs - copied from main 
*       print*
*       print*,' SST4 INSIDE EXPAND'
*       print '(20f5.1)',sst4
*       print*
*     print 283,cx                                                      
  283 format (' cx =',12f10.1)                                          
*     print 284,cy                                                      
  284 format (' cy =',12f10.1)                                          
c dcs

      do 10 i=1,90                                                      
      ip = i + 1                                                        
      do 10 j=1,45                                                      
      jp = j + 1                                                        
      sst4b(ip,jp) = sst4(i,j)                                          
   10 continue                                                          
      do 20 i=2,91                                                      
      sst4b(i,1) = sst4b(i,2)                                           
      sst4b(i,47) = sst4b(i,46)                                         
   20 continue                                                          
      do 25 j=1,47                                                      
      sst4b(1,j) = sst4b(91,j)                                          
      sst4b(92,j) = sst4b(2,j)                                          
   25 continue                                                          
      call intrp (sst4b,92,92,47,sst1,360,360,180,cx,cy,fx,fy,ierr)     
c     if (ierr.ne.0) go to 1600                                         
      do 30 i=1,360                                                     
      ip = i + 10                                                       
      ix = i + 180                                                      
      if (ix.gt.360) ix = ix - 360                                      
      do 30 j=1,180                                                     
      sstoi(ip,j) = sst1(ix,j)                                          
   30 continue                                                          
      do 40 j=1,180                                                     
      do 40 i=1,10                                                      
      sstoi(i,j) = sstoi(i+360,j)                                       
      sstoi(i+370,j) = sstoi(i+10,j)                                    
   40 continue                                                          
      return                                                            
c1600 print 1601                                                        
c1601 format('interpolation error intrp: exit 15')                     
c     call exit (15)
      end                                                               
      subroutine extrap (a,ichk,idim,ni,nj)                             
c$$$  subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    extrap      extrapolates 4-deg value missed by weaver  
c   prgmmr: r. w. reynolds    org: w/np24      date: 1991-07-22           
c                                                                       
c abstract: the field in subroutine anal4 is filled in by weaver.       
c   any values which are not defined by weaver (extrapolated values)    
c   are set to 9999.  these values are filled in by extrapolation       
c   here.  array a is input and output.                                 
c                                                                       
c program history log:                                                  
c   1991-07-22  richard w. reynolds                                       
c                                                                       
c usage:    call extrap (a,ichk,idim,ni,nj)                             
c                                                                       
c   input argument list:                                                
c     a        - 4-deg sst field with possible values not filled by     
c              - subroutine weaver (in deg c)                           
c     idim     - actual first dimension of a in calling routine         
c     ni       - first dimension of a                                   
c     nj       - second dimension of a                                  
c                                                                       
c   output argument list:      (including work arrays)                  
c     a        - 4-deg sst field with extrapolated values               
c     ichk     = 1 if a is all missing                                  
c              = 0 if field filled in ok.                               
c                                                                       
c attributes:                                                           
c   language: fortran 77 
c   machine:  cray                                                       
c                                                                       
c$$$                                                                    
      dimension a(idim,nj)                                              
      ichk = 1                                                          
      nip = ni + 1                                                      
      njp = nj + 1                                                      
c  extrapolate empty lower rows                                         
      do 100 j=1,nj                                                     
      l = j                                                             
      do 100 i=1,ni                                                     
      if (a(i,j).lt.999.) go to 110                                     
  100 continue                                                          
      return                                                            
  110 lm = l-1                                                          
      if (lm.lt.1) go to 120                                            
      do 130 i=1,ni                                                     
      do 130 j=1,lm                                                     
      a(i,j) = a(i,l)                                                   
  130 continue                                                          
c   extrapolate empty upper rows                                        
  120 do 140 jj=1,nj                                                    
      j = njp - jj                                                      
      l = j                                                             
      do 140 i=1,ni                                                     
      if (a(i,j).lt.999.) go to 150                                     
  140 continue                                                          
      return                                                            
  150 lp = l+1                                                          
      if (lp.gt.nj) go to 170                                           
      do 160 i=1,ni                                                     
      do 160 j=lp,nj                                                    
      a(i,j) = a(i,l)                                                   
  160 continue                                                          
  170 do 10 j=1,nj                                                      
c     left extrapolation                                                
      do 20 i=1,ni                                                      
      l = i                                                             
      if (a(i,j).lt.999.) go to 30                                      
   20 continue                                                          
      return                                                            
   30 lm = l-1                                                          
      if (lm.lt.1) go to 15                                             
      do 40 i=1,lm                                                      
      a(i,j) = a(l,j)                                                   
   40 continue                                                          
c    right extrapolation                                                
   15 do 25 ii=1,ni                                                     
      i = nip - ii                                                      
      l = i                                                             
      if (a(i,j).lt.999.) go to 35                                      
   25 continue                                                          
      return                                                            
   35 lp = l+1                                                          
      if (lp.gt.ni) go to 10                                            
      do 45 i=lp,ni                                                     
      a(i,j) = a(l,j)                                                   
   45 continue                                                          
   10 continue                                                          
      ichk = 0                                                          
      return                                                            
      end                                                               
      subroutine intrp (cn,imc,ic,jc,fn,imf,if,jf,cx,cy,fx,fy,ierr)     
c$$$  subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    intrp      finds interpolated field fn                 
c   prgmmr: r. w. reynolds   org: w/np24     date: 1990-12-31             
c                                                                       
c abstract: finds a linearly interpolated field fn defined by           
c     array locations fx and fy within input field cn that is           
c     defined by array locations cx and cy. fx and cx are locations     
c     of first index of fn and cn, respectively, fy and cy are          
c     locations of the second index.                                    
c                                                                       
c program history log:                                                  
c   1990-12-31  richard w. reynolds                                       
c                                                                       
c usage:    call intrp (cn,imc,ic,jc,fn,imf,if,jf,cx,cy,fx,fy,ierr)     
c                                                                       
c   input argument list:                                                
c     cn       - gridded field                                          
c              - at locations defined by cx and cy                      
c     imc      - actual first dimension of cn in calling routine        
c     ic       - first dimension of cn                                  
c     jc       - second dimension of cn                                 
c     cx       - distribution of grid points in cn (first index)        
c     cy       - distribution of grid points in cn (second index)       
c     imf      - actual first dimension of fn in calling routine        
c     if       - first dimension of fn                                  
c     jf       - second dimension of fn                                 
c     fx       - distribution of grid points in fn (first index)        
c     fy       - distribution of grid points in fn (second index)       
c                                                                       
c   output argument list:      (including work arrays)                  
c     fn       - linear interpolated field                              
c              - at locations defined by fx and fy                      
c     ierr     - ierr = 0: interpolated field ok                        
c              - ierr = 1, fn = 0:    no interpolation possible         
c              - because cx, cy, fx, or fy do not monitonically         
c              - increases or fx and fy arrays not within cx and cy     
c              - arrays respectively.                                   
c              - if ierr=1, error messages are printed                  
c                                                                       
c   output files:                                                       
c     fort.06  - printout                                               
c                                                                       
c remarks:  subrotuine orignally written by ron pacanowski, gfdl/noaa   
c                                                                       
c attributes:                                                           
c   language: fortran 77                                                
c   machine:  cray                                                       
c                                                                       
c$$$                                                                    
      dimension cn(imc,jc),cx(ic),cy(jc),fn(imf,jf),fx(if),fy(jf)       
      ierr = 0                                                          
c dcs  copied from main
*     print 283,cx                                                      
  283 format (' cx =',12f10.1)                                          
*     print 284,cy                                                      
  284 format (' cy =',12f10.1)                                          
c dcs
      if(cx(1).le.fx(1).and.cx(ic).ge.fx(if).and.cy(1).le.fy(1).and.    
     1 cy(jc).ge.fy(jf)) go to 10                                       
      write (6,9999)                                                    
 9999 format (1x,'output grid exceeds input grid ')                     
      go to 20                                                          
   10 do 12 i=2,ic                                                      
      if(cx(i).ge.cx(i-1)) go to 12                                     
      write (6,9998)                                                    
 9998 format (1x,'cx,cy,fx, or fy not monotonic')                       
      go to 20                                                          
   12 continue                                                          
      do 14 j=2,jc                                                      
      if (cy(j).ge.cy(j-1)) go to 14                                    
      write (6,9998)                                                    
      go to 20                                                          
   14 continue                                                          
      do 16 i=2,if                                                      
      if (fx(i).ge.fx(i-1)) go to 16                                    
      write (6,9998)                                                    
      go to 20                                                          
   16 continue                                                          
      do 18 j=2,jf                                                      
      if (fy(j).ge.fy(j-1)) go to 18                                    
      write (6,9998)                                                    
      go to 20                                                          
   18 continue                                                          
      go to 30                                                          
   20 write (6,9994)                                                    
 9994 format (1x,'cx,cy,fx,fy follow')                                  
      write (6,9993) (cx(i),i=1,ic)                                     
 9993 format (1x,10e13.6)                                               
      write (6,9993) (cy(j),j=1,jc)                                     
      write (6,9993) (fx(i),i=1,if)                                     
      write (6,9993) (fy(j),j=1,jf)                                     
      print 2222                                                        
 2222 format ('warning interpolation error in intrp')                  
      ierr = 1                                                          
      return                                                            
   30 continue                                                          
      do 100 j=1,jf                                                     
      do 110 l=1,jc                                                     
      if(cy(l).lt.fy(j)) go to 110                                      
      jn=l-1                                                            
      js=l                                                              
      if(jn.eq.0) js=2                                                  
      if(jn.eq.0) jn=1                                                  
      go to 111                                                         
  110 continue                                                          
  111 continue                                                          
      do 100 i=1,if                                                     
      do 120 l=1,ic                                                     
      if(cx(l).lt.fx(i)) go to 120                                      
      ie=l                                                              
      iw=l-1                                                            
      if(iw.eq.0) ie=2                                                  
      if(iw.eq.0) iw=1                                                  
      go to 121                                                         
  120 continue                                                          
  121 continue                                                          
      a=fx(i)-cx(iw)                                                    
      b=cx(ie)-fx(i)                                                    
      tx=cx(ie)-cx(iw)                                                  
      c=fy(j)-cy(jn)                                                    
      d=cy(js)-fy(j)                                                    
      ty=cy(js)-cy(jn)                                                  
  100 fn(i,j)=(c*(a*cn(ie,js)+b*cn(iw,js))+d*(a*cn(ie,jn)+b*cn(iw,jn))) 
     1 /(tx*ty)                                                         
      return                                                            
      end                                                               
*     subroutine biascr (bias,iu,iw,title,ls,xt,yt,cosyt)               
      subroutine biascr (bias,iu,iw,title,wt,xt,yt)               
c$$$  subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    biascr      corrects day and night sat biases          
c   prgmmr: r. w. reynolds    org: w/np24      date: 1991-07-22           
c                                                                       
c abstract: interpolates the 1 degree resolution satellite bias to the  
c   location of the satellite data and adds the interpolated data to    
c   the data.  day and night time satellite fields are processed        
c   separately.                                                         
c                                                                       
c program history log:                                                  
c   1991-07-22  richard w. reynolds                                       
c   1997-05-15  diane c. stokes 
c                 - commented out cosine weighting for bias correction  
c                 - land sea mask changed to weights (0,.25,.5,.75,1)
c                                                                       
c*usage:    call biascr (bias,iu,iw,title,ls,xt,yt,cosyt)               
c usage:    call biascr (bias,iu,iw,title,wt,xt,yt)               
c                                                                       
c   input argument list:                                                
c     bias   -   1-deg resolution bias correction field                 
c            -   uses oi grid defined by xt and yt arrays in            
c            -   subroutine gengrd                                      
c     iu     -   input unit of uncorrected satellite data               
c     iw     -   output unit of corrected satellite data                
c     title  -   character*10 label of type of satellite field          
c            -   being processed (i.e. day or night)                    
c*    ls     -   land/sea tag on oi grid               ! changed to wt  
c*               ls = 1 for ocean and ls = 0 for land  ! 05/15/1997     
c     wt     -   land/sea weight on oi grid                                
c                wt = 1. for ocean and wt = 0. for land 
c                (or 0.75, 0.5, 0.25) 
c     xt       - longitudes on analysis grid                            
c              - betw -189.5 (189.5w) to 189.5 (189.5e)                 
c     yt       - latitudes on analysis grid                             
c              - betw -89.5 (89.5s) to 89.5 (89.5n)                     
c*    cosyt    - cosine(yt+0.5), cosine is offset for subr intrp1       
c*                REMOVED 05/15/1997
c                                                                       
c*remarks:  cosine is offset for subroutine intrp1, 
c*          thus if index is i, cosine of index i                       
c*          is for position midway between yt(i) and yt(i+1)            
c*            NOT APPLICABLE 05/15/1997
c                                                                       
c usage:                                                                
c   input files:                                                        
c     fort.14  -  daytime satellite super observations: &&nsatgrid      
c              -  defined by input variable iu                          
c     fort.15  -  nighttime satellite super observations: &&nsatgrid    
c              -  defined by input variable iu                          
c                                                                       
c   output files:  (including scratch files)                            
c     fort.06  -  printout                                              
c     fort.51  -  daytime satellite super observations: &&dsatcorr      
c              -  bias corrected                                        
c              -  defined by output variable iw                         
c     fort.52  -  nighttime satellite super observations: &&nsatcorr    
c              -  bias corrected                                        
c              -  defined by output variable iw                         
c                                                                       
c   subprograms called:                                                 
c     unique:    - intrp1                                               
c                                                                       
c attributes:                                                           
c   language: fortran 77 
c   machine:  cray                                                       
c                                                                       
c$$$                                                                    
      dimension ictr(2000),rsst(2000),rlon(2000),rlat(2000)             
      dimension rstd(2000)                                              
      character*8 type                                                  
      character*1 cc                                                    
      character*10 title                                                
*     dimension yt(180),xt(380),bias(380,180),ls(380,180),cosyt(180)    
      dimension yt(180),xt(380),bias(380,180),wt(380,180)
      data cc /'c'/                                                     
      rewind iu                                                         
      print 510,iu,iw                                                   
  510 format (/'input sat data unit =',i3,                              
     1 3x,'output sat data unit =',i3)                                  
      read (iu,end=1000) iyrst,imst,idst,iyrend,imend,idend,ndays       
      if(iyrst.lt.100)then
        if(iyrst.lt.21)then
          iyrst=iyrst+2000
        else
          iyrst=iyrst+1900
        endif
        print*,'IYRST MODIFIED TO 4-DIGITS:',iyrst
      endif
      if(iyrend.lt.100)then
        if(iyrend.lt.21)then
          iyrend=iyrend+2000
        else
          iyrend=iyrend+1900
        endif
        print*,'IYREND MODIFIED TO 4-DIGITS:',iyrend
      endif

      print 260,title,iyrst,imst,idst,iyrend,imend,idend,ndays          
  260 format (/,a10,3x, 'start day = ',i4,2i3,3x,' end day = ',i4,2i3,
     1  3x,' num days = ',i3)   
      write (iw)         iyrst,imst,idst,iyrend,imend,idend,ndays       
   50 read (iu,end=1000) type,ict                                       
      type(8:8) = cc                                                    
      write (iw) type,ict                                               
c     print 40,type,ict                                                 
c  40 format (/'read',a10,3x,'numb super obs =',i10)                    
      if (ict.eq.0) go to 550                                           
      amin = +1.30                                                     
      amax = -amin                                                      
      read (iu,end=1000) (ictr(ii),ii=1,ict)                            
      read (iu,end=1000) (rsst(ii),ii=1,ict)                            
      read (iu,end=1000) (rlat(ii),ii=1,ict)                            
      read (iu,end=1000) (rlon(ii),ii=1,ict)                            
*     read (iu,end=1000) (rstd(ii),ii=1,ict)                            
      read (iu,end=1000) 
      do 30 l=1,ict                                                     
      sst = rsst(l)                                                     
      xlat = rlat(l)                                                    
      xlon = rlon(l)                                                    
      if (xlon.gt.180.) xlon = xlon - 360.                              
*     call intrp1 (bias,380,380,180,xt,yt,cosyt,ls,xlon,xlat,corr,ierr) 
      call intrp1 (bias,380,380,180,xt,yt,wt,xlon,xlat,corr,ierr) 
c ierr not equal to 0: no interpolated value possible                   
      if (ierr.eq.0) then                                               
      rsst(l) = sst + corr                                              
      if (rsst(l).lt.-2.) rsst(l) = -2.                                 
      endif                                                             
      if (amax.lt.corr) amax = corr                                     
      if (amin.gt.corr) amin = corr                                     
   30 continue                                                          
      write (iw) (ictr(ii),ii=1,ict)                                    
      write (iw) (rsst(ii),ii=1,ict)                                    
      write (iw) (rlat(ii),ii=1,ict)                                    
      write (iw) (rlon(ii),ii=1,ict)                                    
*     write (iw) (rstd(ii),ii=1,ict)                                    
      write (iw) 
c     print 730,amax,amin                                               
c 730 format (/'sat bias corrections:  max =',f8.2,3x,'min =',f8.2)     
      go to 50                                                          
  550 return                                                            
 1000 print 1001                                                        
 1001 format ('premature end of satellite input data: exit 3')         
      call exit (3)
      end                                                               
*     subroutine intrp1 (cn,imc,ic,jc,cx,cy,cosy,ls,fx,fy,fn,ierr)      
      subroutine intrp1 (cn,imc,ic,jc,cx,cy,wt,fx,fy,fn,ierr)      
c$$$  subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    intrp1      finds interpolated value                   
c   prgmmr: r. w. reynolds   org: w/np24     date: 1990-12-14             
c                                                                       
c abstract: finds a linearly interpolated value fn at location          
c   fx (longitude) and fy (latitude) relative to                        
c   an analyzed field cn that is                                        
c   defined by cx (longitude) and cy (latitude).                        
c*  the interpolation includes an adjustment for the          REMOVED  
c*  convergence of longitude lines as a function of latitude. 05/15/1997
c*  land values (ls=0) are not used in the interpolation.               
c   land values (wt=0) are not used in the interpolation.               
c   note: the location (fx,fy) must lie within the grid                 
c   defined by cx and cy.                                               
c                                                                       
c program history log:                                                  
c   1990-12-14  richard w. reynolds                                       
c                                                                       
c*usage:    call intrp1 (cn,imc,ic,jc,cx,cy,cosy,ls,fx,fy,fn,ierr)      
c usage:    call intrp1 (cn,imc,ic,jc,cx,cy,wt,fx,fy,fn,ierr)      
c                                                                       
c   input argument list:                                                
c     cn       - gridded field                                          
c     imc      - actual 1st dimension of cn in calling routine          
c     ic       - first dimension of cn                                  
c     jc       - second dimension of cn                                 
c     cx       - distribution of longitude grid points in cn            
c     cy       - distribution of latitude grid points in cn             
c*    cosy     - cosine array, centered between grid points             
c*             - cosy(j) = cosine(.5* (lat(j)+lat(j+1) )                
c*    ls       - land sea tag: (=1 over water and =0 over land)         
c     wt     -   land sea weight on oi grid                                
c                wt = 1. for ocean and wt = 0. for land 
c                (or 0.75, 0.5, 0.25) 
c     fx       - longitude of fn relative to cx coordinates             
c     fy       - latitude of fn relative to cy coordinates              
c                                                                       
c   output argument list:                                               
c      fn       - linear interpolated value from cn array               
c               - at location fx and fy                                 
c      ierr     - ierr = 0: interpolated value ok                       
c               - ierr = 1, fn = 0:    no interpolation                 
c               -   because all grid points are over land,              
c               -   or cx(1) is less than or equal to fx,               
c               -   or cx(ic) is greater than or equal to fx,           
c               -   or cx(1) is less than or equal to fx,               
c               -   or cy(jc) is greater than or equal to fy,           
c               -   or cx or cy does not monotonically increase         
c                                                                       
c   output files:                                                       
c     fort.06  - printout                                               
c                                                                       
c remarks:  if ierr=1, cx and cy arrays and fx and fy are printed       
c           unless all grid points are over land                        
c                                                                       
c attributes:                                                           
c   language: fortran 77                                                
c   machine:  cray                                                       
c                                                                       
c$$$                                                                    
*     dimension cn(imc,jc),cx(ic),cy(jc),cosy(jc)                       
*     dimension ls(imc,jc)                                              
      dimension cn(imc,jc),cx(ic),cy(jc)
      dimension wt(imc,jc)                                              
      if (fx.lt.cx(1).or.fx.gt.cx(ic)) go to 10                         
      if (fy.lt.cy(1).or.fy.gt.cy(jc)) go to 10                         
      do 110 l=1,jc                                                     
      if (cy(l).lt.fy) go to 110                                        
      jn = l                                                            
      js = l-1                                                          
      if (js.eq.0) jn = 2                                               
      if (js.eq.0) js = 1                                               
      go to 111                                                         
  110 continue                                                          
      jn = jc                                                           
      js = jc - 1                                                       
  111 continue                                                          
      do 120 l=1,ic                                                     
      if (cx(l).lt.fx) go to 120                                        
      ie = l                                                            
      iw = l-1                                                          
      if (iw.eq.0) ie = 2                                               
      if (iw.eq.0) iw = 1                                               
      go to 121                                                         
  120 continue                                                          
      ie = ic                                                           
      iw = ic - 1                                                       
  121 continue                                                          
*     a = (fx - cx(iw)) * cosy(js)                                      
*     b = (cx(ie) - fx) * cosy(js)                                      
      a = (fx - cx(iw))
      b = (cx(ie) - fx)
      c = fy - cy(js)                                                   
      d = cy(jn) - fy                                                   
*     ac = a*c * float( ls(ie,jn) )                                     
*     bc = b*c * float( ls(iw,jn) )                                     
*     ad = a*d * float( ls(ie,js) )                                     
*     bd = b*d * float( ls(iw,js) )                                     
      ac = a*c * wt(ie,jn)
      bc = b*c * wt(iw,jn)
      ad = a*d * wt(ie,js)
      bd = b*d * wt(iw,js)
      top = ac*cn(ie,jn) + bc*cn(iw,jn) + ad*cn(ie,js) + bd*cn(iw,js)   
c changed back to original bot  12/7/1999
      bot = ac + bc + ad + bd                                           
*     bot = a*c + b*c + a*d + b*d                                           
      ierr = 0                                                          
c put back check of bot  12/7/1999
      if (bot.eq.0.) go to 200                                          
      fn = top/bot                                                      
      return                                                            
c                                                                       
c     no interpolated value                                             
c                                                                       
   10 print 15                                                          
   15 format (/'problem in intrp1: fx or fy out of range;',             
     1 '   no interpolated value')                                      
      print 20,cx(1),cx(ic),fx                                          
   20 format (/'cx(1) =',f10.3,3x,'cx(ic) =',f10.3,3x,'fx =',f10.3)     
      print 25,cy(1),cy(jc),fy                                          
   25 format (/'cy(1) =',f10.3,3x,'cy(jc) =',f10.3,3x,'fy =',f10.3)     
  200 fn = 0.                                                           
      ierr = 1                                                          
      return                                                            
      end                                                               
*     subroutine gengrd (imx,imy,xt,yt,cosyt)                           
      subroutine gengrd (imx,imy,xt,yt)                           
c$$$  subprogram documentation block                                    
c                .      .    .                                       .  
c subprogram:    gengrd      generates lat and long of oi grid          
c   prgmmr: r. w. reynolds   org: w/np24     date: 1990-12-14             
c                                                                       
c abstract: generates the latitude and longitudes                       
c   for the grid center of the oi analysis grid.  it also generates     
c   the cosine of the latitude position                                 
c                                                                       
c program history log:                                                  
c   1990-12-14  richard w. reynolds                                       
c                                                                       
**c usage:    call gengrd (imx,imy,xt,yt,cosyt)                           
c usage:    call gengrd (imx,imy,xt,yt)                           
c                                                                       
c   input argument list:                                                
c     imx      - length of the longitude array                          
c              - (first index of analysis grid)                         
c     imy      - length of the latitude array                           
c              - (second index of analysis grid)                        
c                                                                       
c   output argument list:                                               
c     xt       - longitudes on analysis grid                            
c              - betw -189.5 (189.5w) to 189.5 (189.5e)                 
c     yt       - latitudes on analysis grid                             
c              - betw -89.5 (89.5s) to 89.5 (89.5n)                     
c     cosyt    - cosine(yt+0.5), cosine is offset for subr intrp1       
c*                REMOVED 10/24/1997
c                                                                       
c remarks:  cosine is offset for subroutine intrp1,                     
c           thus if index is i, cosine of index i                       
c           is for position midway between yt(i) and yt(i+1)            
c                                                                       
c attributes:                                                           
c   language: fortran 77                                                
c   machine:  cray                                                       
c                                                                       
c$$$                                                                    
*     dimension xt(imx),yt(imy),cosyt(imy)                              
      dimension xt(imx),yt(imy)
      rad = 3.14159/180.                                                
c - do the longitudes                                                   
      do 50 i = 1,imx                                                   
      xt(i) = float(i) - 190.5                                          
   50 continue                                                          
c - do the latitudes                                                    
      do 100 j = 1,imy                                                  
      yt(j) = float(j) - 90.5                                           
      yy = yt(j) + 0.5                                                  
*     cosyt(j) = cos(rad*yy)                                            
  100 continue                                                          
      return                                                            
      end                                                               
