      PROGRAM OIST
! modified to input and output 360x180 grid (0.5E-359.5E)
!  Still processing as 380x180 for now to reduce number of modifications
!   and thus potential error
!   
C  $  MAIN PROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C MAIN PROGRAM:  OIST    COMPUTES 1-DEG SST OI ANALYSIS
C   PRGMMR: R.W. REYNOLDS    ORG: W/NMC52     DATE: 90-12-12
C
C ABSTRACT: READS SUPER OBSERVATIONS FROM BUOYS, SHIPS, ICE, DAY
C   SATELLITE AND NIGHT SATELLITE AND PREVIOUS ANALYSIS (FIRST
C   GUESS).  THE PROGRAM THEN COMPUTES A NEW ANALYSIS USING OPTIMUM
C   INTERPOLATION (OI) FOLLOWING THE TECHNIQUE OF LORENC (MONTHLY
C   WEATHER REVIEW, 1981).  THE PROGRAM OUTPUTS THE NEW SST OI
C   ANALYSIS ON A 1 DEG GRID AS WELL AS AN ERROR FIELD (RANGE 1 TO 0)
C   SHOWING THE RELATIVE IMPROVEMENT OF THE ANALYSIS OVER THE GUESS.
C   NOTE: SIGMA IMPLIES THE STANDARD DEVIATION.
C         ALL ERROR MEANS ARE ASSUMED TO BE ZERO.
C         OI    =  OPTIMUM INTERPOLATION.
C         SST   =  SEA SURFACE TEMPERATURE
C         OB(S) = OBSERVATION(S)
C
C PROGRAM HISTORY LOG:
C   90-12-12  RICHARD W. REYNOLDS
C
C USAGE:
C   INPUT FILES:
C     FT05F001 - NAMELISTS:  &CONTR1,&CONTR2,&SIGMA1,&SIGMA2
C              - NAMELISTS:  &RANGE1,&RANGE2,&BLOCK1,&BLOCK2
C
C              - DEFINITIONS OF VARIABLES FOLLOW:
C
C              - &CONTR1
C              - IFIL  : IF =1 SMOOTH GUESS FIELD WITH 1-2-1
C              -       : IF =0 DON'T SMOOTH
C              - ERR   : IF ERROR IS GREATER THAN OR EQUAL TO THIS
C              -       : SET ANALYSIS TO GUESS
C              - &END
C              - &CONTR2
C              - LBUOY : LOGICAL, IF TRUE USE BUOY DATA
C              -       : IF FALSE DON'T USE DATA
C              - LSHIP : LOGICAL, IF TRUE USE SHIP DATA
C              -       : IF FALSE DON'T USE DATA
C              - LSTDAY: LOGICAL, IF TRUE USE DAY SATELLITE DATA
C              -       : IF FALSE DON'T USE DATA
C              - LSTNTE: LOGICAL, IF TRUE USE NIGHT SATELLITE DATA
C              -       : IF FALSE DON'T USE DATA
C              - LICE  : LOGICAL, IF TRUE USE ICE DATA
C              -       : IF FALSE DON'T USE DATA
C              - &END
C              - &RANGE1
C              - GSLX  : GUESS LONGITUDE CORRELATION SCALE IN DEG
C              - GSLY  : GUESS LATITUDE  CORRELATION SCALE IN DEG
C              - DSLX  : DATA LONGITUDE CORRELATION SCALE IN DEG
C              - DSLY  : DATA LATITUDE  CORRELATION SCALE IN DEG
C              - &END
C              - &RANGE2
C              - INDEX : VERSION NUMBER: HAS NO EFFECT ON RESULT
C              - REJ   : REJECT LIMIT
C                      : ABSOLUTE SST INCREMENTS > REJ ARE REJECTED
C              - &END
C              - &BLOCK1
C              - XSCL  : LONGITUDE SIZE OF COMPUTATION BOX IN DEG
C              -       : CAN BE MULTIPLE OF 1-DEG (SEE LORENC)
C              -       : ALL BOX EDGES ON EVEN DEGREE
C              -       : DATA POINTS ON HALF DEGREE
C              - YSCL  : LATITUDE  SIZE OF COMPUTATION BOX IN DEG
C              - XBORD : X-BORDER TO INCLUDE DATA OUTSIDE XSCL (DEG)
C                      : TOTAL BOX IS XSCL + 2*XBORD
C              - YBORD : Y-BORDER TO INCLUDE DATA OUTSIDE YSCL (DEG)
C                      : TOTAL BOX IS YSCL + 2*YBORD
C              - &END
C              - &BLOCK2
C              - XMN   : LONGITUDE OF WESTWARD LIMIT OF WEST SIDE
C                      : OF COMPUTATIONAL BOX; EVEN DEGREES
C                      : (XMX-XMN)/XSCL MUST BE INTEGER
C              - XMX   : LONGITUDE OF EASTWARD LIMIT OF EAST SIDE
C                      : OF COMPUTATIONAL BOX
C              - YMN   : LATITUDE OF SOUTHWARD LIMIT OF SOUTH SIDE
C                      : OF COMPUTATIONAL BOX
C                      : (YMX-YMN)/YSCL MUST BE INTEGER
C              - YMX   : LATITUDE OF NORTHWARD LIMIT OF NORTH SIDE
C                      : OF COMPUTATIONAL BOX
C              - &END
C
C     FT09F001 - &&DSATGRID - SATELLITE DAY SUPER OBS   - TEMPORARY
C              - FROM PROGRAM SUPSATDY
C     FT10F001 - &&NSATGRID - SATELLITE NIGHT SUPER OBS - TEMPORARY
C              - FROM PROGRAM SUPSATDY
C     FT11F001 - &&BUOYGRID - SUPER OBS FOR BUOYS - TEMPORARY
C              - FROM PROGRAM SUPINSDY
C     FT12F001 - &&SHIPGRID - SUPER OBS FOR SHIPS - TEMPORARY
C              - FROM PROGRAM SUPINSDY
C     FT22F001 - &&ICEGRID  - SUPER OBS FROM ICE  - TEMPORARY
C              - FROM PROGRAM SUPINSDY
C
C     FT16F001 - OI GUESS FIELD; THIS IS PREVIOUS ANALYSIS AND HAS
C              - AN IDENTICAL FORMAT TO THE OI ANALYSIS FIELD
C              - WHICH IS OUTPUT ON FT03F001.
C              - BEFORE THE START OF THE OI ANALYSIS FILE IS COPIED
C              - ONTO THE OI GUESS FIELD
C
C     FT31F001 - LSTAGS.ONEDEG - 1-DEG LAND SEA TAGS: 3 IS LAND;
C              - 0 IS SEA
C     FT97F001 - TEMPORARY FILE FOR THIS JOB
C              - NEEDED FOR SUBROUTINE WEAVER
C              - WEAVER FILLS IN OI VALUES OVER LAND AFTER ANALYSIS
C
C     PARAMETERS:
C              - IMX IS LONGITUDE DIMENSION OF OI (-189.5 TO 189.5)
C              - IMX = 380                        (189.5W TO 189.5E)
C              - INCLUDES 10 DEG OVERLAPPING REGIONS ON EACH END
C              - IMY IS LATITUDE DIMENSION OF SST (-89.5 TO 89.5)
C                IMY = 180                         (89.5S TO 89.5N)
C              - JREC IS MAXIMUM NUMBER OF TOTAL SUPER OBS
C              - OI PROGRAM CAN STORE: NOW 100000
C
C   OUTPUT FILES:
C     FT03F001 - OI ANALYSIS FIELD; CONSISTS OF DATES, OI SST AND
C              - OI ERROR: SEE SUBROUTINE PUTOI
C     FT06F001 - PRINTOUT
C     FT97F001 - SCRATCH FILE FOR SUBROUTINE WEAVER
C
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - GETDAT, GETFG,  SMOOTH, GENGRD, INTRP1, WRAP,
C              - SSTMAP, CKDIST, DECISN, COMBIN, DECOMP, ESTIM,
C              - PUTOI,  MXMN2,  MAXMIN, STAND
C     LIBRARY:
C       IMSL   -   L2CDS,  LFSDS (VERSION 10)
C       (IMS.COMMON.LOAD  - WEAVER)
C
C   EXIT STATES:
C     COND =   0 - SUCCESSFUL RUN
C          =  11 - NO FIRST GUESS
C          =  12 - NO DATA AVAILABLE
C          =  13 - ICT EXCEEDS JREC; NOT ENOUGH STORAGE FOR DATA
C          =  14 - CANNOT FILL LAND VALUES WITH WEAVER
C          =  15 - NO DATES FOUND IN BUOY FILE
C
C REMARKS:  THIS PROGRAM IS BASED ON AN EARLIER VERSION WRITTEN
C           BY MING JI W/NMC52; LARRY BREAKER W/NMC21 HELPED WITH
C           THE MODIFICATIONS.
C           THE ANALYSIS GRID IS 1-DEG CENTERED ON HALF-DEG.
C           THE FIRST INDEX GOES FROM -189.5(189.5W) TO +189.5(189.5E)
C           THE SECOND INDEX GOES FROM -89.5(89.5S) TO +89.5(89.5N)
C
C  NOTE  :  OI DATES DETERMINED FROM BUOY DATA FILE; VARIABLES ARE:
C        IYRST    - START YEAR (2 DIGITS) OF ANALYSIS
C        IMST     - START MONTH OF ANALYSIS
C        IDST     - START DAY OF ANALYSIS
C        IYREND   - END YEAR (2 DIGITS) OF ANALYSIS
C        IMEND    - END MONTH OF ANALYSIS
C        IDEND    - END DAY OF ANALYSIS
C        NDAYS    - NUMBER OF DAYS FOR ANALYSIS
C
C COMMON BLOCKS: SCALES, SCLSQR, DOMAIN
C              - ALL VARIABLES DEFINED IN NAMELIST
C
C OTHER IMPORTANT VARIBLES:
C     ICT      - NUMBER OF TOTAL OBSERVATIONS IN ARRAYS
C              - ISST,ISIGC,ISIGU,ILAT,ILON,ITYPE (INTEGER)
C              - AND D (REAL)
C     ISST     - ARRAY OF DATA SSTS (100*DEG C)
C     ISIGC    - ARRAY OF CORRELATED SIGMA ERRORS (100*DEG C)
C     ISIGC    - ARRAY OF UNCORRELATED SIGMA ERRORS (100*DEG C)
C     ILAT     - ARRAY OF DATA LATITUDES (100*DEG C)
C     ILON     - ARRAY OF DATA LONGITUDES (100*DEG C)
C     ITYPE    - ARRAY OF DATA TYPES
C              - ITYPE = 1 FOR NIGHTTIME SATELLITE
C              - ITYPE = 2 FOR DAYTIME SATELLITE
C              - ITYPE = 3 FOR BUOYS
C              - ITYPE = 4 FOR SHIPS
C              - ITYPE = 5 FOR SSTS SIMULATED FROM ICE COVER
C     D        - ARRAY OF DATA INCREMENTS (DEG C)
C     GSST     - FIRST GUESS ARRAY
C              - ON ANALYSIS GRID (-189.5 TO 189.5 AND -89.5 TO 89.5)
C     ANAL     - ARRAY OF ANALYSIS INCREMENTS (DEG C)
C              - ON ANALYSIS GRID
C     E        - ARRAY OF ERROR VALUES (RANGE 0 TO 1)
C              - ON ANALYSIS GRID
C     XT       - LONGITUDES ON ANALYSIS GRID
C              - BETW -189.5 (189.5W) TO 189.5 (189.5E)
C     YT       - LATITUDES ON ANALYSIS GRID
C              - BETW -89.5 (89.5S) TO 89.5 (89.5N)
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77 (NFORVCLG);
C             COMPILER OPTIONS (GOSTMT,NOSD,OPT(3))
C   MACHINE:  NAS
C
C  $
C
C  IMX is 380, but input and output use 360.
      PARAMETER (IMX=380,IMY=180,
*    1          JREC=100000)
     1          JREC=200000)  ! increased for Grumbine Ice
      DIMENSION amask(360,180),MASK(360,180),WEAV(1000)
      DIMENSION YT(IMY),XT(IMX),E(IMX,IMY),ANAL(IMX,IMY),
     1          D(JREC),GSST(IMX,IMY),
     2          WK(IMX,IMY),LS(IMX,IMY)
*     EQUIVALENCE (WK,MASK)
      INTEGER   ITYPE(JREC),ISST(JREC),ILAT(JREC),ILON(JREC)
      INTEGER   ISIGU(JREC),ISIGC(JREC)
      DIMENSION GSS(39,19),SDC(39,19),SDU(39,19),SNC(39,19),SNU(39,19),
     1 SPU(39,19),BYU(39,19),SEW(39,19),SNS(39,19),AIC(39,19),
     2 L10(39,19),Z10(39,19),ALAT10(19),ALON10(39)
      LOGICAL LBUOY,LSHIP,LSTDAY,LSTNTE,LICE
      character*1  cice(360,180)

C
      COMMON /DOMAIN/ XSCL,YSCL,XBORD,YBORD,XMN,XMX,YMN,YMX
C
      NAMELIST /CONTR1/ ERR,IFIL
      NAMELIST /CONTR2/ LBUOY,LSHIP,LSTDAY,LSTNTE,LICE
      NAMELIST /RANGE2/ INDEX,REJ
      NAMELIST /BLOCK1/ XSCL,YSCL,XBORD,YBORD
      NAMELIST /BLOCK2/ XMN,XMX,YMN,YMX
C
      open(1,form='formatted',status='old')
*     open(31,form='formatted',status='old')
      open(31,form='unformatted',status='old',access='direct',
     1  recl=360*180*4)

      open(16,form='unformatted',status='unknown')
      open(9,form='unformatted',status='unknown')
      open(10,form='unformatted',status='unknown')
      open(11,form='unformatted',status='unknown')
      open(12,form='unformatted',status='unknown')
      open(22,form='unformatted',status='unknown')
      open(27,form='unformatted',status='unknown')

      open(3,form='unformatted',status='unknown')

*     READ (31,31) ((MASK(I,J),I=1,360),J=1,180)
*  31 FORMAT (80I1)
      READ (31,rec=1) amask    ! dcs
      do j=1,180
        do i=1,360
          mask(i,j)=nint(amask(i,j))
        enddo
      enddo

C
C   MASK IS DEFINED CENTERED ON HALF DEGREE: (.5-359.5E,89.5S-89.5N)
C   MASK = 0 OVER OCEAN; MASK = 3 OVER LAND
C   CONVERT MASK TO LS ARRAY WHICH IS ON ANALYSIS GRID.
C     LS = 1 OVER OCEAN; AND LS = 0 OVER LAND
C
      DO 32 I=1,360
      IP = I + 10
      IX = I + 180
      IF (IX.GT.360) IX = IX - 360
      DO 32 J=1,180
      IVAL = 1
      IF (MASK(IX,J).eq.0) IVAL = 0
      LS(IP,J) = IVAL
   32 CONTINUE
      DO 33 J=1,IMY
      DO 33 I=1,10
      LS(I,J) = LS(I+360,J)
      LS(I+370,J) = LS(I+10,J)
   33 CONTINUE
C
C   READ INPUT NAMELISTS
C
      READ (*,CONTR1)
      READ (*,CONTR2)
      READ (*,RANGE2)
      READ (*,BLOCK1)
      READ (*,BLOCK2)
      READ (11,END=6000) IYRST,IMST,IDST,IYREND,IMEND,IDEND,NDAYS
      REWIND 11
      if(iyrst.lt.100)then
        if(iyrst.lt.21)then
          iyrst=iyrst+2000
        else
          iyrst=iyrst+1900
        endif
        print*,'IYRST MODIFIED TO 4-DIGITS:',iyrst
      endif
      if(iyrend.lt.100)then
        if(iyrend.lt.21)then
          iyrend=iyrend+2000
        else
          iyrend=iyrend+1900
        endif
        print*,'IYREND MODIFIED TO 4-DIGITS:',iyrend
      endif
c   get ice concentration field and check date is same
      IJDST=iw3jdn(iyrst,imst,idst)
      IJDmid=ijdst+ndays/2
      call w3fs26(IJDmid,iyrmid,immid,idmid,idwk,idyr)
      read (27,end=6005) icyr,icmo,icda
      read (27,end=6005) cice(:,180:1:-1)
      if(icyr.ne.iyrmid.or.icmo.ne.immid.or.icda.ne.idmid)go to 6010

      WRITE (*,CONTR1)
      WRITE (*,CONTR2)
      WRITE (*,RANGE2)
      WRITE (*,BLOCK1)
      WRITE (*,BLOCK2)
C
      CALL OISTAT (GSS,IERR)
      IF (IERR.EQ.1) GO TO 5000
      CALL OISTAT (SDC,IERR)
      IF (IERR.EQ.1) GO TO 5000
      CALL OISTAT (SDU,IERR)
      IF (IERR.EQ.1) GO TO 5000
      CALL OISTAT (SNC,IERR)
      IF (IERR.EQ.1) GO TO 5000
      CALL OISTAT (SNU,IERR)
      IF (IERR.EQ.1) GO TO 5000
      CALL OISTAT (SPU ,IERR)
      IF (IERR.EQ.1) GO TO 5000
      CALL OISTAT (BYU ,IERR)
      IF (IERR.EQ.1) GO TO 5000
      CALL OISTAT (SEW,IERR)
      IF (IERR.EQ.1) GO TO 5000
      CALL OISTAT (SNS,IERR)
      IF (IERR.EQ.1) GO TO 5000
      DO 10 I=1,39
      DO 10 J=1,19
      SDC(I,J) = SDC(I,J)/GSS(I,J)
      SDU(I,J) = SDU(I,J)/GSS(I,J)
      SNC(I,J) = SNC(I,J)/GSS(I,J)
      SNU(I,J) = SNU(I,J)/GSS(I,J)
C     SPU(I,J) = SPU(I,J)/GSS(I,J)
      SPU(I,J) = 3.9
C     BYU(I,J) = BYU(I,J)/GSS(I,J)
      BYU(I,J) = 1.5
      AIC(I,J) = 1.
      Z10(I,J) = 0.
      L10(I,J) = 1
   10 CONTINUE
      DO 15 I=1,39
      ALON10(I) = -190 + 10*(I-1)
   15 CONTINUE
      DO 16 J=1,19
      ALAT10(J) = -90 + 10*(J-1)
   16 CONTINUE
C     PRINT 17,ALON10
C  17 FORMAT (' ALON10 =',15F7.1)
C     PRINT 18,ALAT10
C  18 FORMAT (' ALAT10 =',15F7.1)
      ICT = 0
      ICTS = ICT
C - READING IN AND STACKING THE NIGHTTIME SATELLITE DATA
      IUN = 09
      ID = 1
C
      ICT = 0
      ICTS = ICT
C - READING IN AND STACKING THE NIGHTTIME SATELLITE DATA
      IUN = 09
      ID = 1
c      NCUT = 3
      NCUT = 5
      ICOR = 1
      IF (LSTNTE) CALL GETDAT (IUN,ICT,ISST,ISIGC,ISIGU,ILAT,ILON,
     1  iyrst,imst,idst,ITYPE,ID,SNC,SNU,NCUT,L10,ALAT10,ALON10,ICOR)
      JCT = ICT - ICTS
      ICTS = ICT
      PRINT 931,JCT,NCUT
  931 FORMAT (' NUMBER OF NIGHT SAT OBS  =',I10,3X,'WITH NCUT =',I3)
      IF (ICT.GE.JREC) GO TO 3000
C - READING IN AND STACKING THE DAYTIME SATELLITE DATA
      IUN = 10
      ID = 2
c      NCUT = 3
      NCUT = 5
      ICOR = 1
      IF (LSTDAY) CALL GETDAT (IUN,ICT,ISST,ISIGC,ISIGU,ILAT,ILON,
     1  iyrst,imst,idst,ITYPE,ID,SDC,SDU,NCUT,L10,ALAT10,ALON10,ICOR)
      JCT = ICT - ICTS
      ICTS = ICT
      PRINT 932,JCT,NCUT
  932 FORMAT (' NUMBER OF DAT SAT OBS  =',I10,3X,'WITH NCUT =',I3)
      IF (ICT.GE.JREC) GO TO 3000
C - READING IN AND STACKING THE BUOY DATA
      IUN = 11
      ID = 3
      NCUT = 1
      ICOR = 1
      IF (LBUOY) CALL GETDAT (IUN,ICT,ISST,ISIGC,ISIGU,ILAT,ILON,
     1  iyrst,imst,idst,ITYPE,ID,Z10,BYU,NCUT,L10,ALAT10,ALON10,ICOR)
      JCT = ICT - ICTS
      ICTS = ICT
      PRINT 933,JCT,NCUT
  933 FORMAT (' NUMBER OF BUOY OBS  =',I10,3X,'WITH NCUT =',I3)
      IF (ICT.GE.JREC) GO TO 3000
C - READING IN AND STACKING THE SHIP DATA
      IUN = 12
      ID = 4
      NCUT = 1
      ICOR = 1
       IF (LSHIP) CALL GETDAT (IUN,ICT,ISST,ISIGC,ISIGU,ILAT,ILON,
     1  iyrst,imst,idst,ITYPE,ID,Z10,SPU,NCUT,L10,ALAT10,ALON10,ICOR)
      JCT = ICT - ICTS
      ICTS = ICT
      PRINT 934,JCT,NCUT
  934 FORMAT (' NUMBER OF SHIP OBS  =',I10,3X,'WITH NCUT =',I3)
      IF (ICT.GE.JREC) GO TO 3000
C - READING IN AND STACKING THE ICE DATA(-1.8C)
      IUN = 22
      ID = 5
      NCUT = 1
      ICOR = 1
      IF (LICE) CALL GETDAT (IUN,ICT,ISST,ISIGC,ISIGU,ILAT,ILON,
     1  iyrst,imst,idst,ITYPE,ID,Z10,AIC,NCUT,L10,ALAT10,ALON10,ICOR)
      JCT = ICT - ICTS
      ICTS = ICT
      PRINT 936,JCT,NCUT
  936 FORMAT (' NUMBER OF ICE POINTS =',I10,3X,'WITH NCUT =',I3)
      IF (ICT.GE.JREC) GO TO 3000
      CALL MXMN2 (ISST,ICT,AMIN,AMAX,.01)
      PRINT *,' MIN,MAX IN TEMP:',AMIN,AMAX
      CALL MXMN2 (ISIGC,ICT,AMIN,AMAX,.01)
      PRINT *,' MIN,MAX IN SIG-COR:',AMIN,AMAX
      CALL MXMN2 (ISIGU,ICT,AMIN,AMAX,.01)
      PRINT *,' MIN,MAX IN SIG-UNCOR:',AMIN,AMAX
      CALL MXMN2 (ILON,ICT,AMIN,AMAX,.01)
      PRINT *,' MIN,MAX IN LON:',AMIN,AMAX
      CALL MXMN2 (ILAT,ICT,AMIN,AMAX,.01)
      PRINT *,' MIN,MAX IN LAT:',AMIN,AMAX
      CALL MXMN2 (ITYPE,ICT,AMIN,AMAX,1.0)
      PRINT *,' MIN,MAX IN ITYPE:',AMIN,AMAX
C
C - READING IN THE FIRST-GUESS FIELD(I.E.,LAST WEEK'S ANALYSIS)
C
      IUN = 16
      CALL GETFG (IUN,IMX,IMY,IERR,GSST)
      IF (IERR.NE.0) GO TO 1000
      IF (IFIL.EQ.1) CALL SMOOTH (GSST,WK,LS,IMX,IMX,IMY)
C
C - GENERATING THE 1 DEGREE GLOBAL GRID CENTERED ON 1/2-DEGREE NODES
C
      CALL GENGRD (IMX,IMY,XT,YT)
C     PRINT 111,XT
C 111 FORMAT (' XT      =',14F8.1)
C     PRINT 112,YT
C 112 FORMAT (' YT      =',14F8.1)
C
      IF (ICT.EQ.0) GO TO 2000
C
C  COMPUTE DATA INCREMENT "D" FOR EACH DATA POINT
C  IF DATA INCREMENT NOT USEABLE, "D" SET EQUAL TO -999.9
C
      KNT = 0
C      REJ = 5.
C
C  WHEN MAGNITUDE OF SST INCREMENT IS GREATER THAN REJ,
C  SST VALUES ARE REJECTED
C
      DO 20 K = 1,ICT
      D(K) = -999.9
      TMP1 = ILON(K)
      XL = 0.01*TMP1
      TMP2 = ILAT(K)
      YL = 0.01*TMP2
      IF (ABS(YL).GT.89.) GO TO 20
      IF (ABS(XL).GT.189.) GO TO 20
      CALL INTRP1 (GSST,IMX,IMX,IMY,XT,YT,LS,XL,YL,VL,IERR)
C
C  FINDS INTERPOLATED GUESS FIELD VALUE (VL)
C  AT THE DATA LOCATTION (XT,YT)
C  IERR NOT EQUAL TO 0: NO INTERPOLATED VALUE POSSIBLE
C
      IF (IERR.NE.0) GO TO 20
      TMP3 = ISST(K)
      SSST = 0.01*TMP3
      DF = SSST - VL
      IF (ABS(DF).GT.REJ) GO TO 20
      D(K) = DF
      KNT = KNT + 1
C     IF (ABS(YL).LT.1.AND.XL.LT.-170.) PRINT 21,KNT,XL,YL,VL,SSST,DF
C  21 FORMAT (' KNT =',I6,3X,'XL =',F8.3,3X,'YL =',F8.3,3X,
C    1 'VL =',F8.3,3X,'SSST =',F8.3,3X,'DF =',F8.3)
   20 CONTINUE
      CALL MAXMIN (D,ICT,ICT,1,AMIN,AMAX)
      CALL STAND (ICT,D,AMEAN,STDEV)
      PRINT 600,ICT,KNT,AMEAN,STDEV,AMIN,AMAX
  600 FORMAT (' NUMB TOTAL OBS =',I6,5X,
     1 'NUMB OBS FLAGGED OK =',I6,5X,'MEAN OF DATA INC =',F8.2,
     2 5X,'ST. DEV OF DATA INC =',F8.2/
     3 ' MIN DATA INC =',F8.2,5X,'MAX DATA INC =',F8.2)
      print*,'unformatted min and max data inc:',amin,amax
C
C  OBSERVATIONS IN ARRARYS ISST,ILAT,ILON,ISIGC,ISIGU,ITYPE AND D
C  NOW COVER THE REGION -180 TO +180.
C  SUBROUTINE WRAP ADDS ADDITIONAL DATA BY ADDING OR SUBTRACTING
C  360 DEGREES TO THE LONGITUDES SO THAT THE LONGITUDE RANGE EXTENDS
C  FROM -189 TO +189.
C
      CALL WRAP (ICT,ISST,D,ILAT,ILON,ISIGC,ISIGU,ITYPE)
C
C  FIND THE OI SOLUTION
C
      CALL SSTMAP (D,ILON,ILAT,ICT,ANAL,E,IMX,IMY,ISIGC,ISIGU,LS,
     1 XT,YT,ITYPE,SEW,SNS,L10,ALAT10,ALON10)
C
C
      CALL MAXMIN (ANAL,IMX,IMX,IMY,AMIN,AMAX)
      PRINT *,' MIN,MAX IN ANAL (INC ANALYSIS SST): ',AMIN,AMAX
C
C   CONVERT ANALYSIS INCREMENT IN ANAL TO MEAN FIELD BY ADDING GUESS
C   FILED, GSST.
C   FILL IN WK ARRAY FOR WEAVER (WK=1 FOR LAND AND 0 FOR OCEAN)
C   ALSO IF ERROR, E, IS NOT LESS THAN ERR SET ANAL INCREMENT TO 0
C   FILL IN WK ARRAY FOR WEAVER (WK=1 FOR LAND AND 0 FOR OCEAN)
C
      DO 35 J = 1,IMY
      DO 35 I = 1,IMX
      IF (E(I,J).GE.ERR) ANAL(I,J)=0.0
      IF (LS(I,J).EQ.0) THEN
      WK(I,J) = 0.
      ELSE
      WK(I,J) = 1.
      ANAL(I,J) = GSST(I,J) + ANAL(I,J)

c check sst not below 0.0 in Great Lakes or -1.8 elsewhere
c  note, i*j are for 380x180 grid, starting longitude -179.5.
      if(i.ge.98.and.i.le.115.and.j.ge.131.and.j.le.140)then
        IF (ANAL(I,J).LT.0.0) ANAL(I,J) = 0.0
      else
        IF (ANAL(I,J).LT.-1.8) ANAL(I,J) = -1.8
      endif
      ENDIF
   35 CONTINUE
C
C   FILL IN LAND SSTS OVER ANTARTICA FOR STABILITY IN WEAVER
C
      DO 138 I=1,IMX
      DO 138 J=1,30
      IF (LS(I,J).EQ.1) GO TO 138
      WK(I,J) = 1.
      ANAL(I,J) = -1.8
  138 CONTINUE
C
C   FILL IN 2 POINTS NORTH OF 88N AND 2 POINTS SOUTH OF 88S
C   WITH ICE TEMP (-1.8C)
C
      DO 137 I=1,IMX
      ANAL(I,1) = -1.8
      ANAL(I,2) = -1.8
      ANAL(I,179) = -1.8
      ANAL(I,180) = -1.8
      WK(I,1) = 1.
      WK(I,2) = 1.
      WK(I,179) = 1.
      WK(I,180) = 1.
  137 CONTINUE
C
C   FILL IN REGION BETWEEN -190 AND -180 AND +180 AND +190
C
      DO 36 J=1,IMY
      DO 36 I=1,10
      ANAL(I,J) = ANAL(I+360,J)
      ANAL(I+370,J) = ANAL(I+10,J)
      WK(I,J) = WK(I+360,J)
      WK(I+370,J) = WK(I+10,J)
      E(I,J) = E(I+360,J)
      E(I+370,J) = E(I+10,J)
   36 CONTINUE
C
C     FOR WEAVER: IF DATA IN ANAL, WK=1; IF NO DATA, WK=0.
C     FILL IN LAND VALUES WITH WEAVER; OCEAN VALUES UNCHANGED
C
      CALL WEAVER (WEAV,0,ANAL,WK,380,180,-189.5,189.5,-89.5,89.5,
     1 5.,0,0,0,2,0,-1.,ICHK)
      DO 136 J=1,IMY
      DO 136 I=1,10
      ANAL(I,J) = ANAL(I+360,J)
      ANAL(I+370,J) = ANAL(I+10,J)
  136 CONTINUE
      CALL MAXMIN (ANAL,IMX,IMX,IMY,AMIN,AMAX)
      PRINT *,' MIN,MAX IN ANAL (MEAN ANALYSIS SST): ',AMIN,AMAX
      IF (ICHK.NE.0) GO TO 4000
C
C - WRITE OUT OI ANALYSIS TO FT03F001
C
      IUN = 3
      CALL PUTOI (IUN,ANAL,E,IMX,IMY,
     1 IYRST,IMST,IDST,IYREND,IMEND,IDEND,NDAYS,INDEX,cice)
C
      call exit (1)
 1000 PRINT 1001
 1001 FORMAT (' *** NO FIRST GUESS',
     1 5X,'NO OI COMPUTED:  EXIT 11')
      call exit (11)
 2000 PRINT 2001
 2001 FORMAT (' *** NO DATA AVAILABLE',
     1 5X,'NO OI COMPUTED:  EXIT 12')
      call exit (12)
 3000 PRINT 3001
 3001 FORMAT (' *** ICT EXCEEDS JREC; NOT ENOUGH STORAGE FOR DATA',
     1 5X,'NO OI COMPUTED:  EXIT 13')
      call exit (13)
 4000 PRINT 4001
 4001 FORMAT (' *** CANNOT FILL LAND VALUES WITH WEAVER',
     1 5X,'NO OI COMPUTED:  EXIT 14')
      call exit (14)
 5000 PRINT 5001
 5001 FORMAT (' *** PREMATURE END OF OI STATISTICS',
     1 5X,'NO OI COMPUTED:  EXIT 16')
      call exit (16)
 6000 PRINT 6001
 6001 FORMAT (' *** NO DATES FOUND IN BUOY FILE',
     1 5X,'NO OI COMPUTED:  EXIT 15')
      call exit (15)
 6005 PRINT 6006
 6006 FORMAT (' *** ERROR READING ICE CONCENTRATION FIELD',
     1 5X,'NO OI COMPUTED:  call exit 16')
      call exit (16)
 6010 PRINT 6011
 6011 FORMAT (' *** ICE CONCENTRATION DATES DON''T MATCH',
     1 5X,'NO OI COMPUTED:  call exit 17')
      call exit (17)
      END
      SUBROUTINE OISTAT (FLD,IERR)
      DIMENSION FLD(39,19)
      CHARACTER*100 TITLE
      IERR = 0
      READ (1,5,END=100) TITLE
C     PRINT 6,TITLE
      READ (1,5,END=100) TITLE
      PRINT 6,TITLE
      READ (1,5,END=100) TITLE
    5 FORMAT (A100)
C     PRINT 6,TITLE
    6 FORMAT (' OISTAT =  ',A100)
      DO 10 I=3,38
      READ (1,15,END=100) (FLD(I,J),J=1,19)
   15 FORMAT (19F5.0)
   10 CONTINUE
      DO 20 J=1,19
      FLD(1,J) = FLD(37,J)
      FLD(2,J) = FLD(38,J)
      FLD(39,J) = FLD(3,J)
   20 CONTINUE
C     DO 30 I=1,39
C     PRINT 25,I,(FLD(I,J),J=1,19)
C  25 FORMAT (1X,I3,1X,19F6.1)
C  30 CONTINUE
      RETURN
  100 PRINT 110
  110 FORMAT (' PREMATURE END OF STATISTICS: UNIT 1')
      IERR = 1
      RETURN
      END
      SUBROUTINE GETDAT (IUN,ICT,ISST,ISIGC,ISIGU,ILAT,ILON,
     1  iyrst,imst,idst,ITYPE,ID,SGC,SGU,NCUT,L10,ALAT10,ALON10,ICOR)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    GETDAT      GETS SUPER OBS AND FOR ANALYSIS
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-14
C
C ABSTRACT: READ SUPER OBSERVATIONS IN BLOCKS OF UP TO
C   2000. THE SUPER OBS ARRAYS ARE REAL.   THE OUTPUT ARRAYS ARE
C   LINEAR ARRAYS OF ALL OBS IN DEGREES TIMES 100.
C   THIS SUBROUTINE IS CALLED FOR EACH DATA TYPE (BUOY, SHIP,
C   DAY SATELLITE, NIGHT SATELLITE, AND ICE DATA).  THE NEW DATA
C   IS MERGED AT THE END OF THE OLDER DATA.  THE ARRAYS ARE:
C   ISST,ISIGC,ISIGU,ILAT,ILON,ITYPE. THE SIZE IS ICT.
C
C PROGRAM HISTORY LOG:
C   90-12-14  RICHARD W. REYNOLDS
C
C USAGE:    CALL GETDAT (IUN,ICT,ISST,ISIGC,ISIGU,ILAT,ILON,
C          1 iyrst,imst,idst,ITYPE,ID,SIGH,SIGL)
C
C   INPUT ARGUMENT LIST:
C     IUN      - FORTRAN INPUT UNIT NUMBER
C     ICT      - NUMBER OF TOTAL OBSERVATIONS ADDED TO LINEAR ARRAYS
C              - THIS VARIABLE IS INCREASED WITH ADDED DATA
C     ISST     - ARRAY OF SST SUPER OBS (100*DEG C)
C     ISIGU    - ARRAY OF UNCORRELATED SIGMA ERRORS (100*DEG C)
C     ISIGC    - ARRAY OF CORRELATED SIGMA ERRORS (100*DEG C)
C     ILAT     - ARRAY OF LATITUDE SUPER OBS (100*DEG C)
C     ILON     - ARRAY OF LONGITUDE SUPER OBS (100*DEG C)
C     Iyrst    - year of start date
C     Imst     - month of start date
C     Idst     - day of start date
C     ITYPE    - ARRAY OF DATA TYPES (NTESAT,DAYSAT,BUOY,SHIP,ICE)
C              -                     (    =1,    =2,  =3,  =4, =5)
C     ID       - DATA TYPE (INTEGER)
C     SIGH     - MAXIMUM ERROR SIGMA
C     SIGL     - MINIMUM ERROR SIGMA
C
C
C   OUTPUT ARGUMENT LIST:
C     ICT      - NUMBER OF TOTAL OBSERVATIONS ADDED TO LINEAR ARRAYS
C              - THIS VARIABLE IS INCREASED WITH ADDED DATA
C     ISST     - ARRAY OF SST SUPER OBS
C     ISIGU    - ARRAY OF UNCORRELATED SIGMA ERRORS (100*DEG C)
C     ISIGC    - ARRAY OF CORRELATED SIGMA ERRORS (100*DEG C)
C     ILAT     - ARRAY OF LATITUDE SUPER OBS
C     ILON     - ARRAY OF LONGITUDE SUPER OBS
C     ITYPE    - ARRAY OF DATA TYPES (SET EQUAL TO ID)
C
C   INPUT FILES:  DETERMINE BY IUN; SEE MAIN PROGRAM
C
C   OUTPUT FILES:
C     FT06F001 - PRINTOUT
C
C REMARKS:
C   THE OUTPUT LONGITUDES RANGE  FROM -18000 TO
C   +18000 CORRESPONDING TO (180W TO 180E).  THE INPUT LONGITUDES
C   RANGE FROM  0 TO 360E.
C   SIGMA IN ARRAY IS MAXIMUM OF SIGH/SQRT(NOBS) OR SIGL
C   WHERE NOBS IS THE NUMBER OF OBSERVATIONS THAT WERE USED
C   IN THE SUPER OBSERVATION
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
      INTEGER   ITYPE(*),ISIGC(*),ISIGU(*),ISST(*),ILAT(*),ILON(*)
      CHARACTER*8 TYPE
      DIMENSION SGC(39,19),SGU(39,19),ALAT10(19),ALON10(39),L10(39,19)
      DIMENSION SST(2000),RLAT(2000),RLON(2000),NSO(2000)
C
C - READING IN THE DATA
      READ (IUN,END=6000) JYRST,JMST,JDST
      if(jyrst.lt.100)then
        if(jyrst.lt.21)then
          jyrst=jyrst+2000
        else
          jyrst=jyrst+1900
        endif
        print*,'JYRST MODIFIED TO 4-DIGITS:',jyrst
      endif

      IF (iyrst.ne.jyrst.or.imst.ne.jmst.or.idst.ne.jdst) THEN
      PRINT *,' WARNING DATE DOES NOT MATCH FOR ',ID
      print*,' analysis start date:',iyrst,imst,idst
      print*,'     data start date:',jyrst,jmst,jdst
      ENDIF
   50 READ (IUN,END=6000) TYPE,NOBS
      IF (NOBS.EQ.0) GO TO 5000
      READ (IUN,END=6000) (NSO(M),M = 1,NOBS)
      READ (IUN,END=6000) (SST(M),M = 1,NOBS)
      READ (IUN,END=6000) (RLAT(M),M = 1,NOBS)
      READ (IUN,END=6000) (RLON(M),M = 1,NOBS)
C - BLANK READ TO BYPASS SIGMAS
      READ (IUN,END=6000)
C
      DO 350 I = 1,NOBS
      IF (NSO(I).LT.NCUT) GO TO 350
      ICT = ICT + 1
      ISST(ICT) = SST(I)*100.
      XLAT = RLAT(I)
      ILAT(ICT) = XLAT*100.
      XLON = RLON(I)
      IF (XLON.GT.180.) XLON = XLON - 360.
      ILON(ICT) = XLON*100.
      CALL INTRP1 (SGU,39,39,19,ALON10,ALAT10,L10,
     1 XLON,XLAT,SIGTP,IERR)
        IF (IERR.NE.0) THEN
        PRINT 10,XLAT,XLON
   10   FORMAT (' BAD POSITION: SIG-UNCOR SET TO 1:   XLAT =',F10.3,3X,
     1  'XLON =',F10.3)
        SIGTP = 1.
        ENDIF
      ISIGU(ICT) = 100.*SIGTP
      ITYPE(ICT) = ID
      IF (ICOR.EQ.0) THEN
      ISIGC(ICT) = 0.
      ELSE
      CALL INTRP1 (SGC,39,39,19,ALON10,ALAT10,L10,
     1 XLON,XLAT,SIGTP,IERR)
        IF (IERR.NE.0) THEN
        PRINT 20,XLAT,XLON
   20   FORMAT (' BAD POSITION: SIG-COR SET TO 1:   XLAT =',F10.3,3X,
     1  'XLON =',F10.3)
        SIGTP = 1.
        ENDIF
      ISIGC(ICT) = 100.*SIGTP
      ENDIF
C     IF (I.EQ.501) PRINT 80,ICT,ILAT(ICT),ILON(ICT),ISST(ICT),
C    1 ISIGC(ICT),ISIGU(ICT),ITYPE(ICT)
C  80 FORMAT (' ICT =',I7,2X,'ILAT =',I7,2X,'ILON =',I7,2X,'ISST =',I7,
C    1 2X,'ISIGC =',I7,2X,'ISIGU =',I7,2X,'ITYPE =',I3)
  350 CONTINUE
C
      GO TO 50
C
 5000 PRINT 101,JYRST,JMST,JDST,ICT
  101 FORMAT (' INPUT DATA START DATE: YEAR =',I4,3X,'MONTH =',I3,3X,
     1 'DAY =',I3,5X,'TOTAL NUMBER OF DATA POINTS READ =',I8)
C     CALL MXMN2 (ISST,ICT,AMIN,AMAX,.01)
C     PRINT *,' MIN,MAX IN TEMP:',AMIN,AMAX
C     CALL MXMN2 (ISIGU,ICT,AMIN,AMAX,.01)
C     PRINT *,' MIN,MAX IN SIG-UNCOR:',AMIN,AMAX
C     CALL MXMN2 (ISIGC,ICT,AMIN,AMAX,.01)
C     PRINT *,' MIN,MAX IN SIG-COR:',AMIN,AMAX
C     CALL MXMN2 (ILON,ICT,AMIN,AMAX,.01)
C     PRINT *,' MIN,MAX IN LON:',AMIN,AMAX
C     CALL MXMN2 (ILAT,ICT,AMIN,AMAX,.01)
C     PRINT *,' MIN,MAX IN LAT:',AMIN,AMAX
      RETURN
 6000 PRINT 7000
 7000 FORMAT (' WARNING! - PREMATURE END OF INPUT DATA')
      RETURN
      END
      SUBROUTINE GETFG (IUN,IMX,IMY,IERR,GSST)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    GETFG       GETS FIRST GUESS FIELD ON ANALYSIS GRID
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-14
C
C ABSTRACT: READS THE GUESS FIELD FROM UNIT IUN
C
C PROGRAM HISTORY LOG:
C   90-12-14  RICHARD W. REYNOLDS
C
C USAGE:    CALL GETFG (IUN,IMX,IMY,IERR,GSST)
C
C   INPUT ARGUMENT LIST:
C     IUN      - FORTRAN INPUT UNIT NUMBER
C     IMX      - FIRST DIMENSION OF GSST
C     IMY      - SECOND DIMENSION OF GSST
C
C   OUTPUT ARGUMENT LIST:
C     IERR     - ERROR PARAMETER
C              - IF 0 GUESS FIELD FOUND, IF 1 NO GUESS
C     GSST     - FIRST GUESS ARRAY
C              - ON ANALYSIS GRID (-189.5 TO 189.5 AND -89.5 TO 89.5)
C
C   INPUT FILES:  DETERMINE BY IUN; SEE MAIN PROGRAM
C
C   OUTPUT FILES:
C     FT06F001 - PRINTOUT
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
      DIMENSION GSST(IMX,IMY)
      IERR = 0
C - READING IN THE FIRST-GUESS
      READ (IUN,END=6000) JYRST,JMST,JDST
      PRINT 10,JYRST,JMST,JDST
   10 FORMAT (' FIRST GUESS START DATE:  YEAR =',I4,3X,'MONTH =',I3,
     1 3X,'DAY =',I3)
*     READ (IUN,END=6000) ((GSST(I,J),I = 1,IMX),J = 1,IMY)
      read (iun,end=6000) ((gsst(i,j),i = 191,370),(gsst(i,j),i=11,190),
     1     j=1,imy)
*     do j=1,imy
*       do i=1,10
*         gsst(i,j)=gsst(i+360,j)
*         gsst(i+370,j)=gsst(i+10,j)
*       enddo
*     enddo
      gsst(1:10,:)=gsst(361:370,:)
      gsst(371:380,:)=gsst(11:20,:)
      CALL MAXMIN (GSST,IMX,IMX,IMY,AMIN,AMAX)
      PRINT *,' MIN,MAX IN GUESS:',AMIN,AMAX
      RETURN
 6000 PRINT 7000
 7000 FORMAT(' ERROR! - NO FIRST-GUESS: IERR=1 IN GETFG')
      IERR = 1
      RETURN
      END
      SUBROUTINE SMOOTH (A,B,LS,IDIM,M,N)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    SMOOTH      PROVIDES 1-2-1 SMOOTHER
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-14
C
C ABSTRACT: SMOOTHS INPUT ARRAY A WITH 1-2-1 SMOOTHER
C   IN THE NORTH/SOUTH AND EAST/WEST DIRECTION AND RETURNS RESULT IN
C   ARRAY A.
C   SMOOTHER ONLY IS DONE USING OCEAN POINTS (LS=1)
C
C PROGRAM HISTORY LOG:
C   90-12-14  RICHARD W. REYNOLDS
C
C USAGE:    CALL SMOOTH (A,B,LS,IDIM,M,N)
C
C   INPUT ARGUMENT LIST:
C     A        - TWO INDEX ARRAY TO BE SMOOTHED
C     LS       - LAND SEA TABLE (OCEAN IS 1, LAND IS 0)
C     IDIM     - ACTUAL FIRST INDEX OF A,B AND LS
C              - AS DIMENSIONED IN MAIN PRGM
C     M        - FIRST INDEX OF A, B AND LS
C     N        - SECOND INDEX OF A, B, AND LS
C
C   OUTPUT ARGUMENT LIST:
C     A        - SMOOTHED RESULT
C     B        - WORK ARRAY
C
C REMARKS:
C   SMOOTHER IS CALLED TO SMOOTH GUESS FIELD IF IFIL=1 IN MAIN PRGM
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
C   B IS WORK ARRAY
      DIMENSION A(IDIM,N),B(IDIM,N),LS(IDIM,N)
      PRINT 5
    5 FORMAT (' CALL SUBROUTINE SMOOTH')
      DO 10 I=1,M
      DO 10 J=1,N
      B(I,J) = A(I,J)
   10 CONTINUE
      NM = N - 1
      MM = M - 1
      DO 20 I=2,MM
      IP = I + 1
      IM = I - 1
      DO 30 J=2,NM
      JP = J + 1
      JM = J - 1
      IF (LS(I,J).EQ.0) GO TO 30
      WT = LS(IP,J) + LS(IM,J) + LS(I,JP) + LS(I,JM) + 4
      CIP = B(IP,J) * FLOAT( LS(IP,J) )
      CIM = B(IM,J) * FLOAT( LS(IM,J) )
      CJP = B(I,JP) * FLOAT( LS(I,JP) )
      CJM = B(I,JM) * FLOAT( LS(I,JM) )
      A(I,J) = (4.*B(I,J) + CIP + CIM + CJP + CJM)/WT
   30 CONTINUE
   20 CONTINUE
      RETURN
      END
      SUBROUTINE GENGRD (IMX,IMY,XT,YT)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    GENGRD      GENERATES LAT AND LONG OF OI GRID
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-14
C
C ABSTRACT: GENERATES THE LATITUDE AND LONGITUDES
C   FOR THE GRID CENTER OF THE OI ANALYSIS GRID.  IT ALSO GENERATES
C   THE COSINE OF THE LATITUDE POSITION
C
C PROGRAM HISTORY LOG:
C   90-12-14  RICHARD W. REYNOLDS
C
C USAGE:    CALL GENGRD (IMX,IMY,XT,YT)
C
C   INPUT ARGUMENT LIST:
C     IMX      - LENGTH OF THE LONGITUDE ARRAY
C              - (FIRST INDEX OF ANALYSIS GRID)
C     IMY      - LENGTH OF THE LATITUDE ARRAY
C              - (SECOND INDEX OF ANALYSIS GRID)
C
C   OUTPUT ARGUMENT LIST:
C     XT       - LONGITUDES ON ANALYSIS GRID
C              - BETW -189.5 (189.5W) TO 189.5 (189.5E)
C     YT       - LATITUDES ON ANALYSIS GRID
C              - BETW -89.5 (89.5S) TO 89.5 (89.5N)
C
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
      DIMENSION XT(IMX),YT(IMY)
C - DO THE LONGITUDES
      DO 50 I = 1,IMX
      XT(I) = FLOAT(I) - 190.5
   50 CONTINUE
C - DO THE LATITUDES
      DO 100 J = 1,IMY
      YT(J) = FLOAT(J) - 90.5
  100 CONTINUE
      RETURN
      END
      SUBROUTINE INTRP1 (CN,IMC,IC,JC,CX,CY,LS,FX,FY,FN,IERR)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    INTRP1      FINDS INTERPOLATED VALUE
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-14
C
C ABSTRACT: FINDS A LINEARLY INTERPOLATED VALUE FN AT LOCATION
C   FX (LONGITUDE) AND FY (LATITUDE) RELATIVE TO
C   AN ANALYZED FIELD CN THAT IS
C   DEFINED BY CX (LONGITUDE) AND CY (LATITUDE).
C   THE INTERPOLATION INCLUDES AN ADJUSTMENT FOR THE
C   CONVERGENCE OF LONGITUDE LINES AS A FUNCTION OF LATITUDE.
C   LAND VALUES (LS=0) ARE NOT USED IN THE INTERPOLATION.
C   NOTE: THE LOCATION (FX,FY) MUST LIE WITHIN THE GRID
C   DEFINED BY CX AND CY.
C
C PROGRAM HISTORY LOG:
C   90-12-14  RICHARD W. REYNOLDS
C
C USAGE:    CALL INTRP1 (CN,IMC,IC,JC,CX,CY,LS,FX,FY,FN,IERR)
C
C   INPUT ARGUMENT LIST:
C     CN       - GRIDDED FIELD
C     IMC      - ACTUAL 1ST DIMENSION OF CN IN CALLING ROUTINE
C     IC       - FIRST DIMENSION OF CN
C     JC       - SECOND DIMENSION OF CN
C     CX       - DISTRIBUTION OF LONGITUDE GRID POINTS IN CN
C     CY       - DISTRIBUTION OF LATITUDE GRID POINTS IN CN
C     LS       - LAND SEA TAG: (=1 OVER WATER AND =0 OVER LAND)
C     FX       - LONGITUDE OF FN RELATIVE TO CX COORDINATES
C     FY       - LATITUDE OF FN RELATIVE TO CY COORDINATES
C
C   OUTPUT ARGUMENT LIST:
C      FN       - LINEAR INTERPOLATED VALUE FROM CN ARRAY
C               - AT LOCATION FX AND FY
C      IERR     - IERR = 0: INTERPOLATED VALUE OK
C               - IERR = 1, FN = 0:    NO INTERPOLATION
C               -   BECAUSE ALL GRID POINTS ARE OVER LAND,
C               -   OR CX(1) IS LESS THAN OR EQUAL TO FX,
C               -   OR CX(IC) IS GREATER THAN OR EQUAL TO FX,
C               -   OR CX(1) IS LESS THAN OR EQUAL TO FX,
C               -   OR CY(JC) IS GREATER THAN OR EQUAL TO FY,
C               -   OR CX OR CY DOES NOT MONOTONICALLY INCREASE
C
C   OUTPUT FILES:
C     FT06F001 - PRINTOUT
C
C REMARKS:  IF IERR=1, CX AND CY ARRAYS AND FX AND FY ARE PRINTED
C           UNLESS ALL GRID POINTS ARE OVER LAND
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
      DIMENSION CN(IMC,JC),CX(IC),CY(JC)
      DIMENSION LS(IMC,JC)
      IF (FX.LT.CX(1).OR.FX.GT.CX(IC)) GO TO 10
      IF (FY.LT.CY(1).OR.FY.GT.CY(JC)) GO TO 10
      DO 110 L=1,JC
      IF (CY(L).LT.FY) GO TO 110
      JN = L
      JS = L-1
      IF (JS.EQ.0) JN = 2
      IF (JS.EQ.0) JS = 1
      GO TO 111
  110 CONTINUE
      JN = JC
      JS = JC - 1
  111 CONTINUE
      DO 120 L=1,IC
      IF (CX(L).LT.FX) GO TO 120
      IE = L
      IW = L-1
      IF (IW.EQ.0) IE = 2
      IF (IW.EQ.0) IW = 1
      GO TO 121
  120 CONTINUE
      IE = IC
      IW = IC - 1
  121 CONTINUE
      A = (FX - CX(IW))
      B = (CX(IE) - FX)
      C = FY - CY(JS)
      D = CY(JN) - FY
      AC = A*C * FLOAT( LS(IE,JN) )
      BC = B*C * FLOAT( LS(IW,JN) )
      AD = A*D * FLOAT( LS(IE,JS) )
      BD = B*D * FLOAT( LS(IW,JS) )
      TOP = AC*CN(IE,JN) + BC*CN(IW,JN) + AD*CN(IE,JS) + BD*CN(IW,JS)
      BOT = AC + BC + AD + BD
      IERR = 0
      IF (BOT.EQ.0.) GO TO 200
      FN = TOP/BOT
      RETURN
C
C     NO INTERPOLATED VALUE
C
   10 PRINT 15
   15 FORMAT (' PROBLEM IN INTRP1: FX OR FY OUT OF RANGE;',
     1 '   NO INTERPOLATED VALUE')
      PRINT 20,CX(1),CX(IC),FX
   20 FORMAT (' CX(1) =',F10.3,3X,'CX(IC) =',F10.3,3X,'FX =',F10.3)
      PRINT 25,CY(1),CY(JC),FY
   25 FORMAT (' CY(1) =',F10.3,3X,'CY(JC) =',F10.3,3X,'FY =',F10.3)
  200 FN = 0.
      IERR = 1
      RETURN
      END
      SUBROUTINE WRAP (ICT,ISST,D,ILAT,ILON,ISIGC,ISIGU,ITYPE)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    WRAP        ADDS EXTRA DATA IN 10 DEG BORDER ZONE
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-14
C
C ABSTRACT: THE OI ANALYSIS IS DONE FROM -180 (180W) TO +180 (180E)
C   TO ASSURE DATA JUST BEYOND THE EDGE OF THE ANALYSIS DOMAIN
C   DATA FROM 171E TO 180E IS COPIED INTO 180W TO 189W AND
C   DATA FROM 171W TO 180W IS COPIED INTO 180E TO 189E.
C   THESE NEW REGIONS ARE THE ADDED BORDER REGIONS
C   FOR DATA VALUES WITH ABSOLUTE LONGITUDE GREATER THAN 171 DEG.
C   ALL THE DATA VALUES (ISST,D,ILAT,ISIGC,ISIGU AND ITYPE)
C   ARE REPEATED EXCEPT LONGITUDE (ILON).
C   LONGITUDES ARE ADJUSTED BY ADDING 360 DEG FOR NEGATIVE LONGITUDES
C   AND SUBTRACTING 360 DEG FROM POSITIVE LONGITUDES.  THE NEW DATA
C   IS MERGED AT THE END OF THE OLDER DATA.  THE SIZE OF ICT IS
C   INCREASED TO ADJUST FOR THE ADDITIONAL OF DATA.
C
C PROGRAM HISTORY LOG:
C   90-12-14  RICHARD W. REYNOLDS
C
C USAGE:    CALL  WRAP (ICT,ISST,D,ILAT,ILON,ISIGC,ISIGU,ITYPE)
C
C   INPUT/OUTPUT ARGUMENT LIST:
C     ICT      - NUMBER OF TOTAL OBSERVATIONS ADDED TO LINEAR ARRAYS
C              - THIS VARIABLE IS INCREASED WITH ADDED DATA
C     ISST     - ARRAY OF SSTS (100*DEG C)
C     D        - ARRAY OF DATA INCREMENTS (DEG C)
C     ISIGC    - ARRAY OF CORRELATED SIGMA ERRORS (100*DEG C)
C     ISIGU    - ARRAY OF UNCORRELATED SIGMA ERRORS (100*DEG C)
C     ILAT     - ARRAY OF LATITUDES (100*DEG)
C     ILON     - ARRAY OF LONGITUDES (100*DEG)
C     ITYPE    - ARRAY OF DATA TYPES (NTESAT,DAYSAT,BUOY,SHIP,ICE)
C              -                     (    =1,    =2,  =3,  =4, =5)
C
C   OUTPUT FILES:
C     FT06F001 - PRINTOUT
C
C REMARKS:
C   ALL VALUES ARE BOTH INPUT AND OUTPUT BECAUSE DATA ADDED TO
C   BORDERS ARE ADDED ON AT THE END OF THE INPUT ARRAYS.
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
      INTEGER   ISST(*),ILAT(*),ILON(*),ISIGC(*),ISIGU(*),ITYPE(*)
      DIMENSION D(*)
C     PRINT *,' ICT IN WRAP',ICT
      K = ICT + 1
      JCT = ICT
C     CALL MXMN2 (ISST,ICT,AMIN,AMAX,.01)
C     PRINT *,' MIN,MAX IN TEMP:',AMIN,AMAX
C     CALL MXMN2 (ISIGC,ISIGU,ICT,AMIN,AMAX,.01)
C     PRINT *,' MIN,MAX IN SIG-UNCOR:',AMIN,AMAX
C     CALL MXMN2 (ISIGC,ISIGC,ICT,AMIN,AMAX,.01)
C     PRINT *,' MIN,MAX IN SIG-COR:',AMIN,AMAX
C     CALL MXMN2 (ILON,ICT,AMIN,AMAX,.01)
C     PRINT *,' MIN,MAX IN LON:',AMIN,AMAX
C     CALL MXMN2 (ILAT,ICT,AMIN,AMAX,.01)
C     PRINT *,' MIN,MAX IN LAT:',AMIN,AMAX
C     CALL MXMN2 (ITYPE,ICT,AMIN,AMAX,.01)
C     PRINT *,' MIN,MAX IN ITYPE:',AMIN,AMAX
      DO 5 J = 1,JCT
        TMP = ILON(J)
        RLON = .01*TMP
      IF (ABS(RLON).LT.171.) GO TO 5
C   FIRST - WRAP BETWEEN -171. AND -180.
      IF (RLON.LE.0.) THEN
        RLON = RLON + 360.
        ILON(K) = RLON*100.
        ILAT(K) = ILAT(J)
        ISST(K) = ISST(J)
        D(K) = D(J)
        ISIGU(K) = ISIGU(J)
        ISIGC(K) = ISIGC(J)
        ITYPE(K) = ITYPE(J)
        K = K + 1
C   SECOND - WRAP BETWEEN +171. AND +180.
      ELSE
        RLON = RLON - 360.
        ILON(K) = RLON*100.
        ILAT(K) = ILAT(J)
        ISST(K) = ISST(J)
        D(K) = D(J)
        ISIGU(K) = ISIGU(J)
        ISIGC(K) = ISIGC(J)
        ITYPE(K) = ITYPE(J)
        K = K + 1
      ENDIF
    5 CONTINUE
      ICT = K - 1
      NNEW = ICT - JCT
      PRINT *,' NUMBER OF WRAPPED OBSERVATIONS = ',NNEW,
     1 '     NEW TOTAL NUMBER OF OBSERVATIONS = ',ICT
      RETURN
      END
      SUBROUTINE SSTMAP (D,ILON,ILAT,ICT,ANAL,E,IMX,IMY,ISIGC,ISIGU,LS,
     1 XT,YT,ITYPE,SEW,SNS,L10,ALAT10,ALON10)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    SSTMAP      DETERMINES BOXES FOR OI SOLUTION
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-18
C
C ABSTRACT: FINDS ALL COMPUTATIONAL BOXES FOR THE OI
C   SOLUTION.  EACH BOX IS DEFINED VIA DO LOOPS (LABEL 10)
C   ON THE LONGITUDE AND LATITUDE INDICES.  THE DATA WITHIN
C   EACH BOX PLUS THE BORDER AROUND IT IS THEN USED
C   FOR THE OI SOLUTION.  DATA POINTS TOO CLOSE TOGETHER
C   -- LESS THAN RMIN (INITIALLY SET TO DMIN=25KM) -- ARE
C   THINNED BY SUBROUTINE CKDIST.  IF THE NUMBER OF
C   OBSERVATIONS PER BOX PLUS ITS BORDEER IS LARGER
C   THAN KMAX (NOW = 10*IMAX) AN ERROR MESSAGE IS PRINTED.  IF THE
C   NUMBER OF OBS IS GREATER IMAX (NOW =200) AFTER THINNING,
C   RMIN IS INCREASED BY A FACTOR OF 1.5 UNTIL THE NUMBER
C   OF OBS IS NOT GREATER THAN IMAX.  IMAX IS THE MAXIMUM
C   SIZE OF THE LINEAR SYSTEM TO BE SOLVED.  THE EQUATION
C   CAN BE WRITTEN AS "A"*"X"="B" WHERE "A" IS A TWO DIM.
C   ARRAY OF COEFFICIENTS DETERMINED BY THE DATA AND THE
C   GUESS AT THE LOCATIONS OF THE DATA. "B" IS A ONE
C   DIMENSIONAL ARRAY OF COEFFICIENTS DETERMINED BY THE GUESS
C   AT THE LOCATIONS OF THE DATA AND THE ANALYSIS GRID.
C   THE ARRAY "X" IS A ONE DIMENSIONAL ARRAY OF THE REQUIRED
C   WEIGHTS FOUND BY THE SOLUTION OF THE LINEAR EQUATION.
C   THE WEIGHTS TIMES THE DATA INCREMENTS YIELDS THE OI SSTS.
C   ANOTHER LINEAR RELATIONSHIP GIVES THE ASSOCIATED ERRORS.
C   SUBROUTINE DECOMP FORMS THE ARRAY "A" AND ATTEMPTS
C   TO DECOMPOSE IT.  ESTIM CALCULATES "B" AND FINDS "X"
C   BE USING THE RESULTS FROM THE DECOMPOSITION.
C   DECOMP IS CALLED ONCE FOR EACH
C   BOX. ESTIM IS CALLED FOR EACH ANALYSIS GRID POINT IN THE
C   BOX.  IF "A" CAN NOT BE ACCURATELY DECOMPOSED
C   (I.E. THE VARIABLE "COND" FROM DECOMP IS GREATER THAN CNMX
C   (NOW = 100.), RMIN IS INCREASED BY A FACTOR OF 1.5,
C   THE DATA ARE THINNED AGAIN BY CKDIST AND THE
C   DECOMPOSITION IS DONE AGAIN.  THIS PROCEDURE IS
C   REPEATED UNTIL "COND" IS NOT GREATER THAN CNMX.
C   THE OI SOLUTION IS RETURNED IN ANAL (ANALYSIS
C   INCREMENT SST) AND E (ANALYSIS ERROR).
C
C PROGRAM HISTORY LOG:
C   90-12-18  RICHARD W. REYNOLDS
C
C USAGE:    CALL SSTMAP (D,ILON,ILAT,ICT,ANAL,E,IMX,IMY,ISIG,LS,GSIG,
C          1 XT,YT,ITYPE)
C
C   INPUT ARGUMENT LIST:
C     D        - ARRAY OF DATA INCREMENTS (DEG C)
C     ILON     - ARRAY OF LONGITUDE OF SUPER OBS (100*DEG C)
C     ILAT     - ARRAY OF LATITUDE OF SUPER OBS (100*DEG C)
C     ICT      - NUMBER OF TOTAL OBSERVATIONS
C     IMX      - FIRST INDEX OF LS, ANAL AND E (ON THE ANALYSIS GRID)
C     IMY      - SECOND INDEX OF LS, ANAL AND E (ON THE ANALYSIS GRID)
C     ISIG     - ARRAY OF SUPER OBS SIGMA ERROR (100*DEG C)
C     LS       - ARRAY OF LAND SEA TAGS
C              - (LS=1 OVER WATER AND LS=0 OVER LAND)
C     GSIG     - GUESS SIGMA ERROR (DEG C)
C     XT       - ARRAY LONGITUDE LOCATIONS OF ANALYSIS GRID
C              - (CENTERED ON HALF DEG BETW -189.5 TO 189.5)
C     YT       - ARRAY OF LATITUDE LOCATIONS
C              - (CENTERED ON HALF DEG BETW -89.5 TO 89.5)
C     ITYPE    - ARRAY OF DATA TYPES (NTESAT,DAYSAT,BUOY,SHIP,ICE)
C              -                     (    =1,    =2,  =3,  =4, =5)
C
C     PARAMETERS:
C              - IMAX = 200, MAXIMUM SIZE OF LINEAR EQUATION
C              - KMAX = 10*IMAX, MAXIMUM NUMBER OF DATA POINTS
C              - PER BOX AND BORDER
C
C   OUTPUT ARGUMENT LIST:
C     ANAL     - ARRAY OF ANALYSIS INCREMENTS (DEG C)
C              - ON ANALYSIS GRID
C     E        - ARRAY OF ERROR VALUES (RANGE 0 TO 1)
C              - ON ANALYSIS GRID
C
C COMMON BLOCKS: SCALES, SCLSQR, DOMAIN
C              - XSCL   LONGITUDE SIZE OF COMPUTATION BOX IN DEG
C              -        CAN BE MULTIPLE OF 1-DEG (SEE LORENC)
C              -        ALL BOX EDGES ON EVEN DEGREE
C              -        DATA POINTS ON HALF DEGREE
C              - YSCL   LATITUDE  SIZE OF COMPUTATION BOX IN DEG
C              - XBORD : X-BORDER TO INCLUDE DATA OUTSIDE XSCL (DEG)
C                      : TOTAL BOX IS XSCL + 2*XBORD
C              - YBORD : Y-BORDER TO INCLUDE DATA OUTSIDE YSCL (DEG)
C                      : TOTAL BOX IS YSCL + 2*YBORD
C              -XMN     LONGITUDE OF WESTWARD LIMIT OF WEST SIDE
C                       OF COMPUTATIONAL BOX; EVEN DEGREES
C                       (XMX-XMN)/XSCL MUST BE INTEGER
C              - XMX    LONGITUDE OF EASTWARD LIMIT OF EAST SIDE
C                       OF COMPUTATIONAL BOX
C              - YMN    LATITUDE OF SOUTHWARD LIMIT OF SOUTH SIDE
C                       OF COMPUTATIONAL BOX
C                       (YMX-YMN)/YSCL MUST BE INTEGER
C              - YMX    LATITUDE OF NORTHWARD LIMIT OF NORTH SIDE
C                       OF COMPUTATIONAL BOX
C
C   OUTPUT FILES:
C     FT06F001 - PRINTOUT
C
C   SUBPROGRAMS CALLED:
C     UNIQUE:  - SSTMAP, CKDIST, DECOMP, ESTIM, MAXMIN
C
C REMARKS:  GWEST IS LONGITUDE OF MOST WESTERN BOX
C           GSOUTH IS MINIMUM LATITUDE OF MOST SOUTHERN BOX
C           CMIN IS THE MINIMUM CONDITION CODE (COND) FROM DECOMP
C           CMX1 IS THE MAXIMUM CONDITION CODE (COND) FROM DECOMP
C           WITHOUT THINNING BY CKDIST
C           CMX2 IS THE MAXIMUM CONDITION CODE (COND) FROM DECOMP
C           WITH THINNING BY CKDIST
C           NOTE: IF THE SIZE OF ARRAY TO BE DECOMPOSED, IMAX,
C           IS CHANGED, NMX WHICH IS = TO IMAX MUST ALSO BE
C           CHANGED IN DECOMP AND ESTIM.
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
      PARAMETER (IMAX=200,KMAX=IMAX*10)
      DIMENSION D(*),ANAL(IMX,*),E(IMX,*),XT(IMX),YT(IMY),SZT(KMAX),
     1 XZ(KMAX),YZ(KMAX),DZ(KMAX),SZC(KMAX),SZU(KMAX),LS(IMX,*)
      COMMON /DOMAIN/ XSCL,YSCL,XBORD,YBORD,XMN,XMX,YMN,YMX
      COMMON /SCLSQR/ SCX2,SCY2
      DIMENSION SEW(39,19),SNS(39,19),L10(39,19),ALAT10(19),ALON10(39)
      INTEGER   IFLG(KMAX),ITYPE(*),JTYPE(KMAX),ILAT(*),ILON(*)
      INTEGER   ISIGC(*),ISIGU(*),INDX(KMAX)
      DIMENSION NITER(10)
      DATA NITER /10*0/
      DATA NUMDEC /0/
      DATA CMIN,CMX1,CMX2/9999.,0.,0./,DMIN/25./,CNMX/100/
C
C
      DLX = XMX - XMN
      DLY = YMX - YMN
      OXSL = XSCL + XBORD*2.
      OYSL = YSCL + YBORD*2.
      DX = OXSL/2.
      DY = OYSL/2.
      NX  = DLX/XSCL
      NY  = DLY/YSCL
      IF (NX*XSCL.NE.DLX) PRINT *,' NOT AN EVEN DIVISION IN E-W'
      IF (NY*YSCL.NE.DLY) PRINT *,' NOT AN EVEN DIVISION IN N-S'
C
      DO 80 JJ = 1,IMY
      DO 80 II = 1,IMX
      ANAL(II,JJ) = 0.
      E(II,JJ) = 1.
   80 CONTINUE
      GWEST = -190.
      GSOUTH = -90.
      XADD = XMN - GWEST
      YADD = YMN - GSOUTH
C     IF (XADD.LT.0.0.OR.YADD.LT.0.0) EXIT 999
      DO 10 I=1,NX
      IX1 = FLOAT(I-1)*XSCL + 1. + XADD
      IX2 = IX1 + NINT(XSCL) - 1
      XC  = FLOAT(2*I-1)*XSCL/2. + XMN
      ALONM = XC - DX
      ALONP = XC + DX
C     PRINT 21,IX1,IX2,ALONM,ALONP,XC,I
C  21 FORMAT(' RANGE IN X:',2I5,3F7.2,3X,' I =',I3)
      DO 10 J=1,NY
      IY1 = FLOAT(J-1)*YSCL + 1. + YADD
      IY2 = IY1 + NINT(YSCL) - 1
      YC = FLOAT(2*J-1)*YSCL/2. + YMN
      ALATM = YC - DY
      ALATP = YC + DY
C     PRINT 22,IY1,IY2,ALATM,ALATP,YC,J
C  22 FORMAT(' RANGE IN Y:',2I5,3F7.2,' J=',I3)
      CALL INTRP1 (SEW,39,39,19,ALON10,ALAT10,L10,
     1 XC,YC,SCLX,IERR)
        IF (IERR.NE.0) THEN
        PRINT 208,XC,YC
  208   FORMAT (' BAD POSITION: SCLX-KM SET = 500:   XC =',F10.3,3X,
     1  'YC =',F10.3)
        SCLX = 500.
        ENDIF
      CALL INTRP1 (SNS,39,39,19,ALON10,ALAT10,L10,
     1 XC,YC,SCLY,IERR)
        IF (IERR.NE.0) THEN
        PRINT 209,XC,YC
  209   FORMAT (' BAD POSITION: SCLY-KM SET = 500:   XC =',F10.3,3X,
     1  'YC =',F10.3)
        SCLY = 500.
        ENDIF
      SCX2 = SCLX**2
      SCY2 = SCLY**2
C     IF (I.EQ.NX/2) PRINT 23,I,J,SCLX,SCLY,XC,YC,
C    1 ALONM,ALONP,ALATM,ALATP
C  23 FORMAT (' I =',I3,3X,'J =',I3,3X,'SCLX =',F10.0,3X,
C    1 'SCLY =',F10.0/' XC =',F6.2,3X,'YC =',F6.2,3X,
C    2 'LON RANGE =',F6.2,F8.2,3X,'LAT RANGE =',F6.2,F8.2)
C
C     PRINT *,' XC,YC: ',XC,YC
C     PRINT *,' ALATM,ALATP: ',ALATM,ALATP
C     PRINT *,' ALONM,ALONP: ',ALONM,ALONP
C     PRINT *,' XT(IX1),XT(IX2): ',XT(IX1),XT(IX2)
C     PRINT *,' YT(IY1),YT(IY2): ',YT(IY1),YT(IY2)
      LL=0
      DO 71 L=1,ICT
      IF (D(L).LE.-900.) GO TO 71
      TMP1 = ILON(L)
      XZC = 0.01*TMP1
      TMP2 = ILAT(L)
      YZC = 0.01*TMP2
      IF (YZC.LT.ALATM.OR.YZC.GT.ALATP) GO TO 71
      IF (XZC.LT.ALONM.OR.XZC.GT.ALONP) GO TO 71
      LL=LL+1
      IF (LL.GT.KMAX) GO TO 71
      XZ(LL) = XZC
C     PRINT *,' XZ(LL) =',XZ(LL)
      YZ(LL) = YZC
C     PRINT *,' YZ(LL) =',YZ(LL)
      DZ(LL) = D(L)
      TMP3 = ISIGC(L)
      SZC(LL) = .01*TMP3
      TMP4 = ISIGU(L)
      SZU(LL) = .01*TMP4
      SZT(LL) = SQRT(SZC(LL)**2 + SZU(LL)**2)
      JTYPE(LL)= ITYPE(L)
   71 CONTINUE
C     PRINT *,' XZ(I) '
C     WRITE (6,607) (XZ(I),I=1,N)
C     PRINT *,' YZ(I) '
C     WRITE (6,607) (YZ(I),I=1,N)
C 607 FORMAT(9(1PE8.4))
C
      IF (LL.EQ.0) GO TO 10
C     IF (LL.EQ.0) THEN
C     PRINT *,' NO OBS FOUND, I,J=',I,J
C     GO TO 10
C     ENDIF
C
      IF (LL.GT.KMAX) THEN
      PRINT 59,KMAX,LL,I,J
   59 FORMAT (' *** LL WILL BE REDUCED TO KMAX:  KMAX =',I5,3X,
     1 'LL =',I5,3X,'I =',I3,3X,'J =',I3)
      LL = KMAX
      ENDIF
      N=LL
      RMIN = DMIN
      ITER = 0
C     DO 179 L=1,N
C     PRINT *,' BEFORE CKDIST',L,XZ(L),YZ(L),DZ(L),SZT(L),JTYPE(L)
C 179 CONTINUE
  200 CALL CKDIST (RMIN,XZ,YZ,DZ,SZC,SZU,SZT,N,IFLG,JTYPE,INDX)
C     DO 178 L=1,N
C     PRINT *,' AFTER  CKDIST',L,XZ(L),YZ(L),DZ(L),SZT(L),JTYPE(L)
C 178 CONTINUE
      IF (N.LT.2) GO TO 10
C     IF (N.LT.2) THEN
C     PRINT *,' TOO FEW OBS AFTER CKDIST, I,J=',I,J
C     GO TO 10
C     ENDIF
      IF (N.GT.IMAX) THEN
      RMIN = RMIN*1.5
      ITER = ITER + 1
C     PRINT 58,N,IMAX,RMIN,I,J
      PRINT 58,RMIN,N,IMAX,I,J
   58 FORMAT (' *** TOO MANY DATA POINTS; ITERATE: NEW RMIN=',
     1 F8.1,3X,'N =',I3,3X,
     2 'IMAX =',I3,3X,'I =',I3,3X,'J =',I3)
      PRINT *,' ALATM,ALATP: ',ALATM,ALATP
      PRINT *,' ALONM,ALONP: ',ALONM,ALONP
      GO TO 200
      ENDIF
      CALL DECOMP (XZ,YZ,SZC,SZU,N,JTYPE,COND)
      IF (COND.GT.CMX1) CMX1 = COND
      IF (COND.GT.CNMX) THEN
      RMIN = RMIN*1.5
      ITER = ITER + 1
C     PRINT 57,RMIN,I,J,N
C  57 FORMAT (' *** STABILITY PROBLEM IN DECOMP; ITERATE: NEW RMIN =',
C    1 F8.1,3X,'I =',I3,3X,'J =',I3,3X,'N =',I4)
      GO TO 200
      ENDIF
      IF (COND.LT.CMIN) CMIN = COND
      NUMDEC = NUMDEC + 1
      IF (ITER.GT.0) THEN
C     PRINT 557,ITER,RMIN,I,J,XC,YC,N
C 557 FORMAT (' *** STABILITY PROBLEM ***',5X,
C    1 'ITER =',I3,3X,'RMIN =',F10.1,3X,' I =',I3,3X,' J =',I3,
C    2 3X,'XC =',F10.2,3X,'YC =',F10.2,3X,'N =',I4)
      IF (ITER.GT.10) ITER=10
      NITER(ITER) = NITER(ITER) + 1
      ENDIF
      IF (COND.GT.CMX2) CMX2 = COND
C
C     PRINT *,' XZ(I) '
C     WRITE (6,603) (XZ(I),I=1,N)
C     PRINT *,' YZ(I) '
C     WRITE (6,603) (YZ(I),I=1,N)
C 603 FORMAT(9(1PE8.4))
C
      DO 100 II = IX1,IX2
      XE = XT(II)
      DO 100 JJ = IY1,IY2
      YE = YT(JJ)
      IF (LS(II,JJ).EQ.1)
     1 CALL ESTIM (XZ,YZ,DZ,N,XE,YE,ANAL(II,JJ),E(II,JJ))
C     PRINT *,' ANAL,ERR,N FROM ESTIM:',ANAL(II,JJ),
C    * E(II,JJ),N
  100 CONTINUE
   10 CONTINUE
      PRINT 101,NUMDEC
  101 FORMAT (' NUMBER OF SUCCESSFUL CALLS TO DECOMP =',I8)
      DO 103 I=1,10
      IF (NITER(I).LT.1) GO TO 103
      PRINT 102,NITER(I),I
  102 FORMAT (1X,I5,'  REGIONS REQUIRED',I5,
     1 '  ITERATIVE CALL(S) TO DECOMP FOR THINNING')
  103 CONTINUE
      CALL MAXMIN (ANAL,IMX,IMX,IMY,EMIN,EMAX)
      PRINT *,' MIN.,MAX. OF CORRECTION FIELD:',EMIN,EMAX
      CALL MAXMIN (E,IMX,IMX,IMY,EMIN,EMAX)
      PRINT *,' MIN., MAX., OF ERROR  MATRIX:',EMIN,EMAX
      PRINT *,' MIN. MAX. OF COND. #:',CMIN,CMX1,CMX2
      RETURN
      END
      SUBROUTINE CKDIST (RMIN,X,Y,D,SC,SU,ST,N,IFLG,JTYPE,INDX)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    CKDIST      DETERMINES DISTANCE BETWEEN OBSERVATIONS
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-18
C
C ABSTRACT: CHECKS THE DISTANCE BETWEEN OBSERVATIONS AND
C   THINS THE OBSERVATIONS IF THEY ARE WITHIN RMIN (IN KM).
C   THE THINNING IS DONE BY DROPPING OBSERVATIONS WITH THE
C   HIGHER ERROR SIGMA OR COMBINING OBSERVATIONS WITH THE
C   SAME SIGMA.  THE VARIABLES WERE SELECTED
C   BY SSTMAP TO LIE WITHIN THE BOX PLUS THE BORDER.  ALL
C   ARRAYS X, Y, D AND S ARE OF LENGTH N.  IF THE
C   ARRAYS ARE THINNED THE REDUCED VALUES ARE RETURNED IN
C   X, Y, D AND S AND THE VARIABLE N IS REDUCED TO GIVE
C   THE CORRECT LENGTH OF THE ARRAYS.
C
C PROGRAM HISTORY LOG:
C   90-12-18  RICHARD W. REYNOLDS
C
C USAGE:    CALL CKDIST (RMIN,X,Y,D,SC,SU,IFLG,JTYPE,INDX)
C
C   INPUT        ARGUMENT LIST:
C     RMIN     - MINIMUM DISTANCE ALLOWED BETWEEN OBSERVATIONS
C              - BEFORE THINNING
C
C   INPUT/OUTPUT ARGUMENT LIST:
C     X        - ARRAY OF LONGITUDE LOCATIONS OF DATA POINTS (DEG)
C              - VARIABLE XZ IN SUBROUTINE SSTMAP
C     Y        - ARRAY OF LATITUDE LOCATIONS OF DATA POINTS (DEG)
C              - VARIABLE YZ IN SUBROUTINE SSTMAP
C     D        - ARRAY OF DATA INCREMENTS (DEG C) AT LOCATION (X,Y)
C              - VARIABLE DZ IN SUBROUTINE SSTMAP
C     ST       - ARRAY OF ERROR TOTAL SIGMA (DEG C) AT LOCATION (X,Y)
C              - VARIABLE SZT IN SUBROUTINE SSTMAP
C     SU       - ARRAY OF ERROR UNCORR SIGMA (DEG C) AT LOCATION (X,Y)
C              - VARIABLE SZU IN SUBROUTINE SSTMAP
C     SC       - ARRAY OF ERROR CORR SIGMA (DEG C) AT LOCATION (X,Y)
C              - VARIABLE SZC IN SUBROUTINE SSTMAP
C     N        - NUMBER OF OBSERVATIONS
C     IFLG     - ARRAY DETERMINING WHAT HAPPENS TO OBSERVATION
C                 = 0 OBSERVATION KEPT
C                 = 1 OBSERVATION DROPPED
C                 = -1 OBSERVATION COMBINED
C     JTYPE    - ARRAY OF DATA TYPES (NTESAT,DAYSAT,BUOY,SHIP,ICE)
C              -                     (    =1,    =2,  =3,  =4, =5)
C              - EXTRACTED FROM ITYPE
C
C REMARKS:  IFLG (INTEGER)   IS A WORK ARRAY
C           DATA VALUES TO BE DROPPED AR INDICATED BY SETTING
C           D TO -999.9
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
      DIMENSION X(*),Y(*),D(*),SC(*),SU(*),ST(*)
      INTEGER   IFLG(*),JTYPE(*),INDX(*)
      DATA DTK,RPD/111.19,0.0174533/
      DO 100 I = 1,N-1
        IF (D(I).LE.-900.) GO TO 100
        DO 10 J = I,N
        IFLG(J) = 0
   10   CONTINUE
      DO 20 J = I+1,N
        IF (D(J).LE.-900.) GO TO 20
        YLT= 0.5*(Y(I)+Y(J))*RPD
        XD = (X(I)-X(J))*COS(YLT)*DTK
        YD = (Y(I)-Y(J))*DTK
        DG = SQRT(XD*XD+YD*YD)
C    DG IS DISTANCE BETWEEN DATA POINTS IN KM
        IF (DG.LT.RMIN) THEN
          CALL DECISN (SC,SU,ST,I,J,IFLG,JTYPE)
          IF (IFLG(I).EQ.1) THEN
            D(I) = -999.9
            GO TO 100
          ENDIF
        ENDIF
   20 CONTINUE
C     DO 178 L=1,N
C     PRINT *,' IN CKDIST',L,X(L),Y(L),D(L),ST(L),JTYPE(L),IFLG(L)
C 178 CONTINUE
      ICMB = 0
      DO 25 J = I+1,N
        IF (IFLG(J).EQ.1) D(J) = -999.9
        IF (IFLG(J).EQ.-1) THEN
          ICMB = ICMB + 1
C         IF (ICMB.GT.100) GO TO 50
          INDX(ICMB) = J
        ENDIF
   25 CONTINUE
C     GO TO 55
C  50 PRINT *,' ICMB = ',ICMB
C     ICMB = 100
   55 IF (ICMB.GT.0) CALL COMBIN (INDX,ICMB,D,SC,SU,ST,I,X,Y)
  100 CONTINUE
      KK = 0
      DO 30 I = 1,N
        IF (D(I).LE.-900.) GO TO 30
        KK = KK + 1
        D(KK) = D(I)
        ST(KK) = ST(I)
        SU(KK) = SU(I)
        SC(KK) = SC(I)
        X(KK) = X(I)
        Y(KK) = Y(I)
        JTYPE(KK) = JTYPE(I)
   30 CONTINUE
      IF (KK.NE.N) THEN
C       PRINT *,' DIST. CHECK:',N,KK
        N = KK
      ENDIF
      RETURN
      END
      SUBROUTINE DECISN (SC,SU,ST,I,J,IFLG,JTYPE)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    DECISN      COMPARES TWO DATA PTS AND FLAGS AS NEEDED
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-18
C
C ABSTRACT: CHECKS TWO DATA POINTS TO DETERMINE HOW THEY
C   SHOULD BE THINNED.  IF THE TYPE (JTYPE) OF OBSERVATION IS
C   THE SAME OR BOTH OBSERVATION ARE FROM SATELLITE
C   (JTYPE=1 OR =2), THE TWO OBSERVATIONS ARE FLAGGED
C   TO BE COMBINED.  ALSO IF THE TWO OBSERVATIONS HAVE THE
C   SAME SIGMA THEY ARE ALSO FLAGGED TO BE COMBINED.
C   OTHERWISE THE OBSERVATION WITH THE LOWER SIGMA IS FLAGGED
C   TO BE KEPT AND THE OBSERVATION WITH THE HIGHER SIGMA
C   IS FLAGGED TO BE DISCARDED.
C
C PROGRAM HISTORY LOG:
C   90-12-18  RICHARD W. REYNOLDS
C
C USAGE:    CALL DECISN (SC,SU,SC,I,J,IFLG,JTYPE)
C
C   INPUT        ARGUMENT LIST:
C     ST       - ARRAY OF TOTAL ERROR SIGMA (DEG C) AT LOCATION (X,Y)
C     SC       - ARRAY OF CORR. ERROR SIGMA (DEG C) AT LOCATION (X,Y)
C     SU       - ARRAY OF UNCORR. ERROR SIGMA (DEG C) AT LOCATION (X,Y)
C     I        - ITH LOCATION IN ARRAY OF DATA TO BE CONSIDERED
C     J        - JTH LOCATION IN ARRAY OF DATA TO BE CONSIDERED
C     JTYPE    - ARRAY OF DATA TYPES (NTESAT,DAYSAT,BUOY,SHIP,ICE)
C              -                     (    =1,    =2,  =3,  =4, =5)
C              - EXTRACTED FROM ITYPE
C
C   OUTPUT       ARGUMENT LIST:
C     IFLG     - ARRAY DETERMINING WHAT HAPPENS TO OBSERVATION
C              - IFLG = 0  KEEP OBSERVATION
C              - IFLG = 1  DROP OBSERVATION
C              - IFLG = -1 COMBINE OBSERVAT
C
C REMARKS:  IFLG (INTEGER)   IS INITIALLY SET TO 0 IN SUBROUTINE
C           CKDIST
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
      DIMENSION SC(*),SU(*),ST(*)
      INTEGER   IFLG(*),JTYPE(*)
      SI = ST(I)
      SJ = ST(J)
      IP = JTYPE(I)
      JP = JTYPE(J)
      IF (IP.EQ.JP) GO TO 10
C     IF (IP.LE.2.AND.JP.LE.2) GO TO 10
      IF (SI.EQ.SJ) GO TO 10
      IF (SI.LT.SJ) THEN
      IFLG(J) = 1
      ELSE
      IFLG(I) = 1
      IFLG(J) = 0
      ENDIF
      RETURN
   10 IFLG(J) = -1
      RETURN
      END
      SUBROUTINE COMBIN (INDX,N,D,SC,SU,ST,I,X,Y)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    COMBIN      COMBINES FLAGGED OBSERVATION TO ONE OB
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-18
C
C ABSTRACT: COMBINES FLAGGED OBSERVATIONS INTO ONE
C   OBSERVATION.  THE NEW OBSERVATION (D(I),S(I),X(I),Y(I) )
C   IS THE AVERAGE OF THE ALL THE VALUES TO BE COMBINED IN D,
C   SC,SU,SC, X, AND Y RESPECTIVELY. THE OLD ITH VALUE IS THUS
C   REPLACED.  ALL OTHER OLD VALUES WILL BE DELETED IN CHDIST.
C
C PROGRAM HISTORY LOG:
C   90-12-18  RICHARD W. REYNOLDS
C
C USAGE:    CALL COMBIN (INDX,N,D,SC,SU,ST,I,X,Y)
C
C   INPUT        ARGUMENT LIST:
C     INDX     - INDEX OF LOCATIONS IN ARRAYS (D,S,X, AND Y) TO BE
C              - COMBINED WITH D(I), S(I), X(I) AND Y(I).
C              - VALUES IN THESE LOCATIONS WILL BE DELETED.
C     I        - LOCATION OF NEW COMBINED OBSERVATIONS
C
C   INPUT/OUTPUT ARGUMENT LIST:
C     X        - ARRAY OF LONGITUDE LOCATIONS OF DATA POINTS (DEG)
C              - VARIABLE XZ IN SUBROUTINE SSTMAP
C     Y        - ARRAY OF LATITUDE LOCATIONS OF DATA POINTS (DEG)
C              - VARIABLE YZ IN SUBROUTINE SSTMAP
C     D        - ARRAY OF DATA INCREMENTS (DEG C) AT LOCATION (X,Y)
C              - VARIABLE DZ IN SUBROUTINE SSTMAP
C     ST       - ARRAY OF TOTAL ERROR SIGMA (DEG C) AT LOCATION (X,Y)
C              - VARIABLE SZT IN SUBROUTINE SSTMAP
C     SU       - ARRAY OF UNCORR ERROR SIGMA (DEG C) AT LOCATION (X,Y)
C              - VARIABLE SZU IN SUBROUTINE SSTMAP
C     SC       - ARRAY OF CORR ERROR SIGMA (DEG C) AT LOCATION (X,Y)
C              - VARIABLE SZC IN SUBROUTINE SSTMAP
C     N        - NUMBER OF OBSERVATIONS
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
      DIMENSION D(*),SC(*),SU(*),ST(*),X(*),Y(*)
      INTEGER   INDX(*)
C - WE NOW ASSIGN COMBINED VALUES TO THEIR MEAN LOCATIONS
      DENM = FLOAT(N+1)
      SIGC = 0.
      SIGU = 0.
      SIGT = 0.
      DIF = 0.
      XX = 0.
      YY = 0.
      DO 10 J = 1,N
        JJ = INDX(J)
        SIGC = SIGC + SC(JJ)**2
        SIGU = SIGU + SU(JJ)**2
        SIGT = SIGT + ST(JJ)**2
        DIF = DIF + D(JJ)
        XX = XX + X(JJ)
        YY = YY + Y(JJ)
        D(JJ) = -999.9
  10  CONTINUE
      SIGC = SIGC + SC(I)**2
      SIGU = SIGU + SU(I)**2
      SIGT = SIGT + ST(I)**2
      DIF = DIF + D(I)
      XX = XX + X(I)
      YY = YY + Y(I)
      SC(I) = SQRT(SIGC/DENM)
      SU(I) = SQRT(SIGU/DENM)
      ST(I) = SQRT(SIGT/DENM)
      D(I) = DIF/DENM
      X(I) = XX/DENM
      Y(I) = YY/DENM
      RETURN
      END
      SUBROUTINE DECOMP (X,Y,SC,SU,N,JTYPE,COND)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    DECOMP      FORMS ARRAY "A" IN LINEAR EQUATION
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-18
C
C ABSTRACT: THIS ROUTINE AND ESTIM SOLVES THE LINEAR SYSTEM OF
C   EQUATIONS WHICH CAN BE WRITTEN AS "A"*"X"="B" WHERE
C   "A" IS A TWO DIMENSIONAL ARRAY OF COEFFICIENTS.
C   "B" AND "X" ARE ONE DIMENSIONAL ARRAYS OF COEFFICIENTS.
C   DECOMP FORMS "A" IN THE ARRAY A.  DECOMP CALLS THE
C   IMSL ROUTINE  L2CDS TO COMPUTE THE CHOLESKY FACTORIZATION
C   OR DECOMPOSITION OF "A".
C   THIS RESULT IS RETURNED IN "A" OVERWRITING IT.
C   ESTIM COMPUTES "B" AND
C   USES THE DECOMPOSED ARRAY "A" TO COMPUTE THE UNKNOWN "X".
C   SEE SUBROUTINE SSTMAP FOR MORE DETAILS.
C
C PROGRAM HISTORY LOG:
C   90-12-18  RICHARD W. REYNOLDS
C
C USAGE:    CALL DECOMP (X,Y,SC,SU,N,GSIG,JTYPE,COND)
C
C   INPUT        ARGUMENT LIST:
C     X        - ARRAY OF LONGITUDE LOCATIONS OF DATA POINTS (DEG)
C              - VARIABLE XZ IN SUBROUTINE SSTMAP
C     Y        - ARRAY OF LATITUDE LOCATIONS OF DATA POINTS (DEG)
C              - VARIABLE YZ IN SUBROUTINE SSTMAP
C     SC       - ARRAY OF CORR. ERROR SIGMA (DEG C) AT LOCATION (X,Y)
C              - VARIABLE SZC IN SUBROUTINE SSTMAP
C     SU       - ARRAY OF UNCORR. ERROR SIGMA (DEG C) AT LOCATION (X,Y)
C              - VARIABLE SZU IN SUBROUTINE SSTMAP
C     N        - NUMBER OF OBSERVATIONS
C     JTYPE    - ARRAY OF DATA TYPES (NTESAT,DAYSAT,BUOY,SHIP,ICE)
C              -                     (    =1,    =2,  =3,  =4, =5)
C
C     PARAMETERS:
C     NMX      - SIZE OF "A" ARRAY; MUST BE THE SAME AS
C              - IMAX IN SUBROUTINE SSTMAX AND NMX IN ESTIM
C
C   OUTPUT       ARGUMENT LIST:
C     COND     - ESTIMATE OF THE CONDITION CODE OF "A"
C
C COMMON BLOCK : SCLRSQR
C     SCX2     -  SQUARE OF LONGITUDE SCALE
C     SCY2     -  SQUARE OF LATITUDE SCALE
C              - SCX2,SCY2 - THE UNITS ARE KM**2
C              - DTK = 111.19 (CONVERTS DEG TO KM)
C COMMON BLOCK : COMWORK -- A WORK ARRAY
C     A        - "A" ARRAY BEFORE CALLING  L2CDS
C              - DECOMPOSED ARRAY "A" AFTER CALLING  L2CDS
C     B        - WORK ARRAY
C
C REMARKS:
C           SEE IMSL ROUTINE LFCDS/LF2DS (MATH LIBRARY VOL 1)
C           FOR DESCRIPTION
C           NOTE: COMWORK IS WORK COMMON: ALSO NEEDED FOR ESTIM
C
C     LIBRARY:
C       IMSL   -   L2CDS (VERSION 10)
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
      PARAMETER (NMX=200)
C
C
      COMMON /SCLSQR/ SCX2,SCY2
      DIMENSION X(N),Y(N),SC(N),SU(N)
      INTEGER   JTYPE(*)
      REAL   A(NMX,NMX),DG,EXPD,EXPG,RC,B(NMX)
      COMMON /COMWRK/ A,B
      DATA DTK,RPD/111.19,0.0174533/
C
      IF (N.GT.NMX) THEN
        PRINT *,' WARNING, N.GT.NMX, DATA TRUNCATED IN DECOMP'
        N = NMX
      ENDIF
C     PRINT *,' IN DECOMP ISKC=',SKC
C
      DO 10 I = 1,N
        DO 20 J = 1,I
          YLT= 0.5*(Y(I)+Y(J))*RPD
          XD = (X(I)-X(J))*COS(YLT)*DTK
          YD = (Y(I)-Y(J))*DTK
          DG = XD*XD/SCX2+YD*YD/SCY2
          IF (DG.GT.16.  ) THEN
            EXPG = 0.
          ELSE
            EXPG =  EXP(-DG)
          ENDIF
C
          IF (I.EQ.J) THEN
            A(I,J) = EXPG + SC(I)*SC(J) + SU(I)*SU(J)
          ELSE
            EXPD = 0.
            IF (JTYPE(I).EQ.JTYPE(J)) EXPD = EXPG
            A(I,J) = EXPG + SC(I)*SC(J)*EXPD
            A(J,I) = A(I,J)
          ENDIF
C           IF (I.EQ.J) PRINT *,' DIAGONAL,I,J,A:',I,J,A(J,I)
C           IF (I.NE.J) PRINT *,' OFF DIAGONAL,I,J,A:',I,J,A(J,I)
  20    CONTINUE
  10  CONTINUE
C     PRINT * ,' N = ',N
C     PRINT * ,' A'
C     PRINT * ,A
C
*     CALL L2CDS (N,A,NMX,A,NMX,RC,B)
      call spoco (a,nmx,n,rc,b,info)
      if (info.ne.0) rc = 1.0e-10
c
 
      COND = 1./RC
*      PRINT * ,' CONDITION # FOR A = ',COND,'     info =',info
      RETURN
      END
      SUBROUTINE ESTIM (X,Y,T,N,XE,YE,TE,ERR)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    ESTIM       FORMS "B" AND FINDS "X" IN LINEAR EQN
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-18
C
C ABSTRACT: THIS ROUTINE AND DECOMP SOLVES THE LINEAR SYSTEM OF
C   EQUATIONS WHICH CAN BE WRITTEN AS "A"*"X"="B" WHERE
C   "A" IS A TWO DIMENSIONAL ARRAY OF COEFFICIENTS.
C   "B" AND "X" ARE ONE DIMENSIONAL ARRAY OF COEFFICIENTS.
C   ESTIM FORMS "B" ARRAY.  ESTIM CALLS THE
C   IMSL ROUTINE  LFSDS TO COMPUTE UNKNOWN "X" FROM "B" AND
C   THE DECOMPOSED "A". ESTIM THEN COMPUTES
C   TE (THE OI SST ANALYSIS INCREMENT) AND ERR (THE OI ERROR)
C   ESTIMATE AT GRID POINT XE AND YE.
C   NOTE: THE ARRAY NAME COEF IS USED FOR THE UNKNOWN "X".
C
C PROGRAM HISTORY LOG:
C   90-12-18  RICHARD W. REYNOLDS
C
C USAGE:    CALL ESTIM (X,Y,T,N,XE,YE,TE,ERR)
C
C   INPUT        ARGUMENT LIST:
C     X        - ARRAY OF LONGITUDE LOCATIONS OF DATA POINTS (DEG)
C              - VARIABLE XZ IN SUBROUTINE SSTMAP
C     Y        - ARRAY OF LATITUDE LOCATIONS OF DATA POINTS (DEG)
C              - VARIABLE YZ IN SUBROUTINE SSTMAP
C     T        - ARRAY OF DATA INCREMENTS AT LOCATION (X,Y)
C              - VARIABLE DZ IN SUBROUTINE SSTMAP
C     N        - NUMBER OF OBSERVATIONS
C     XE       - LONGITUDE OF GRID POINT ON ANALYSIS GRID FOR
C              - OI SOLUTION (MUST LIE WITHIN BOX DEFINED IN SSTMAP
C              - AND USED IN DECOMP
C     YE       - LATITUDE OF GRID POINT ON ANALYSIS GRID FOR
C              - OI SOLUTION (MUST LIE WITHIN BOX DEFINED IN SSTMAP
C              - AND USED IN DECOMP
C
C     PARAMETERS:
C     NMX      - SIZE OF "A" ARRAY; MUST BE THE SAME AS
C              - IMAX IN SUBROUTINE SSTMAX AND NMX IN DECOMP
C
C   OUTPUT       ARGUMENT LIST:
C     TE      -  OI SST ANALYSIS INCREMENT (DEG C)
C             -  AT LOCATION (XE,YE)
C     ERR     -  OI SST ANALYSIS ERROR (RANGE 0 TO 1)
C             -  AT LOCATION (XE,YE)
C
C COMMON BLOCK : SCLRSQR
C     SCX2     -  SQUARE OF LONGITUDE GUESS SCALE
C     SCY2     -  SQUARE OF LATITUDE GUESS SCALE
C              - SCX2,SCY2 - THE UNITS ARE KM**2
C              - DTK = 111.19 (CONVERTS DEG TO KM)
C COMMON BLOCK :  COMWORK -- WORK ARRAY
C     A        - DECOMPOSED "A" AFTER  L2CDS
C     B        - "B" ARRAY
C
C REMARKS:
C           SEE IMSL ROUTINE LFSDS       (MATH LIBRARY VOL 1)
C           FOR DESCRIPTION
C           NOTE:  COEF IS THE UNKNOWN "X" ARRAY
C           RETURNED BY CALLING LFSDS.
C           NOTE: COMWORK IS WORK COMMON: ALSO NEEDED FOR DECOMP
C
C     LIBRARY:
C       IMSL   -  LFSDS (VERSION 10)
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
      PARAMETER (NMX=200)
      REAL   A(NMX,NMX),B(NMX),COEF(NMX),D,SM1,CTS
      COMMON /COMWRK/ A,B
      COMMON /SCLSQR/ SCX2,SCY2
      DIMENSION X(*),Y(*),T(*)
      DATA DTK,RPD/111.19,0.0174533/
C
      IF (N.GT.NMX) THEN
        PRINT *,' DIMENSION IS INCORRECTLY SET, RESET PARAMETERS'
        N = NMX
      ENDIF
C
C     PRINT *,' N =',N
C  FORM RIGHT HAND SIDE VECTOR OF THE LINEAR SYSTEM.
      DO 30 I=1,N
        YLT=0.5*(YE+Y(I))*RPD
        XD=(XE-X(I))*COS(YLT)*DTK
        YD=(YE-Y(I))*DTK
        D = (XD*XD/SCX2+YD*YD/SCY2)
        IF (D.GT.16.  ) THEN
          B(I) = 0.
          coef(i) = b(i)
          GO TO 30
        ENDIF
        B(I) =  EXP(-D)
        coef(i) = b(i)
 30   CONTINUE
C     PRINT *,' XE ',XE
C     PRINT *,' YE ',YE
C     PRINT *,' X(I) '
C     WRITE (6,601) (X(I),I=1,N)
C     PRINT *,' Y(I) '
C     WRITE (6,601) (Y(I),I=1,N)
C     PRINT *,' B(I)'
C     WRITE (6,601) (B(I),I=1,N)
C 601 FORMAT(9(1PE8.1))
C
C  SOLVE FOR LINEAR SYSTEMS WITH PIVOTING.
C  SM1 = TR(R.H.S.)*INV(COV.MATRIX)*(R.H.S) = TR(R.H.S)*COEFF.
C
C     PRINT *,' COEF(I) '
C     WRITE (6,601) (COEF(I),I=1,N)
C
*     CALL  LFSDS (N,A,NMX,B,COEF)
      call sposl (a,nmx,n,coef)

      SM1=0.
      DO 40 I=1,N
        SM1=SM1+COEF(I)*B(I)
 40   CONTINUE
C
C  COMPUTE ANALYZED CORRECTIONS AND  ESTIMATE ANALYSIS ERROR:
C     ERR = 1.- TR(R.H.S)*INV(COV.MATRIX)*(R.H.S)
C
C     PRINT *,' SM1 =',SM1
C     PRINT *,' ANOMALY DATA:'
C     WRITE (6,601) (T(I),I=1,N)
      CTS = 0.
      DO 60 I=1,N
        D = T(I)
        CTS = CTS + COEF(I)*D
 60   CONTINUE
      ERR = 1.   - SM1
      TE  = CTS
      RETURN
      END
      SUBROUTINE PUTOI (IUN,ANAL,E,IMX,IMY,
     1 IYRST,IMST,IDST,IYREND,IMEND,IDEND,NDAYS,INDEX,cice)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    PUTOI       WRITES OUT DATES, OI ANALYSIS AND ERROR
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-18
C
C ABSTRACT: WRITES OUT THE DATES, OI ANALYSIS AND OI ERROR
C
C PROGRAM HISTORY LOG:
C   90-12-18  RICHARD W. REYNOLDS
C
C USAGE:    CALL PUTOI (IUN,ANAL,E,IMX,IMY,
C          1 IYRST,IMST,IDST,IYREND,IMEND,IDEND,NDAYS,INDEX)
C
C   INPUT ARGUMENT LIST:
C     IUN      - FORTRAN OUTPUT UNIT NUMBER
C     ANAL     - ARRAY OF SST OI ANALYSIS MEAN FIELD (DEG C)
C     E        - ARRAY OF OI ERROR VALUES (RANGE 0 TO 1)
C     IMX      - FIRST DIMENSION OF ANAL AND E ARRAYS
C     IMY      - SECOND DIMENSION OF ANAL AND E ARRAYS
C     IYRST    - START YEAR (2 DIGITS) OF ANALYSIS
C     IMST     - START MONTH OF ANALYSIS
C     IDST     - START DAY OF ANALYSIS
C     IYREND   - END YEAR (2 DIGITS) OF ANALYSIS
C     IMEND    - END MONTH OF ANALYSIS
C     IDEND    - END DAY OF ANALYSIS
C     NDAYS    - NUMBER OF DAYS FOR ANALYSIS
C     INDEX    - OI VERSION NUMBER: HAS NO EFFECT ON RESULT
C
C   OUTPUT FILE:  DETERMINE BY IUN; SEE MAIN PROGRAM
C
C REMARKS:  THE OUTPUT ANALYSIS FILE HAS THE SAME ARGUMENT LIST
C           AS THE INPUT GUESS FILE
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
      DIMENSION ANAL(IMX,IMY),E(IMX,IMY)
      character*1 cice(360,180)
      if(iyrst.lt.100)then
        if(iyrst.lt.21)then
          iyrst=iyrst+2000
        else
          iyrst=iyrst+1900
        endif
        print*,'IYRST MODIFIED TO 4-DIGITS:',iyrst
      endif
      if(iyrend.lt.100)then
        if(iyrend.lt.21)then
          iyrend=iyrend+2000
        else
          iyrend=iyrend+1900
        endif
        print*,'IYREND MODIFIED TO 4-DIGITS:',iyrend
      endif

      PRINT 7,IYRST,IMST,IDST,IYREND,IMEND,IDEND,NDAYS,INDEX
    7 FORMAT (' IYRST = ',I4,3X,'IMST = ',I3,3X,'IDST = ',I3,3X,
     1 'IYREND = ',I4,3X,'IMEND = ',I3,3X,'IDEND = ',I3,3X,
     2 'NDAYS = ',I3,3X,'INDEX = ',I10,3X,'OI DATES OUTPUT')
      WRITE (IUN) IYRST,IMST,IDST,IYREND,IMEND,IDEND,NDAYS,INDEX
*     WRITE(IUN) ((ANAL(I,J),I = 1,IMX),J = 1,IMY)
*     WRITE(IUN) ((E(I,J),I = 1,IMX),J = 1,IMY)
      write(iun) ((anal(i,j),i=191,370),(anal(i,j),i=11,190),
     1      j = 1,imy)
      write(iun) ((e(i,j),i=191,370),(e(i,j),i=11,190),
     1      j = 1,imy)
      write(iun) ((cice(i,j),i=1,360),j=1,180)
      RETURN
      END
      SUBROUTINE MXMN2 (IA,ICT,AMIN,AMAX,SCALE)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    MAXMIN      FINDS MINIMUM AND MAXIMUM OF ARRAY A.
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-14
C
C ABSTRACT: FINDS THE MINIMUM AND THE MAXIMUM OF THE INTEGER
C   ARRAY A. THE INTEGER A ARRAY IS ASSUMED TO BE SCALED
C   BY MULTIPLYING BY SCALE
C
C PROGRAM HISTORY LOG:
C   90-12-14  RICHARD W. REYNOLDS
C
C USAGE:    CALL MXMN2 (IA,ICT,AMIN,AMAX,SCALE)
C
C   INPUT ARGUMENT LIST:
C     IA       - ARRAY TO SEARCH FOR MAX AND MIN
C     ICT      - LENGTH OF IA ARRAY
C
C   OUTPUT ARGUMENT LIST:
C     AMIN     - MINIMUM OF ARRAY A DIVIDED BY 100
C     AMAX     - MAXIMUM OF ARRAY A DIVIDED BY 100
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
      INTEGER   IA(ICT)
      MAX =-32767
      MIN = 32767
C
      DO 30 I=1,ICT
        IF (IA(I).LE.-32000) GO TO 30
        IF (IA(I).GE. 32000) GO TO 30
        IF (IA(I).LT.MIN) MIN = IA(I)
        IF (IA(I).GT.MAX) MAX = IA(I)
   30 CONTINUE
      AMAX = FLOAT(MAX)*SCALE
      AMIN = FLOAT(MIN)*SCALE
      RETURN
      END
      SUBROUTINE MAXMIN (A,IA,IMX,IMY,AMIN,AMAX)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    MAXMIN      FINDS MINIMUM AND MAXIMUM OF ARRAY A.
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-14
C
C ABSTRACT:  FINDS THE MINIMUM AND THE MAXIMUM OF THE REAL ARRAY A.
C
C PROGRAM HISTORY LOG:
C   90-12-14  RICHARD W. REYNOLDS
C
C USAGE:    CALL MAXMIN (A,IA,IMX,IMY,AMIN,AMAX)
C
C   INPUT ARGUMENT LIST:
C     A        - ARRAY TO SEARCH FOR MAX AND MIN
C     IA       - ACTUAL FIRST DIMENSION IN CALLING PROGRAM
C     IMX      - FIRST DIMENSION OF A ARRAY
C     IMY      - SECOND DIMENSION OF A ARRAY
C
C   OUTPUT ARGUMENT LIST:
C     AMIN     - MINIMUM OF ARRAY A
C     AMAX     - MAXIMUM OF ARRAY A
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
      DIMENSION A(IA,IMY)
      AMAX =-1.E30
      AMIN = 1.E30
C
      DO 30 J=1,IMY
      DO 30 I=1,IMX
        IF (A(I,J).LE.-999.) GO TO 30
        IF (A(I,J).LT.AMIN) AMIN = A(I,J)
        IF (A(I,J).GT.AMAX) AMAX = A(I,J)
   30 CONTINUE
      RETURN
      END
      SUBROUTINE STAND (N,D,AMEAN,STDEV)
C  $  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    STAND       FINDS MEAN AND ST. DEV. OF DATA INCR.
C   PRGMMR: R. W. REYNOLDS   ORG: W/NMC52    DATE: 90-12-14
C
C ABSTRACT: FINDS THE MEAN AND THE STANDARD DEVIATION OF THE
C   DATA INCREMENT ARRAY D.
C
C PROGRAM HISTORY LOG:
C   90-12-14  RICHARD W. REYNOLDS
C
C USAGE:    CALL  STAND (N,D,AMEAN,STDEV)
C
C   INPUT ARGUMENT LIST:
C     N        - LENGTH OF D ARRAY
C     D        - ARRAY OF DATA INCREMENTS
C              - VALUES LESS THAN -900. ARE NOT USED IN ANALYSIS
C              - AND ARE IGNORED IN THE COMPUTATION
C
C   OUTPUT ARGUMENT LIST:
C     AMEAN    - MEAN OF ARRAY D (IN DEG C)
C     AMAX     - STANDARD DEVIATION OF ARRAY D (IN DEG C)
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  NAS
C
C  $
      DIMENSION D(*)
      IF (N.EQ.0) RETURN
      TMP1 = 0.
      TMP2 = 0.
      M = 0
      DO 10 I=1,N
        IF (D(I).LE.-900.) GO TO 10
        M = M + 1
        TMP1 = TMP1 + D(I)
        TMP2 = TMP2 + D(I)**2
   10 CONTINUE
      IF (M.LT.1) M = 1
      AMEAN = TMP1/FLOAT(M)
      VAR   = TMP2/FLOAT(M) - AMEAN**2
      STDEV = SQRT(AMAX1(0.,VAR))
      RETURN
      END
      include 'w3date.f'
      include 'weaver.f'
      include 'sposl.f'
      include 'spoco.f'
      
