#!/bin/sh
#***p* PROJECT/ERSST
#
# NAME 
#  ersstv3b_oper_situ_only_col.sh
#
# LOCATION
#  /raid2/ersst/ftn/ersstv3b_oper_situ_only_col.0.sh   
# 
# WARNING
#  -DO NOT USE except for operational run on the 3rd of every month
#  -assumes previous month's run was fine, so CHECK previous month's output first
#  -while ERSST is the focus, the Land and Merged products are generated or written over
#  -for non-operational use, CHANGE DESTINATION DIRECTORIES
#
# PURPOSE
#  To generate the analyzed Extended Reconstruction Sea Surface Temperature (ERSST) on a 2 deg 
#  grid from in situ data (ship and buoy, NOT satellite: v3b uses in situ only),
#  and transfer to distribution directories.
#
# DESCRIPTION
# 	This is the main script that launches a series of fortran programs for computing 
# ERSST for a specific month and year (determined from the current machine date). 
# The operational runs will affect values in recent past years due the long-term averaging
# Also, the program uses output from a one-time climatological run (1880 
# to around 1985).  Most programs write output for all the years sequentially in one binary
# file. However, depending on the program, the month processed may just be added to the
# the pre-exisitng file, or the entire file may be rewritten from 1985 onward. 
# The processing is a s follows:
# First, in situ data (ship and buoy) is ftp'd from source locations.  Adjustments
# are made for distance of point obs from grid center, difference in dependability of ship 
# and buoy data, etc. and other quality checks are made. The data is placed on a 2-deg grid, 
# and anomalies are computed. Adjustments are also made for sea ice presence. The
# sea ice data comes from the daily OISST analysis. Statistical analysis is done in 2 steps: 
#  1)The decadal or low frequency component is determined from the anomalies and then the 
#    residuals are computed
#  2)The high frequency analysis is performed on the residuals.  
# The ERSST is then computed from the sum of the two components and error variance is estimated.
# 	After the ERSST computation, other programs are run to update related products
# (land and merged land-ocean SST) that  use ERSST.  These other products are continually
# updated on a different schedule, external to this script. Areal averages are computed for the  
# Climate Monitoring group, and plots are made to check the ERSST output.  
# Comparisons with v2 and the v3 (satellite) are also made, that are produced separately.
#
# AUTHOR
#  Chunying Liu
#
# CREATION DATE
#  04/01/2008 
#
# COPYRIGHT
#  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
#  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
#  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
#  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
#  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
#  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
#  TO USERS.
#
# MODIFICATION HISTORY
#  04/30/2008 - Chunying modified from /raid2/ersst/ftn/ersstv3b_oper_situ_only_col.sh 
#                       to remove satellite data
#  08/28/2009 - Viva Banzon added comments
#  08/17/2010 - Viva Banzon removed Land and merged comments after Liu removed related codes
#               The conditions for writing ascii for previous decade were put in but left commented
#               since that was not in the original script
#
# INPUTS
#  buoyship_quarter/nqyyyymm.Z - ftp ship and buoy data
#
#  Outputs from the Fortran programs are passed to other Fortran programs
#  with each program building/modifying the data for the next program
#
# OUTPUTS
#  Updated ERSST integer and data files
#	ersst/datat/ersst-v3b/situ/ersst.v3b.yr1.yr2.asc
#	ersst/datat/ersst-v3b/situ/ersst.esd.v3b.yr1.yr2.asc
#  NetCDF formated ERSST file
#	ersst/data/netcdf-v3b/situ/ersst.yyyymm.nc
#
# PARAMETERS 
#  smult=4 -  standard deviation multiplier (range 2-6) for QC of in situ data
#
# VARIABLES
#  	$chyr 	= two digit year
#  	$chmon = two digit month
#
# SUBPROGRAMS
#   	monice1d-med-oper.f
#   	gtsqc.situ.v3b.f
#   	ice1t2.f
#   	ssta.merg.situ.v3b.f
#   	lfsst.situ.v3b.f
#   	hfsst.situ.v3b.f
#   	sst2d.situ.v3b.f
#   	err.norm.map2.upd.situ.v3b.f
#   	dati2.upd.situ.v3b.f
#   	ersst_netcdf.situ.v3b.f
# LIBRARY
#      netcdf bin library at: /usr/local/netcdf-3.6.1/bin
#      netcdf modules at:/usr/local/netcdf-3.6.1/include
#      date function at: /lib/w3lib
#
# TODO
#  -ftp part should be separated out
#
# LANGUAGE
#  Linux Bourne shell script
#
#****
######################################################################################
#set -x
#### Set paths ################################################################

. ./ersst_config.sh
#. /path/to/your/ersst_config.sh


echo "Monthly ERSST V3 in situ only Run"

#### Find previous month, abbreviation, and 2- and 4-digit year ###############
curryear=`date '+%Y'`
currmon=`date '+%m'`
today=`date '+%d'`
year=`date --date="last month" +%Y` 
chyr=`date --date="last month" +%y`
month=`date --date="last month" +%m`

chmon=$month
ccmon=`date --date="last month " +%b`
ccmon=`echo $ccmon|tr [a-z] [A-Z]`

echo "Updated on: " `date +%Y,%m,%d`
echo "The processing month is: $year $month ($ccmon)"

#### Find out how many days in previous month #################################
ndays=`cal $month $year | egrep -v '[A-Za-z]' | wc -w`
echo $ndays 'days in' $year $month

#### Link to OI ice files#####################################################
###Could be modified to year by year structure###############################
#find /raid2/sstday -wholename "*med/icecon.*" -exec ln -f -s {} $ersst_DAT_DIR/input/ice/ncep/  \;
find ${ersst_ice_dir} -wholename "*med/icecon.*" -exec ln -f -s {} $ersst_DAT_DIR/input/ice/ncep/  \;


#### FTP the data #############################################################
#. ./ersst_ftp.sh
$ersst_SCRP_DIR/ersst_ftp.sh

#### Compile codes ###########################################################
####the compile also runs the config script

#$ersst_SCRP_DIR/ersst_compile.sh

#####logname will be composed of date and time###############################
########the logpath is defined after config is run##########################

logdat=`date '+%Y-%m-%d'`
loghms=`date '+%T'`
lognam=$ersst_LOG_DIR/$logdat-$loghms.log
echo $logname
### Function to check stopcodes and terminate if necessary ###################
function checkStopcode {
  echo "{$currFile}: call exit $stopcode"

  if [ $stopcode -eq 10 ]
  then
    echo "YELLOW-WARNING: ERROR IN {$currFile} $stopcode"
  fi

  if [ $stopcode -eq 100 ]
  then
    exit
  fi

  if [ $stopcode -ne 1 -a $stopcode -ne 10 -a $stopcode -ne 100 ]
  then
    echo "RED-STOP: {$currFile} data error"
    exit
  fi
}

#000000000000000000000000000000000000000000000000000000000000000000000000000000
#### convert 1/2 degree monthly ice to 1 degree monthly #######################
currFile=monice1d-med-oper.v3b.x
$ersst_BIN_DIR/$currFile -Wl,-T << ieof >>$lognam 
$year
1
1
$year
$month
$ndays
ieof

stopcode=$?  

checkStopcode $currFile $stopcode

#111111111111111111111111111111111111111111111111111111111111111111111111111111
#### run the GTS code #########################################################
# define the standard deviation multiplier (range 2-6): smult
smult=4.0
echo "Standard deviation multiplier: $smult"
currFile=gtsqc.situ.v3b.x
$ersst_BIN_DIR/$currFile -Wl,-T << ieof  >>$lognam
$year
$month
$smult
ieof

stopcode=$?  

checkStopcode $currFile $stopcode

###############################################################################
currFile=ice1t2.x
$ersst_BIN_DIR/$currFile -Wl,-T << ieof  >>$lognam
$year
$year
$month
ieof

stopcode=$?  

checkStopcode $currFile $stopcode

###############################################################################
currFile=ssta.merg.situ.v3b.x

$ersst_BIN_DIR/$currFile -Wl,-T << ieof  >>$lognam
$year
$month
ieof

stopcode=$?  

checkStopcode $currFile $stopcode

###############################################################################
currFile=lfsst.situ.v3b.x
$ersst_BIN_DIR/$currFile -Wl,-T << ieof  >>$lognam
$year
$month
ieof

stopcode=$?  

checkStopcode $currFile $stopcode

###############################################################################
currFile=hfsst.situ.v3b.x
$ersst_BIN_DIR/$currFile -Wl,-T << ieof  >>$lognam
$year
$month
ieof

stopcode=$?  

checkStopcode $currFile $stopcode

###############################################################################
currFile=sst2d.situ.v3b.x
$ersst_BIN_DIR/$currFile -Wl,-T << ieof  >>$lognam
$year
$month
ieof

stopcode=$?  

checkStopcode $currFile $stopcode

###############################################################################
currFile=err.norm.map2.upd.situ.v3b.x
$ersst_BIN_DIR/$currFile -Wl,-T << ieof  >>$lognam
$year
$month
ieof

stopcode=$?  

checkStopcode $currFile $stopcode

###############################################################################
###Original operational script updates only the decadal ascii file that contains
###the year being processed.
currFile=dati2.upd.situ.v3b.x  
$ersst_BIN_DIR/$currFile -Wl,-T << ieof  >>$lognam 
$year
$month
ieof

stopcode=$?  

checkStopcode $currFile $stopcode

years=`expr currdec*10`
yeare=`expr $years+9`

###############################################################################

###These portion should be used  to update ascii files of prev decade
###if any of the past 7 years fall within the previous decade
currdec=$year/10
pastyrs=`expr $year-7`
pastdec=$pastyrs/10
echo $currdec $pastdec
yearss=`expr pastdec*10`
yearee=`expr $yearss+9`
#if [ $currdec -gt $pastdec ]; then
#  currFile=dati2.upd.situ.v3b.x
#  $ersst_BIN_DIR/$currFile -Wl,-T << ieof
#  $yearss
#  12
#  ieof
#  stopcode=$?  
#  checkStopcode $currFile $stopcode
#fi
###############################################################################


# Generate NetCDF files for the recent 8 years 
# lfsst.f 15-year LF periods. back 7 years and after 7 years
# So the last 7 years ERSST keep updated due to more data avalable.
# e.g. in 2007, when run 2005, only 2006 in the future available,
# But when rerun in 2008, there will be 2006 and 2007 data available.

iyrs=`expr $year - 7`
imos=01
echo $iyrs

TMs=$iyrs$imos
TMe=$year$chmon

echo $TMs
echo $TMe

while [ "$TMs" -le "$TMe" ]; do
cat << netcdfEOF > netcdf.parm
$iyrs 
$imos
$curryear $currmon $today
netcdfEOF
cat netcdf.parm

currFile=ersst_netcdf.situ.v3b.x
$ersst_BIN_DIR/$currFile -Wl,-T <netcdf.parm >>$lognam
stopcode=$?  

checkStopcode $currFile $stopcode 

#find next month
imos=`expr $imos + 1`
if [ $imos -le 9 ]; then
  imos='0'$imos
fi 
if [ $imos -eq 13 ]; then
  imos=01
  iyrs=`expr $iyrs + 1`
fi

TMs=$iyrs$imos
echo $TMs

done

echo 'finish update ERSST of' $year $month

exit

######################################################################
####remove old ascii files before zipping new output
if [ -f  $ersst_DATA_DIR/output/ascii/ersst.v3b.$years.$yeare.asc.gz ]
then 
rm  $ersst_DATA_DIR/output/ascii/ersst.v3b.$years.$yeare.asc.gz
fi
 
gzip $ersst_DATA_DIR/output/ascii/ersst.v3b.$years.$yeare.asc
gzip $ersst_DATA_DIR/output/ascii/ersst.esd.v3b.$years.$yeare.asc

#uncomment if ascii files from prev 7 years need to be updated
# original script did  not do this
#if [ $currdec -gt $pastdec ]; then
#if [ -f  $ersst_DATA_DIR/situ/ersst.esd.v3b.$yearss.$yearee.asc.gz ]
#then 
#rm $ersst_DATA_DIR/situ/ersst.esd.v3b.$yearss.$yearee.asc.gz
#fi 
#gzip $ersst_DATA_DIR/ersst.v3b.$yearss.$yearee.asc
#gzip $ersst_DATA_DIR/situ/ersst.esd.v3b.$yearss.$yearee.asc
#fi

#########################################################################
###send out mail messages
rm mail_file
echo "ERSST V3b data updated for "$chmon$year > mail_file
#mail -s ersst.situ.v3b.$years.$yeare_updated Yan.Xue@noaa.gov,Tom.Smith@noaa.gov,Glenn.Rutledge@noaa.gov,Danny.Brinegar@noaa.gov,Dan.Swank@noaa.gov,Chunying.Liu@noaa.gov,David.Levinson@noaa.gov,David.Wuertz@noaa.gov,Viva.Banzon@noaa.gov < mail_file
#mail -s ersst.situ.v3b.$years.$yeare_updated chunying.liu@noaa.gov,Richard.W.Reynolds@noaa.gov < mail_file


###########################################################################################

exit


