c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
c     7                                                               72
c
c@***h*  ERSST/gtsqc.situ.v3b.f
c*
c* NAME
c*  gtsqc.situ.v3b.f
c*
c* LOCATION
c*  $ersst_SRC_DIR/gtsqc.situ.v3b.f
c*
c* WARNING
c*  Change destination directories if not running operationally
c* 
c* PURPOSE
c*  To compute monthly SST averages on a 2 deg grid based on single point observations
c*  from ships and buoys
c*
c* DESCRIPTION
c*  Computes monthly SST averages on a 2 deg grid for ship
c*  and buoy data separately after a series of quality control checks. 
c*  Number of observations are also tallied on the grid and written out.
c*  In situ data is read per line (=1 SST value) and can be from ship, 
c*  buoy or other sources (not used).  For quality control of in situ 
c*  SSTs, the code is set up to use monthly ERSST and/or climatology, 
c*  but only the ERSST check is actually used. For the latest month,
c*  there is no ERSST so the SST check uses the previous months ERSST plus
c*  the climatological difference between current and previous month.
c*  Other QC checks skip data over land or ice, etc.
c*
c*  Notes: All processing is on the ERSST 2-deg grid
c*
c* AUTHOR
c*  Richard W. Reynolds
c*
c* CREATION DATE
c*  Jan 2008 - Richard W. Reynolds
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  08 Aug 2010  V. Banzon changed buoyship  path to $ersst_FTP_DIR instead of DAT_DIR
c*  03/03/2010 - C. Liu hardcoding
c*  08/28/2009 - V. Banzon added comments
c*  02/01/2008 - C. Liu modified from coadsqc.f
c*  Jan 2007  - Chunying Liu modified the code to run at NCDC 
c*
c* INPUTS
c* passed by script
c*     iyre ......... user-specified end year
c*     imoend ....... user-specified end month
c*     smult ........ user-specified standard deviation multiplier
c* In situ data as point SST values (coordinates provided)
c* program substitutes in yy,  mm
c*     unit 31 ...... $ersst_FTP_DIR/buoyship/nqyy2mm
c* 1 deg monthly avg SST climatology (1971-2000)COADS files
c*     unit 23 ...... $ersst_DAT_DIR/static/clim.71.00.gdat-fill contains averages
c* 1 deg monthly std dev SST climatology (1950-1970)COADS files
c*     unit 24 ...... $ersst_DAT_DIR/static/stdev1d-coads3-fill contains std deviations
c* 2 deg monthly ERSST  (1985-recent)
c*     unit 21 ...... $ersst_DAT_DIR/output/sst2d/sst2d.situ.YYYYMM.v3b.dat
c* 2 deg land mask
c*     unit 3 ....... $ersst_DAT_DIR/static/mask2d.dat
c*
c* OUTPUTS
c* program substitutes ciyear
c* 2 deg monthly average SST and data counts
c*     unit 51 ...... $ersst_DAT_DIR/inter/situ/ship.avg2.mon.ciyear.dat
c*     unit 52 ...... $ersst_DAT_DIR/inter/situ/buoy.avg2.mon.ciyear.dat
c* 2 deg monthly rejected SSTs based on climatology and reject counts
c*     unit 61 ...... $ersst_DAT_DIR/inter/situ/gts_climrej_ship.ciyear.dat
c*     unit 62 ...... $ersst_DAT_DIR/inter/situ/gts_climrej_buoy.ciyear.dat
c* 2 deg monthly rejected SSTs based on ERSST and reject counts
c*     unit 71 ...... $ersst_DAT_DIR/inter/situ/gts_ersstrej_ship.ciyear.dat
c*     unit 72 ...... $ersst_DAT_DIR/inter/situ/gts_ersstrej_buoy.ciyear.dat           
c*
c* PARAMETERS
c*	imx, imy 	= x and y dimension of 2 degree grid
c* 	imxp		= x dim of padded grid (to repeat first column at end)
c*	imx1,imy1	= x and y parameters of 1 degree grid
c*	iyrs		= start year for averaging
c*
c* VARIABLES
c*     xt, yt    	= 1 deg lat and lon vectors
c*     alon, alat	= 2 deg lat and lon vectors
c*     alonp		= 2 deg lon vector with first value repeated at end
c*     ersst		= gridded 2 deg ERSST data for month, year
c*     wk1, wk2   	= range-restrained 2 deg clim mean and sdev
c*     bigclm		= climatology avg on padded 2 deg grid
c*     bigsd		= climatology std dev on padded 2 deg grid
c*     mask		= 2 deg mask
c*     amask		= padded 2 deg mask
c*     cl1, sd1  	= climatology on original 1 deg grid
c*
c*     2 deg Grids where third dimension is month:
c*     clm 		=SST climatology
c*     stdev		=std dev of climatology
c*
c*     Grids where third dimension is data source (ship or buoy):
c*     dat		= avg in situ SST on 2 deg grid
c*     ct        	= counts of valid in situ SST used to compute avg
c*     rejclm		= 2 deg grid containing rejected SSTs (based on clim)
c*     ctrejc    	= 2 deg grid containing counts of clim-based rejects
c*     rejinc		= 2 deg grid containing rejected SSTs (based on ersst)
c*     ctreji    	= 2 deg grid containing counts of ersst-based rejects
c*   
c*     2-D Variables used for quality control of ship and buoy arrays
c*     are explained when they occur:
c*     ipos,iintp,irejc,ireji,irang,iok, izero, iland,isum,itot
c*     aland,rejc,reji,rang
c*
c* TODO
c* clean up
c*     Not used but declared:     ersst_tmp, tmp
c*     In program, not declared but not used either: i_sst999, i_globe
c*     maxmin subroutine also used in other programs so better to put in subdir and include
c*
c* SUBPROGRAMS
c*  intrpl
c*  maxmin
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
c
      parameter(imx=180,imy=89,imxp=imx+1,imx1=360,imy1=180,iyrs=1985)
      dimension alon(imx),alat(imy),alonp(imxp)
      dimension xt(imx1),yt(imy1)
      dimension ersst(imxp,imy),ersst_tmp(imxp,imy),tmp(imxp,imy)
      dimension wk1(imx,imy),wk2(imx,imy)
      dimension dat(imx,imy,2),ct(imx,imy,2)
      dimension rejclm(imx,imy,2),ctrejc(imx,imy,2)
      dimension rejinc(imx,imy,2),ctreji(imx,imy,2)
      dimension stdev(imx,imy,12),clm(imx,imy,12)
      dimension bigsd(imxp,imy),bigclm(imxp,imy)
      dimension cl1(imx1,imy1),sd1(imx1,imy1)
      dimension mask(imx,imy),amask(imxp,imy)
      dimension ipos(2),iintp(2),irejc(2),ireji(2),irang(2),iok(2)
      dimension izero(2),iland(2),isum(2),itot(2)
      dimension aland(2),rejc(2),reji(2),rang(2)
      character*2 cimonth,ciyear2
      character*4 ciyear,ctype(2)
      character*6 cid,date2,date
      character fm*11,st*3
      character*80 gts_in, name
      
      CHARACTER(LEN=60) :: Dir_data,Dir_ftp,Dir_out
      
      integer year, mon,day, IDID,year1
      data ctype /'ship','buoy'/
      data fm/'unformatted'/, st/'old'/

      print*,'Year month to process? '
      read *, iyre, imoend
      print*, 'The processing year, month :  ', iyre, imoend

      print *,'enter standard deviation multiplier (range 2-6): smult'
      read *,smult
      print 301,smult
  301 format ('smult =',f6.2)
      if (smult.lt.1.0.or.smult.gt.9.0) then
        print *,'smult out of range stop 650'
      endif
      
666   format(a80)
                 
c
      undef = -999.9
      
      CALL getenv("ersst_DAT_DIR", Dir_data)
c      print*, Dir_data
      CALL getenv("ersst_FTP_DIR", Dir_ftp)

c
      open(3,form=fm,file=
     & adjustl(adjustr(Dir_data)//'/input/static/mask2d.dat')) 

      read (3,err=601,end=601) mask
      do i=1,imx
        do j=1,imy
	  amask(i,j) = mask(i,j)
        enddo
      enddo
      do j=1,imy
	amask(imxp,j) = amask(1,j)
      enddo
c      write (63) ((amask(i,j),i=1,imx),j=1,imy)
      call maxmin (amask,imxp,imy,0.,'mask 2-d')      
c
c     read in coads clim. and standard deviation for 1950-79
c
      open (23,form=fm,file=
     & adjustl(adjustr(Dir_data)//'/input/static/clim.71.00.gdat-fill'))
     
      open (24,form=fm,file=
     & adjustl(adjustr(Dir_data)//'/input/static/stdev1d-coads3-fill'))

      do imo=1,12
        read (23,err=602,end=602) iyrrr,jmonrr,cl1
        read (24,err=602,end=602) iyr,jmon,sd1
c
c  convert to 2-degree
c      	
        do i=1,imx
          ip = i*2 - 1 
          im = ip - 1
          do j=1,imy
            jp = j*2 + 1
            jm = jp - 1
  	    topc = 0.
	    bot  = 0.
  	    tops = 0.
	    do i1=im,ip
	      if (i1.eq.0) then
	        ix1 = imx1
	      else
	        ix1 = i1
	      endif	
	      do j1=jm,jp
	         topc = topc + cl1(ix1,j1)
	         tops = tops + sd1(ix1,j1)
		 bot = bot + 1.
	      enddo
            enddo
	     ss = tops/bot
	     clm(i,j,imo) = topc/bot
	     if (ss.lt.1.0) ss = 1.0                                           
             if (ss.gt.4.5) ss = 4.5 
	     stdev(i,j,imo) = ss*smult
	     wk2(i,j) = ss*smult  
	     wk1(i,j) = clm(i,j,imo)  
          enddo
        enddo
        if (imo.eq.12) then
	  print *,'climatology base period =',iyrrr,'   month =',jmonrr
	  print *,'standard deviation base period =',iyr,' month =',jmon
          call maxmin (cl1,imx1,imy1,undef,'clm-orig')
          call maxmin (wk1,imx,imy,undef,  'clm-2-dg')
          call maxmin (sd1,imx1,imy1,undef,'sd-orig ')
          call maxmin (wk2,imx,imy,undef,  'sd-2-deg')
	endif
      enddo
c
      dall = 2.
      ay1 = -88
      ax1 = 0.
      do i=1,imx
        alon(i) = dall*float(i-1) + ax1
      enddo
      do i=1,imxp
        alonp(i) = dall*float(i-1) + ax1
      enddo
      call maxmin (alon,imx,1,undef,'alon:2dg')
      do j=1,imy
        alat(j) = dall*float(j-1) + ay1
      enddo
      call maxmin (alat,imy,1,undef,'alat:2dg')
c
      xt1 = 0.5
      yt1 = -89.5
      dxy = 1.
      do i=1,imx1
        xt(i) = dxy*float(i-1) + xt1
      enddo
      call maxmin (xt,imx1,1,undef,'xt:1-deg')
      do j=1,imy1
        yt(j) = dxy*float(j-1) + yt1
      enddo
      call maxmin (yt,imy1,1,undef,'yt:1-deg')
c
      i = 1
      ip = i*2 - 1 
      im = ip - 1
      if (im.eq.0) im = imx1
      print 101,alon(i),xt(im),xt(ip)
  101 format ('2-deg lon =',f8.1,5x,'uses: 1-deg lon =',2f8.1)
      i = imx
      ip = i*2 - 1 
      im = ip - 1
      print 101,alon(i),xt(im),xt(ip)
      j = 1
      jp = j*2 + 1
      jm = jp - 1
      print 102,alat(j),yt(jm),yt(jp)
  102 format ('2-deg lat =',f8.1,5x,'uses: 1-deg lat =',2f8.1)
      j = imy
      jp = j*2 + 1
      jm = jp - 1
      print 102,alat(j),yt(jm),yt(jp)
            

cv The QC loop starts in 2007 because the COADS
cv dataset is filtered in historical run 
cv The GTS data  is used from 2007 onwards
       do iyear=2007,iyre


c   open gts obs output files	  
	  
	 
	  write (ciyear(1:4),'(i4.4)') iyear	  
	  
          open(51, form=fm,file= adjustl(adjustr(Dir_data) // 
     &     '/inter/situ/ship.avg2.mon.'//ciyear//'.dat'))
     
          open(61,form=fm,file=adjustl(adjustr(Dir_data) //
     &     '/inter/situ/gts_climrej_ship.'//ciyear//'.dat'))
          open(71,form=fm,file=adjustl(adjustr(Dir_data) //
     &     '/inter/situ/gts_ersstrej_ship.'//ciyear//'.dat'))
     
          open(52,form=fm,file=adjustl(adjustr(Dir_data) //
     &     '/inter/situ/buoy.avg2.mon.'//ciyear//'.dat'))
          open(62,form=fm,file=adjustl(adjustr(Dir_data) //
     &     '/inter/situ/gts_climrej_buoy.'//ciyear//'.dat'))
          open(72,form=fm,file=adjustl(adjustr(Dir_data) //
     &     '/inter/situ/gts_ersstrej_buoy.'//ciyear//'.dat'))
     
        
	  
c big loop to add data from Jan.to processing month for the processing year
         
       imoe=12
       if(iyear.eq.iyre)imoe=imoend
       
       do imonth=1,imoe
       
        write (date,'(i4.4,i2.2)')  iyear,imonth
	print*,'processing iyear,imonth=', iyear,imonth

        iyr=iyear
        mon=imonth-1
        if(mon.le.0)then
        iyr=iyear-1
        mon=12
        endif

	
	write (date2,'(i4.4,i2.2)')  iyr,mon
	
	
c  get ersst input data 
	

c for the last month, ersst is previous month ersst + clim delta SST betwen current and prev months
c for all other month, ersst already available from previous run
        if(iyear.eq. iyre.and.imonth.eq.imoe)then
          open(21, form='unformatted',file= adjustl(adjustr(Dir_data)//
     &       '/output/sst2d/sst2d.situ.' // date2 // '.v3b.dat'))
     
c          print *,'open ersstp: sst2d.situ.', date2, '.v3b.dat' 
          read (21,err=603,end=603) ((ersst(i,j),i=1,imx),j=1,imy)
          call maxmin (ersst(1:imx,1:imy),imx,imy,undef,'ersst   ')
          close(21)



c    calculate the processing month ersst climatology
c    ersst(k)=ersst(k-1)-clm(k-1)+clm(k)
c         print*, 'climatology month=',mon,imonth
   
         do i=1,imx
            do j=1,imy
              if (ersst(i,j).ne.undef) then		
              ersst(i,j)=ersst(i,j) - clm(i,j,mon) + clm(i,j,imonth)
              endif
             enddo
          enddo
       
c for all other month, ersst already available. use ersst from reading       
       else
       
           
         open(21, file= adjustl(adjustr(Dir_data)//
     1     '/output/sst2d/sst2d.situ.' // date // '.v3b.dat'),
     2            form='unformatted')
     
c        print *,'open ersst: sst2d.situ.', date, '.v3b.dat'        
        read (21,err=603,end=603) ((ersst(i,j),i=1,imx),j=1,imy)	
	call maxmin (ersst(1:imx,1:imy),imx,imy,undef,'ersst   ')
        close(21)
	
	endif
              
       
c    end of getting ersst data
c
c  fill in ersst missing with climatology
c
	    do i=1,imx
	      do j=1,imy
		bigclm(i,j) = clm(i,j,imonth)
	        if (ersst(i,j).eq.undef) then
		  ersst(i,j) = clm(i,j,imonth)
		endif
	      enddo
	    enddo
	    do i=1,imx
	      do j=1,imy
	        bigsd(i,j) = stdev(i,j,imonth)
	      enddo
	    enddo
	    do j=1,imy
	      bigclm(imxp,j) = bigclm(1,j)
	       bigsd(imxp,j) =  bigsd(1,j)
	       ersst(imxp,j) =  ersst(1,j)
	    enddo
	   
	     
      
c    get gts input data
c    define the input file name of each month

       write (ciyear(1:4),'(i4.4)') iyear 
      
       if(iyear.ge.2000)then
       iyear2=iyear-2000
       else
       iyear2=iyear-1900
       endif
       write (ciyear2(1:2),'(i2.2)') iyear2
      
       write (cimonth(1:2),'(i2.2)') imonth
      
       gts_in= adjustl(adjustr(Dir_ftp) // 
     &        '/buoyship/nq'//ciyear2//cimonth)
       print*,'Input :'//gts_in
      
       open(31,file=gts_in,form='formatted',status='old',ERR=1234)
      
       do kk=1,2
	
	do i=1,imx
          do j=1,imy
	    dat(i,j,kk) = 0.
	     ct(i,j,kk) = 0.
	    rejclm(i,j,kk) = 0.
	    ctrejc(i,j,kk) = 0.
	    rejinc(i,j,kk) = 0.
	    ctreji(i,j,kk) = 0.
           enddo
         enddo


c     read in buoy ship observations and check if it is on land etc. 
c     mark them as buoy or ship so that it can be added to the right array demesion
	
            ipos(kk) = 0
 
	    iintp(kk) = 0
	    irejc(kk) = 0
	    ireji(kk) = 0
            irang(kk) = 0 

	    iok(kk) = 0
	    itot(kk) = 0
	    
	    izero(kk) = 0
	    iland(kk) = 0
	    
	    rang(kk) = 0 
	    rejc(kk) = 0
	    reji(kk) = 0
	    aland(kk) = 0
	    
       enddo		

       i_baddate=0
       
800    read(31,108, err=820,END = 8000) year1,mon,day,ihour,
     1      ilat,ilon,IDID,iwind,cid,isst
cliu    using monthly sst from ftp NCEP. For historic run     
108    format(3i2,i4,2i5,2i1,a6,18x,i3)

c      convert year from 2 digital to 4 digital
       year=year1+2000 
       if(year1.ge.91)then
       year=year1+1900
       endif 


       if (year.ne.iyear .or. mon .ne.imonth)then
         i_baddate=i_baddate + 1
         go to 800
       endif

       if (isst.ge.999)then
         i_sst999=i_sst999+1
         go to 800
       endif


c    decide it is buoy or ship

c  count idid>=4 or IDID =0
       
       if(IDID.ge.4 .or .IDID .eq.0)then
         i_idid=i_idid+1
         go to 800
       endif

      if(IDID .eq.1)then
        nship=nship+1
		                  
c      record as ship	  
       kk=1
       
c     end of put to ship array.
                                    

c      if ID=2 or 3, put to buoy array 
     
       else
   
         nbuoy=nbuoy+1  
		  
c      record as buoy	  
       kk=2
      	                                                                                
      endif
      
      itot(kk)=itot(kk)+1
                            
c
c      end of put to buoy array 

              	         	 
       
c***** ******************************************************************* 
c  do  the 1/4 degree land mask check and 
c  using the new standard deviation and OISST to do the climatology check

c      make sure lat lon not out of the globe
      
       if (iabs(ilat).gt.8889.or.abs(ilon).gt.36000)then
       i_globe=i_globe +1
       ipos(kk)=ipos(kk)+1
       go to 800
       endif

c       calculate the col & row number of each obs

c      convert alon from 0~360w to 0~360E) 

         ilon=36000-ilon    

        rlat=0.01*ilat
	rlon=0.01*ilon
	rsst=0.1*isst	           	
  
       
       if (isst.lt.-20 .or.isst.gt.400) then
	  irang(kk)=irang(kk)+1
c	  print*,'irang(',kk,')=',irang(kk)
                     
          go to 800                                                     
        endif 	
	
	if (rsst.eq.0.0.and.rlat.gt.84.9) then
                print 158,rlat,rlon,year,mon
  158	         format ('SST=0, lat>84.9 : lat =',f7.2,2x,'lon ='f7.2,2x,
     1          'year =',i5,2x,'month =',i3)
	        izero(kk) = izero(kk) + 1
		go to 800
	      endif
				
c       do both climatology and ersst screen	
              call intrp1 (ersst,imxp,imxp,imy,alonp,alat,
     1          rlon,rlat,ssters,ierr)
	      if (ierr.eq.1) then
	        iintp(kk) = iintp(kk) + 1
	        go to 800
	      endif            

	      call intrp1 (bigclm,imxp,imxp,imy,alonp,alat,
     1          rlon,rlat,sstclm,ierr)
	      if (ierr.eq.1) then
	        iintp(kk) = iintp(kk) + 1
	        go to 800
	      endif
	      
	      call intrp1 (bigsd,imxp,imxp,imy,alonp,alat,
     1          rlon,rlat,sdint,ierr)
	      if (ierr.eq.1) then
	        iintp(kk) = iintp(kk) + 1
	        go to 800
	      endif
	      
	      ii = nint((rlon-ax1)/dall) + 1
	      if (ii.eq.imxp) ii = 1
	      jj = nint((rlat-ay1)/dall) + 1 
	      
	      if(ii .lt.1 .or. ii .gt. 180)then
	      print*,'ii,rlon=',ii,rlon
	      endif
	      
	      if(jj .lt.1 .or. jj .gt. 89)then
	      print*,'jj,rlat=',jj,rlat
	      endif
	      
	      anom = abs(rsst-sstclm)
	      ainc = abs(rsst-ssters)
	      
	      if (amask(ii,jj).lt.0.1) then
	        iland(kk) = iland(kk) + 1
c		print*,'iland(kk)=',iland(kk)
	        go to 800
	      endif
	      	      
        
c    climatology check
	      
	      if (anom.gt.sdint) then
	        rejclm(ii,jj,kk) =  rejclm(ii,jj,kk) + rsst
	        ctrejc(ii,jj,kk) =  ctrejc(ii,jj,kk) + 1.
	        irejc(kk) = irejc(kk) + 1
	        
	      endif
c    ersst check	      
	      if (ainc.gt.sdint) then
	        rejinc(ii,jj,kk) =  rejinc(ii,jj,kk) + rsst
	        ctreji(ii,jj,kk) =  ctreji(ii,jj,kk) + 1.
	        ireji(kk) = ireji(kk) + 1	        
	        go to 800
	      endif
	      
	      dat(ii,jj,kk) =  dat(ii,jj,kk) + rsst
	       ct(ii,jj,kk) =  ct(ii,jj,kk)  + 1.
	      iok(kk) = iok(kk) + 1
	      
	    go to 800
	    
820   print *,'nerr',year1,mon,day,ihour,ilat,ilon,IDID,iwind,cid,isst 
       nerr=nerr+1
       go to 800
103    format(a4,1x,3i2,i4,2i5,2i1,a6,18x,i3)
      	     
8000   continue
c       print*,'i_baddate=',i_baddate 

cliu   added on 2/24/2008 
cliu   if the buoy ship data dose not match the month, then total obs=0 run will stop
cliu   if the total obs<= 500000, continue run but get a warnning

       iitot=itot(1)+itot(2)
       print*,'iitot=',iitot 
       if (iitot.eq.0)then
       print*,'RED-STOP! Dates do not match: processing yearmonth=', 
     2 iyear,imonth,'Input data yearmonth=',year,mon
       call exit(100)
       endif

       if (iitot.le.500000)then
       print*,'YELLOW-WARNING! buoy ship observation number too low' 
c       call exit(10)
       endif
    
       do kk=1,2

          do i=1,imx
          do j=1,imy
	    if (ct(i,j,kk).gt.0.5) then
	      dat(i,j,kk) = dat(i,j,kk)/ct(i,j,kk)
	    else
	      dat(i,j,kk) = undef
	    endif
	    if (ctreji(i,j,kk).gt.0.5) then
	      rejinc(i,j,kk) = rejinc(i,j,kk)/ctreji(i,j,kk)
	    else
	      rejinc(i,j,kk) = undef
	    endif
	    if (ctrejc(i,j,kk).gt.0.5) then
	      rejclm(i,j,kk) = rejclm(i,j,kk)/ctrejc(i,j,kk)
	    else
	      rejclm(i,j,kk) = undef
	    endif
          enddo
          enddo 
	  
	if(iyear.eq.2009.and.imonth.eq.4)then
	print*, '??????????ct', 
     2      ct(63,64,1),dat(63,64,1),rejinc(63,64,1),rejclm(63,64,1)
	endif
	
	
	call maxmin (dat(:,:,kk),imx,imy,undef,'obs     ')
	call maxmin (ct(:,:,kk) ,imx,imy,undef,'num obs ')
	call maxmin (rejclm(:,:,kk),imx,imy,undef,'rej clm ')
	call maxmin (rejinc(:,:,kk),imx,imy,undef,'rej inc ')
	
	isum(kk) = ipos(kk) + iintp(kk) + izero(kk)
	if (isum(kk).gt.0) then
	  print 202,ipos(kk),iintp(kk),izero(kk)
  202     format ('ipos =',i3,2x,'inpt =',i3,2x,'izero =',i8)
	endif
	
	print*,'-----------------------------------------------------'
	print*,ctype(kk),iyear,imonth
c	print*,'iok=',iok(kk)
	print*,'itot',itot(kk)
	  
c	  if (iok(kk).gt.0) then
c	  rang(kk) = 100.*float(irang(kk))/float(iok(kk))
c	  rejc(kk) = 100.*float(irejc(kk))/float(iok(kk))
c	  reji(kk) = 100.*float(ireji(kk))/float(iok(kk))
c	  aland(kk) = 100.*float(iland(kk))/float(iok(kk))
	  
	if (itot(kk).gt.0) then
	  rang(kk) = 100.*float(irang(kk))/float(itot(kk))
	  rejc(kk) = 100.*float(irejc(kk))/float(itot(kk))
	  reji(kk) = 100.*float(ireji(kk))/float(itot(kk))
	  aland(kk) = 100.*float(iland(kk))/float(itot(kk))  
	  
	else
	  rang(kk) = 0.
	  rejc(kk) = 0.
	  reji(kk) = 0.
	endif
	print 201,rang(kk),aland(kk),rejc(kk),reji(kk),iok(kk)
  201   format ('%REJ: range =',f6.2,2x,'land =',f6.2,2x,
     1  'clim =',f6.2,2x,'inc =',f6.2,2x,'num ok =',i8)
        iw=50+kk
	irc=60+kk
	iri=70+kk
c	print*,iw,irc,iri
	write (iw) dat(:,:,kk)
	write (iw) ct(:,:,kk)
	write (irc) rejclm(:,:,kk)
	write (irc) ctrejc(:,:,kk)
	write (iri) rejinc(:,:,kk)
	write (iri) ctreji(:,:,kk)
        
	enddo
	
        enddo
	enddo
  
      print*, 'GREEN-SUCCESSFUL!: gtsqc.situ.v3b.f'
      call exit(1)
      
 1000 print 701                                                         
  701 format ('RED-STOP BUOYSHIP-MAIN: 
     1            that day not available in archive')   
      call exit(100) 
      
1234  print*,'RED-STOP! BUOYSHIP-MAIN: ',
     1       'Buoy ship input file does not exist ?????????'
      call exit (100)
      
c  100 print*, 'RED-STOP! ersst input data error in gtsqc.f 100'    
c      call exit (100)     
  501 print*, 'RED-STOP! data input problem in gtsqc.f 501'        
      call exit(100)
  601 print *,'RED-STOP! premature end of mask data: stop 601'
      call exit(100)
  602 print *,'RED-STOP! premature end of climate data: stop 602'
      call exit(100)
  603 print *,'RED-STOP! premature end of ersst data: stop 603'
      call exit(100)
  604 print *,'RED-STOP! premature end of coads data: stop 604'
      call exit(100)
      
      end

c==========END OF MAIN PROGRAM===============================
c@***s* ERSST/gtsqc.situ.v3b.0.f/intrpl
c*
c* NAME
c*  intrpl
c*
c* LOCATION
c*  $ersst_SRC_DIR/gtsqc.situ.v3b.0.f
c* 
c* PURPOSE
c*   To interpolate the value taking into account position within the grid cell
c*
c* DESCRIPTION 
c* In order to average multiple observations at differnt locations within the
c* grid cell, the values have to be adjusted taking into account the their
c* relative position in the cell.  This subroutine uses the known corner values
c* of the cell weighted by area (computed using delta degrees,not distance), to
c* interpolate the value of a point within a grid cell
c*
c* AUTHOR
c*  Richard W. Reynolds
c*
c* CREATION DATE 
c*  Unknown -pre-2009
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  2009 Sept -Viva Banzon added comments and put into robodoc format
c*
c* INPUTS
c* cn	= gridded reference SST
c* imc	= 2 deg x-grid size (nlons) padded, i.e. with start lon repeated at end
c* ic 	= 2 deg x-grid size (nlons)
c* jc 	= 2 deg y-grid size (nlats)
c* cx 	= lons array padded
c* cy 	= lats array
c* fx 	= point lon
c* fy 	= point lat
c*
c* RETURNED VALUE
c* fn 	 = point SST (interpolated)
c* ierr = error message
c*
c* VARIABLES
c* cn	= gridded reference SST
c* cx 	= lons array padded
c* cy 	= lats array
c*
c* TODO
c*	Put this in a subroutine library as this is used in many programs
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
      
      subroutine intrp1 (cn,imc,ic,jc,cx,cy,fx,fy,fn,ierr)
      dimension cn(imc,jc),cx(ic),cy(jc)
      ierr = 1
      if (fx.lt.cx(1).or.fx.gt.cx(ic)) go to 200
      if (fy.lt.cy(1).or.fy.gt.cy(jc)) go to 200
      do 110 l=1,jc
      if (cy(l).lt.fy) go to 110
      jn = l
      js = l-1
      if (js.eq.0) jn = 2
      if (js.eq.0) js = 1
      go to 111
  110 continue
      jn = jc
      js = jc - 1
  111 continue
      do 120 l=1,ic
      if (cx(l).lt.fx) go to 120
      ie = l
      iw = l-1
      if (iw.eq.0) ie = 2
      if (iw.eq.0) iw = 1
      go to 121
  120 continue
      ie = ic
      iw = ic - 1
  121 continue
      a = (fx - cx(iw))
      b = (cx(ie) - fx)
      c = fy - cy(js)
      d = cy(jn) - fy
      ac = a*c 
      bc = b*c  
      ad = a*d
      bd = b*d
      top = ac*cn(ie,jn) + bc*cn(iw,jn) + ad*cn(ie,js) + bd*cn(iw,js)
      bot = ac + bc + ad + bd
      ierr = 0
      if (bot.lt.1.e-10) go to 200
      fn = top/bot
      ierr = 0
      return
c
c     no interpolated value
c
   10 print 15
   15 format (' problem in intrp1: fx or fy out of range;',
     1 '   no interpolated value')
      print 20,cx(1),cx(ic),fx
   20 format (' cx(1) =',f10.3,3x,'cx(ic) =',f10.3,3x,'fx =',f10.3)
      print 25,cy(1),cy(jc),fy
   25 format (' cy(1) =',f10.3,3x,'cy(jc) =',f10.3,3x,'fy =',f10.3)
      return
  200 print 210,ie,iw,jn,js,bot
  210 format ('ie =',i3,3x,'iw =',i3,3x,'jn =',i3,3x,'js =',i3,
     1 3x,'bot =',f10.3)
      print 220,ie,jn,cn(ie,jn)
      print 220,iw,jn,cn(iw,jn)
      print 220,ie,js,cn(ie,js)
      print 220,iw,js,cn(iw,js)
  220 format ('cn('i3','i2') =',f9.2)
      return
      end
c==================================================================      
c@***s* ERSST/gtsqc.situ.v3b.f/maxmin
c*
c* NAME
c*	 maxmin
c*
c* LOCATION
c*  $ersst_SRC_DIR/gtsqc.situ.v3b.f
c* 
c* PURPOSE
c*   To compute maximum, minium and average of input field fld and print to screen
c*
c* DESCRIPTION 
c* The minimum value holder is first set to a very large number and
c* the maximum value holder is first set to a very small number.  Then 
c* the current value replaces the value in the min holders if the current
c* value is smaller.  The maximum is determined the same way. The data is
c* summed and counted within the loop, then the average is computed upon
c* exiting the loop. The max, min and average are displayed on the screen.
c*
c* AUTHOR
c*  Thomas M Smith 
c*
c* CREATION DATE 
c*  Unknown
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  2009 Sept -Viva Banzon added comments and put into robodoc format
c*
c* INPUTS
c*	fld	= data to be examined for statistics
c*	im,jm	= x and y size of array
c*	undef	= code for missing data
c*	name	= name for input data field
c*
c* RETURNED VALUE
c*	none
c*
c* PARAMETERS
c*    im1, jm1 = xsize and ysize of 1 degree grid
c*    im2, jm2 = xsize and ysize of 2 degree grid
c*
c* VARIABLES
c*	fldmin	= minimum of field
c*	fldmax	= maximum of field
c*	fld 	= data field
c*	ave	= average of field
c*	inum	= number of valid data in field
c*
c* TODO
c*	Put this in a subroutine library as this is used in many programs
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
      
      
      subroutine maxmin (fld,im,jm,undef,name)
c
c   compute maximum, minium and average of fld and print
c
      dimension fld(im,jm)
      character*8 name
      fldmin = 1.e30
      fldmax = -fldmin
      inum = 0
      ave = 0.
      do 90 i=1,im
      do 90 j=1,jm
      if (fld(i,j).ne.undef) then
        inum = inum + 1
        ave = ave + fld(i,j)
        if (fldmin.gt.fld(i,j))  fldmin = fld(i,j)
        if (fldmax.lt.fld(i,j))  fldmax = fld(i,j) 
      endif
   90 continue
      if (inum.gt.0) then
        r = 1./float(inum)
        ave = ave*r
      else
        fldmin = 0.
        fldmax = 0.
      endif
      print 88,ave,fldmin,fldmax,inum,name
   88 format (' ave =',f10.3,2x,'min =',f10.3,2x,
     1 'max =',f10.3,2x,'inum =',i8,2x,a8)
      return 
      end
