c    67  1         2         3         4         5         6         7 2
c@***h* ERSST/ice1t2.f
c*
c* NAME
c*  ice1t2.f
c*
c* LOCATION
c*  $ersst_SRC_DIR/ice1t2.f
c*
c* WARNING 
c* Change destination directories if not running operationally.
c* Assumes that annual updated files of 1-deg monthly sea ice
c* are available for input (unit 21). These are generated by the
c* OISST daily processing.
c* 
c* PURPOSE
c* To average monthly sea-ice concentration data from the 1-deg grid
c* to the 2-deg grid.
c*
c* DESCRIPTION
c* Values in the input 1-deg grid (0.5E-359.5E by 89.5S-89.5N) are 
c* averaged to produce the 2-deg grid (0E-358E by 88S-88N).  GSFC ice
c* concentrations are used 1985-2004, while NCEP ice is used afterwards.
c* Prior to averaging, two adjustments are made: 
c* 1) To compensate for microwave biases (warm pools id'd as open water),
c* an ice-fraction-dependent adjustment factor is applied. 
c* 2) Data are weighted by the relative area of each superobservation.
c* See Reynolds et al. (2002 section 5b). The adjustment factors were
c* determined in a one-time analysis of several years of ice data by T. Smith 
c* and codes are not at NCDC.  This strategy was adopted because ice data can
c* be delayed and therefore, are not complete or available at time of analysis.
c*
c* AUTHOR
c*  Thomas M Smith
c*
c* CREATION DATE 
c*  December 2006
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  2010 March - Chunying Liu hardcoding.
c*  2009 July -Viva Banzon added comments and put into robodoc format
c*  July 2006  - Chunying Liu modified the code to run at NCDC              
c*
c* INPUTS
c*    iyrend  ..... year to process
c*    imoend  ..... month to process
c* adjustment factors 
c*    unit 21 ..... adj.gsfc_ice2.mon.cyc.dat 	 for GSFC sea ice (1985-2004)
c*    unit 21 ..... adj.ncep_ice2.mon.cyc.dat    for NCEP sea ice (2005 onward)
c* 1-deg monthly sea ice (GSFC or NCEP depending on yr); cyr updated by program
c*    unit 21 ..... medmonice.ave-1d.cyr	
c*    mon ......... the month of the year to do
c*    iy .......... the year to do.
c*
c* OUTPUTS
c* 2-deg monthly sea ice; cyr updated by program
c*    unit 51 ..... sice.avg2.mon.cyr.dat 
c*
c* PARAMETERS
c*    im1, jm1 = xsize and ysize of 1 degree grid
c*    im2, jm2 = xsize and ysize of 2 degree grid
c*
c* VARIABLES 
c*	a1	= monthly median sea ice conc on 1 deg grid
c*	a2	= sea ice conc on 2 deg grid
c* array of corrections for the microwave-based sea ice values  
c* factors vary by month, hemisphere, and fraction of ice cover
c*	adjg	= adjustment factors for GSFC data (1985-2004)
c*	adjn	= adjustment factors for NCEP data (2005 onwards)
c*
c* TODO
c*  switch to array notation
c*  error message if input file read fails
c*  put immutable files from T. Smith in a separate directory
c*  needs documentation on where monthly ice and adjustments come from
c*  could put subroutines separately into a library?
c*
c* SUBPROGRAMS
c*  adj1d
c*  av1t2a
c*  range1
c*  range
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
c
      parameter(im1=360,jm1=180,im2=180,jm2=89)
      real a1(im1,jm1),a2(im2,jm2),adjg(11,2,12),adjn(11,2,12)
      character fm*11,st*3,cyr*4,pd1*19,pd2*18
      data fm/'unformatted'/, st/'old'/
      CHARACTER(LEN=60) :: Dir_data
      
      CALL getenv("ersst_DAT_DIR", Dir_data)
      print*, 'Input begin year (automatically begins in jan):'
      read(5,*) iyrbeg
      print*, 'Input the year to do:'
      read(5,*) iyrend
      print*, 'Input the month to do:'
      read(5,*) imoend

      print*, 'begin year :', iyrbeg
      print*, 'update year month:',iyrend,imoend

c* Adjustments are for each month, kh=1 is SH, kh=2 is NH, i=1,11 for
c* fractions = 0 to 1 by 0.1.
c* Read in adjustments for GSFC ice (used 1985-2004).
      open(21,status=st,form=fm,file= adjustl(adjustr(Dir_data)// 
     & '/input/static/adj.gsfc_ice2.mon.cyc.dat'),err=81)
      do mon=1,12  
       do kh=1,2
        read(21,end=81,err=81) (adjg(i,kh,mon),i=1,11)
       end do
      end do
      close(21)
      go to 11
  81  print*,'YELLOW-WARNING! problem with GSFC adjustments  ice1t2.f'
c* Read in adjustments for NCEP ice (used from 2005 on).
  11  open(21,status=st,form=fm,file=adjustl(adjustr(Dir_data) // 
     & '/input/static/adj.ncep_ice2.mon.cyc.dat'), err=82)
      do mon=1,12
       do kh=1,2
        read(21,end=82,err=82) (adjn(i,kh,mon),i=1,11)
       end do
      end do
      close(21)
      go to 12
  82  print*,'YELLOW-WARNING! problem with NCEP adjustments  ice1t2.f'
  12  do 1000 iy=iyrbeg,iyrend
        mondo=12
        if(iy.eq.iyrend)mondo=imoend
 
        write(cyr,'(i4)') iy
        open(21,form=fm,status=st,file=adjustl(adjustr(Dir_data)//
     &   '/inter/ice1d/medmonice.ave-1d.'//cyr))
c
        open(51,form=fm,file=adjustl(adjustr(Dir_data)//
     &   '/inter/ice2d/sice.avg2.mon.'//cyr//'.dat'))
c
        do mon=1,mondo
c* Read the sea ice concentrations (a1).
           read(21,end=100,err=100) iyx,imx,idx,a1
c* Adjust the concentrations.
           if(iy.le.2004)then
              call adj1d(a1,mon,adjg)
           else
              call adj1d(a1,mon,adjn)
           endif
c
c       call range1(a1,amn,amx)
c       print*, 'Rge 1-d dat:',amn,amx
c
           call av1t2a(a1,a2)
c
c       call range(a2,amn,amx)
c       print*, 'Rge 2-d dat:',amn,amx
c       stop 'chk1'
c
c* Write the 2-d sea ice concentrations (a2).
           write(51) a2
        end do
        close(21)
        close(51)
c
 1000 continue
 
      print*, 'GREEN-SUCCESSFUL! ice1t2.f'
      call exit(1)
 100  print*, 'RED-STOP! Data problem ice1t2.f '
      call exit(100)
      end
        
c=============END OF MAIN PROGRAM===============================
c@***s* ERSST/ice1t2.f/adj1d
c* NAME
c*    adj1d
c*
c* LOCATION
c*    inside $ersst_SRC_DIR/ice1t2.f
c*
c* WARNING
c*  Adjustment factors may change in future.
c*
c* PURPOSE
c* To adjust the ice concentrations on a 1-deg grid using the a set of factors
c* passed from the main program.
c*
c* DESCRIPTION
c* Adjustments are for each month, hemisphere: adj(kf,kh,mon), where kh=1 is SH,
c* kh=2 is NH, kf=1,11 for fractions = 0 to 1 by 0.1. No adjustmnet is made
c* for low latitudes (30S-30N). These factors were computed by examining biases
c* in microwave-based sea ice estimates due to the presence of shallow pools.  
c* Thus, the correction is dependent on geography (hemisphere), ice fraction (ice
c* pools more likely to form when not completely frozen or melted), and season.
c*
c* AUTHOR
c*  Thomas M Smith
c*
c* CREATION DATE 
c*  December 2006 (presumed)
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  2009 July -Viva Banzon added comments and put into robodoc format
c*
c* INPUTS
c*   a1	= unadjusted monthly ice on 1 deg grid
c*   mon	= month
c*   adj	= adjustment factors
c*
c* RETURNED VALUE
c*   a1	= adjusted ice data on 1 deg grid
c*
c* PARAMETERS
c*   im1, jm1	= xsize and ysize of 1 degree grid
c*
c* VARIABLES
c*   kf	= index for adjustment factor based on ice conc 
c*
c* TODO
c*	separate out subroutines
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
      subroutine adj1d(a1,mon,adj)
      parameter(im1=360,jm1=180)
      real a1(im1,jm1),adj(11,2,12)
      do j=1,60
       do i=1,im1
        if(a1(i,j).gt.0.3)then
         kf=nint(10.0*a1(i,j)) + 1
         if(kf.gt.11) kf=11
         a1(i,j)=a1(i,j)+adj(kf,1,mon)
        endif
       end do
      end do
      do j=121,180
       do i=1,im1
        if(a1(i,j).gt.0.3)then
         kf=nint(10.0*a1(i,j)) + 1
         if(kf.gt.11) kf=11
         a1(i,j)=a1(i,j)+adj(kf,2,mon)
        endif
       end do
      end do
      return
      end

c------------------------------------------------------------------
c@***s* ERSST/ice1t2.f/av1t2a
c* NAME
c*    av1t2a
c*
c* LOCATION
c*    inside $ersst_SRC_DIR/ice1t2.f
c*
c* WARNING
c*  None
c*
c* PURPOSE
c* To average from the 1-deg grid (0.5E-359.5E by 89.5S-89.5N centers)
c* to the 2-deg grid (0E-358E by 88S-88N centers).
c*
c* DESCRIPTION
c* For each 2-deg gridpoint, the 1-deg grid indices of the values to be
c* averaged are first determined.  Then the 1-deg area weights are applied
c* to compute and 2-deg area average.
c*
c* AUTHOR
c*  Thomas M Smith
c*
c* CREATION DATE 
c*  December 2006 (presumed)
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  2009 July -Viva Banzon added comments and put into robodoc format
c*
c* INPUTS
c*   a1	= adjusted monthly ice conc on 1 deg grid
c*
c* RETURNED VALUE
c*   a2	= ice conc on 2 deg grid
c*
c* PARAMETERS
c*   im1, jm1 = xsize and ysize of 1 degree grid
c*   im2, jm2 = xsize and ysize of 2 degree grid
c*   cj	= latitudinally varying area weights
c*
c* VARIABLES
c*   s1	= sum of ice conc weighted by area
c*   s2	= sum of area weights
c*   is,ie	= x-indices of 1-deg grid of start and end for computing 2 deg avg
c*   js,je	= y-indices of 1-deg grid of start and end for computing 2 deg avg
c*
c* TODO
c*	separate out subroutines
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
      subroutine av1t2a(a1,a2)
      parameter(im1=360,jm1=180,im2=180,jm2=89)
      real a1(im1,jm1),a2(im2,jm2),cj(jm1)
      data cj/0.01,0.03,0.04,0.06,0.08,0.10,0.11,0.13,0.15,0.17,
     2        0.18,0.20,0.22,0.23,0.25,0.27,0.28,0.30,0.32,0.33,
     3        0.35,0.37,0.38,0.40,0.41,0.43,0.45,0.46,0.48,0.49,
     4        0.51,0.52,0.54,0.55,0.57,0.58,0.59,0.61,0.62,0.64,
     5        0.65,0.66,0.68,0.69,0.70,0.71,0.73,0.74,0.75,0.76,
     6        0.77,0.78,0.79,0.80,0.81,0.82,0.83,0.84,0.85,0.86,
     7        0.87,0.88,0.89,0.89,0.90,0.91,0.92,0.92,0.93,0.94,
     8        0.94,0.95,0.95,0.96,0.96,0.97,0.97,0.98,0.98,0.98,
     9        0.99,0.99,0.99,0.99,1.00,1.00,1.00,1.00,1.00,1.00,
     1        1.00,1.00,1.00,1.00,1.00,1.00,0.99,0.99,0.99,0.99,
     1        0.98,0.98,0.98,0.97,0.97,0.96,0.96,0.95,0.95,0.94,
     2        0.94,0.93,0.92,0.92,0.91,0.90,0.89,0.89,0.88,0.87,
     3        0.86,0.85,0.84,0.83,0.82,0.81,0.80,0.79,0.78,0.77,
     4        0.76,0.75,0.74,0.73,0.71,0.70,0.69,0.68,0.66,0.65,
     5        0.64,0.62,0.61,0.59,0.58,0.57,0.55,0.54,0.52,0.51,
     6        0.49,0.48,0.46,0.45,0.43,0.41,0.40,0.38,0.37,0.35,
     7        0.33,0.32,0.30,0.28,0.27,0.25,0.23,0.22,0.20,0.18,
     8        0.17,0.15,0.13,0.11,0.10,0.08,0.06,0.04,0.03,0.01/
c
      do 4 j2=1,jm2
      js=2*j2
      je=js+1
      do 4 i2=1,im2
       is=2*(i2-1)
       ie=is+1
       s1=0.0
       s2=0.0
       do 2 i1=is,ie
       id=i1
       if(i1.eq.0) id=im1
       do 2 j1=js,je
        if(a1(id,j1).gt.-990.)then
         wgt=cj(j1)
         s1=s1+a1(id,j1)*wgt
         s2=s2+wgt
        endif
 2     continue
       a2(i2,j2)=-999.9
       if(s2.gt.0.0) a2(i2,j2)=s1/s2
 4    continue
      return
      end
c
c*-------------------------------------------------
c@***s* ERSST/ice1t2.f/range1
c* NAME
c*  range1
c*
c* LOCATION
c*  inside $ersst_SRC_DIR/ice1t2.f
c*
c* WARNING
c*  None
c*
c* PURPOSE
c* To compute the array rangefor 1 deg grid.
c*
c* DESCRIPTION
c* Minimum variable is initialized as a very large number and maximum
c* variable is set to a very small number.  Each value is compared to the min or
c* max, and replaces the value in the variable if it is smaller or larger, 
c* respectively.
c*
c* AUTHOR
c*  Thomas M Smith
c*
c* CREATION DATE 
c*  December 2006 (presumed)
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  2009 July -Viva Banzon added comments and put into robodoc format
c*
c* INPUTS
c*   a2	= array on 1 deg grid
c*
c* RETURNED VALUE
c*   amn	= minimum value of array
c*   amx	= maximum value of array
c*
c* PARAMETERS
c*   im, jm 	= xsize and ysize of 1 degree grid
c*
c* VARIABLES
c*   a2	= array on 1 deg grid
c*   i,j	= indices for 1-deg grid
c*
c* TODO
c*	separate out subroutines. these two range routines could be one
c*	also check if fortran 90 has a min max function
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
      subroutine range1(a2,amn,amx)
      parameter(im=360,jm=180)
      real a2(im,jm)
      amn=1.e9
      amx=-amn
      do 2 j=1,jm
      do 2 i=1,im
       if(a2(i,j).gt.-990.)then
        if(a2(i,j).lt.amn) amn=a2(i,j)
        if(a2(i,j).gt.amx) amx=a2(i,j)
       endif
 2    continue
      return
      end
c
c*---------------------------------------------------
c@***s* ERSST/ice1t2.f/range
c* NAME
c*  range
c*
c* LOCATION
c*  inside $ersst_SRC_DIR/ice1t2.f
c*
c* WARNING
c*  None
c*
c* PURPOSE
c* To compute the array range for 2 deg grid.
c*
c* DESCRIPTION
c* Minimum variable is initialized as a very large number and maximum
c* variable is set to a very small number.  Each value is compared to the min or
c* max, and replaces the value in the variable if it is smaller or larger, 
c* respectively.
c*
c* AUTHOR
c*  Thomas M Smith
c*
c* CREATION DATE 
c*  December 2006 (presumed)
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  2009 July -Viva Banzon added comments and put into robodoc format
c*
c* INPUTS
c*   a2	= array on 2 deg grid
c*
c* RETURNED VALUE
c*   amn	= minimum value of array
c*   amx	= maximum value of array
c*
c* PARAMETERS
c*   im, jm 	= xsize and ysize of 2 degree grid
c*
c* VARIABLES
c*   a2	= array on 2 deg grid
c*   i,j	= indices for 2-deg grid
c*
c* TODO
c*  separate out subroutines. these two range routines could be one
c*  also check if fortran 90 has a min max function
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
      subroutine range(a2,amn,amx)
      parameter(im=180,jm=89)
      real a2(im,jm)
      amn=1.e9
      amx=-amn
      do 2 j=1,jm
      do 2 i=1,im
       if(a2(i,j).gt.-990.)then
        if(a2(i,j).lt.amn) amn=a2(i,j)
        if(a2(i,j).gt.amx) amx=a2(i,j)
       endif
 2    continue
      return
      end
