c    67  1         2         3         4         5         6         7 2
c@***h*  ERSST/lfsst.situ.v3b.f
c*
c* NAME
c*  lfsst.situ.v3b.f
c*
c* LOCATION
c*  $ersst_SRC_DIR/lfsst.situ.v3b.f
c*
c* WARNING
c*  Change destination directories if not running operationally.
c*  Operationally, only the more recent years (from 1985) are updated
c*  and the historical period is left alone.
c*  Some variable names in the main program are used 
c*  in subroutines for temporary variables.
c* 
c* PURPOSE
c*  To compute the low-frequency (decadal) component of the in situ-only 
c*  SST anomalies from 1985 to the user-specified year-month, and update
c*  the output file. 
c* 
c* DESCRIPTION
c*  The low frequency (LF) analysis of the SST anomalies post-1984 is performed 
c*  on a 2-deg grid (lon: 0, 2E, ...2W) (lat: 88S,..,0, ...88N). Parameter abmx
c*  is the maximum anomaly value allowed to be used in the LF analysis.  It is 
c*  reduced at high latitudes.  The LF is performed on area-weighted anomalies 
c*  by filtering spatially and temporally. The 2-deg monthly super-observations
c*  are weighted by both the number of observations in each super-observation and
c*  by the noise of the data type. Then the monthly values are spatially smoothed
c*  over moving 26-degree areas, for all areas where at least 3% of the area is 
c*  defined.  The areal averages are then averaged annually when at least 2 months
c*  are defined.  The 15-year median of the annual averages is computed when at 
c*  least 2 smoothed years are defined.  At the end points the median time filter 
c*  is truncated, so that there are only 8 years in the median at the extreme years.
c*  From a previous one-time analysis, the ICOADS LF values are used to fill values
c*  before 1985 for the 15-year filtering. Undefined areas are filled with zero 
c*  anomaly and a spatial binomial filter is applied. Due to the truncated time
c*  window for the latest years, recent results will change in forthcoming
c*  years as more data are acquired.
c*
c* AUTHOR
c*  Thomas M. Smith
c*
c* CREATION DATE
c*  December 2006
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* PROGRAM HISTORY
c*  2010 March - Chunying Liu hardcoding.
c*  2009 Sep: V. Banzon added comments as part of documentation effort
c*  July 2006  - Chunying Liu modified the code to run at NCDC 
c*
c*
c* INPUTS
c*  passed interactively or by script
c*    	iy ......... user-specified year to do
c*	mondo .......user-specified month to do
c*  land mask (land = 0, sea = 1)
c* 	unit 30 ........ mask2d.dat
c*  2-deg ICOADS annual-spatial yearly averages 
c*  values from 1970-1984 are used to initially populate the median averaging window 
c*  when the window is moved the next year, the merged in situ anomaly from 1985 is used
c*  and so on
c* 	unit 31 ........ saan.situ.1854.2006.dat
c*  2-deg monthly merged ship-buoy updated anomalies; program substitutes in ciyear
c*  current analysis begins in 1985
c*  	unit 21 ........ $ersst_DAT_DIR/inter/situmerg/ssta.merg.situ.mon.v3b.cyr.dat
c*
c* OUTPUTS 
c*  updated LF anomaly from 1978, with the latter part using a half-length truncated time
c*  series for the end point.
c*	unit 61 ........$ersst_DAT_DIR/inter/analysis/lfsst.1978.situ.last.v3b.dat
c*  an optional output to print out intermediate array before median filtering
c*	unit 63 ........ $ersst_DAT_DIR/inter/analysis/saan.upd.1985.2006.v3b.dat
c*
c* PARAMETERS 
c*	im, jm	= x- and y-size of 2 deg grid
c*	abmx	= maximum anomaly accepted by LF analysis
c*
c* VARIABLES 
c*   Arrays that are on the 2 deg grid
c*	a2	= represents different quantities in different parts of program
c*		= first contains the COADS annual average read for 1 year
c*		= monthly in situ avg merged SST anomaly
c*		= returned anomaly array from each subroutine that performs
c*		 	+annual averaging after spatial averaging of each monthly  
c*		 	+Spatial binomial filtering of annual,then Polar damping
c*		 	+Median filtering within 15 year window  (LF output)
c*	co	= contains counts for COADS annual averages
c*	cn	= holds the monthly in situ avg merged SST anomaly counts
c*	mask	=2  deg land mask

c*   Monthly time series of 2-deg gridded data
c*	sstp	=holds the 12 months of in situ merged anomalies
c*	ndp		=integerized monthly counts associated with sstp

c*   Annual time series of 2-deg gridded data
c*	sm	= different quantities in program
c*		= COADS annual averages from 1970-1984
c*		= modified to left shift time series and hold
c*                the 15 years data used for median filtering
c*   15 years data (or less for recent years) at one grid point
c*	wk	= holds valid data to passed on to median subroutine
c*
c* SUBPROGRAMS
c*  avgsa
c*  spsmb2
c*  median
c*  damphl
c*
c* TODO
c*  decide whether to keep subroutines in program or put in a subroutine subdirectory
c*
c* LANGUAGE
c*  Fortran
c*
c@****
c* =======================END MAIN HEADER=======================
c*
      parameter(im=180,jm=89,abmx=8.0)
      real a2(im,jm),co(im,jm),sm(im,jm,15),cn(im,jm)
      real sstp(im,jm,12),wk(15)
      integer mask(im,jm),ndp(im,jm,12)
      character fm*11,st*3,cyr*4
      data fm/'unformatted'/, st/'old'/
      CHARACTER(LEN=80) :: Dir_data,fname
c
      print*, 'Input the year to do:'
      read(5,*) iy
      print*, 'Input the month to do:'
      read(5,*) mondo

cliu    make this changes on 2007/07/31. if run 2005/12,
cliu    then run 2007/07 it will use 2005/12 instead of 2006/12 data
cliu      if(mondo.ne.12)then
cliu       print*, 'GREEN: no LF updated needed'
cliu       call exit(1)
cliu      endif


       CALL getenv("ersst_DAT_DIR", Dir_data)
       
       if(mondo.ne.12)then
       iy=iy-1
       mondo=12 
       print*,'iy=',iy,'mondo=',mondo
       endif

c
      
c      open(61,form=fm,file=adjustl(adjustr(Dir_data)//
      fname=adjustl(adjustr(Dir_data)//
     &'/inter/analysis/lfsst.1978.situ.last.v3b.dat')
      open(61,form=fm,file=fname, iostat=ierr)
      IF (ierr.NE.0) print*, fname, ierr

c      open(63,form=fm,file=adjustl(adjustr(Dir_data)//
      fname=adjustl(adjustr(Dir_data)//
     &'/inter/analysis/saan.upd.1985.2006.v3b.dat')
      open(63,form=fm,file=fname, iostat=ierr)
      IF (ierr.NE.0) print*, fname, ierr
c
c* Open the COADS annual averages, read through and initalize LF array.
c      open(31,status=st,form=fm,file=adjustl(adjustr(Dir_data) //
      fname=adjustl(adjustr(Dir_data)//
     &'/input/static/saan.situ.1854.2006.dat')
      open(31,form=fm,file=fname, iostat=ierr)
      IF (ierr.NE.0) print*, fname, ierr
      do ky=1854,1969
         read(31) a2
      end do
      do k=1,15
       read(31,end=100,err=100) a2
       do j=1,jm
       do i=1,im
        sm(i,j,k)=a2(i,j)
       end do
       end do
      end do
      close(31)
c* Get the mask.
c* Read in the 2-d mask (land = 0, sea = 1).
      fname=adjustl(adjustr(Dir_data)//
     &'/input/static/mask2d.dat')
      open(30,form=fm,file=fname, iostat=ierr)
      IF (ierr.NE.0) print*, fname, ierr
      read(30,end=100,err=100) mask
      close(30)
c
c
      do 95 ii=1985,iy
         write(cyr,'(i4)') ii
c* Loop over the year.
         fname=adjustl(adjustr(Dir_data)//
     &     '/inter/situmerg/ssta.merg.situ.mon.v3b.'//cyr//'.dat')
         open(21,form=fm,file=fname, iostat=ierr)
         IF (ierr.NE.0) print*, fname, ierr

c* Loop over months.
         do mon=1,12
            read(21,end=101,err=101) a2
            read(21,end=101,err=101) cn
       
c            if(ii-7.eq.2002)then
c              print*,ii,mon,'=================',a2(141,47),a2(141,47)
c              print*,ii,mon,'=================',cn(141,47),cn(141,47)
c            endif
       
c           if(ii-7.eq.1998)then
c             print*,ii-7,mon,'=================',a2(179,44),a2(179,45)
c             print*,ii-7,mon,'=================',cn(179,44),cn(179,45)
c           endif
       
            do 4 j=1,jm
              lat=2*j-90
              abmxd=abmx
              if(abs(lat).gt.70) abmxd=0.67*abmx
            do 4 i=1,im
              sstp(i,j,mon)=-999.9
              ndp(i,j,mon)=0
              if(mask(i,j).eq.1)then
cliu     if(a2(i,j).gt.-990.)then
	        if(cn(i,j).gt.0.001)then
                   if(abs(a2(i,j)).le.abmxd)then
                     sstp(i,j,mon)=a2(i,j)
                     ndp(i,j,mon)=nint(cn(i,j))
                   endif
                 endif
                endif
 4          continue
           end do
c
c* Get the spatial and annual average.
           call avgsa(sstp,ndp,mask,a2)
c* Fill out a little with spatial binomial filters.
           call spsmb2(a2,mask)
c* Damp high-lat anomalies to go to 0 at the poles.
           call damphl(a2,mask)
      
c* The saan value for the update year is now in array a2(,).  We can
c* write this out for checks.  Note: update years begin 1985.  The
c* 15-year LF filter affects 7 years backwards to 1978.
           write(63) a2      
      
c* Update the array for LF and compute beginning 1978.
           do 6 j=1,jm
           do 6 i=1,im
              do k=1,14
                sm(i,j,k)=sm(i,j,k+1)
              end do
              sm(i,j,15)=a2(i,j)
              a2(i,j)=-999.9
              if(mask(i,j).eq.0) go to 6
              kn=0
              do k=1,15
                if(sm(i,j,k).gt.-990.)then
                  kn=kn+1
                  wk(kn)=sm(i,j,k)
                endif
              end do
              if(kn.ge.2) call median(wk,kn,a2(i,j))
 6         continue
 
           print*,ii-7,'???????????',a2(179,44),a2(179,45)
           write(61)a2
 95   continue
c* Finish up for the end years.
      do ky=1,7
         do 8 j=1,jm
         do 8 i=1,im
           do k=1,14
              sm(i,j,k)=sm(i,j,k+1)
           end do
           sm(i,j,15)=-999.9
           a2(i,j)=-999.9
           if(mask(i,j).eq.0) go to 8
           kn=0
           do k=1,15
              if(sm(i,j,k).gt.-990.)then
                kn=kn+1
                wk(kn)=sm(i,j,k)
              endif
           end do
           if(kn.ge.2) call median(wk,kn,a2(i,j))
 8       continue
 
c       print*,ii,ky
         print*,'???????????',ii-1-7+ky,a2(179,44),a2(179,45)
         write(61) a2
      end do
      print*, 'GREEN-SUCCESSFUL! lfsst.situ.v3b.f'
      call exit(1)
 100  print*, 'RED-STOP! lfsst.situ.v3b.f data problem 100'
      call exit(100)
 101  print*, 'RED-STOP! lfsst.situ.v3b.f data problem 101'
      call exit(100)
 91   format(10i5)
      end
c=====================END OF MAIN PROGRAM=========================
c@***s* ERSST/lfsst.situ.v3b.f/avgsa
c* NAME
c*  avgsa
c*
c* LOCATION
c*  inside $ERSST_SRC_DIR/lfsst.situ.v3b.f
c*
c* PURPOSE
c*  To compute an annual spatial average field from 12 months of data on a 2 deg grid.
c*
c* DESCRIPTION
c* The averaging on a 2 deg grid is performed in two steps. 1) From the monthly values, the moving 
c* average (window =26 deg width) of area-weighted input values is computed.  The corresponding
c* areal coverage is determined averages computed with less than 3% spatial coverage are screened out.
c* 2) The annual average per grid cell is then computed if there is at least 2 months of data.
c*
c* WARNING
c* Input data is assumed to be monthly, on a 2 deg grid  (lon: 0, 2E, ...2W)  (lat: 88S,..,0, ...88N). 
c* The area weights reflect the assumed grid points and cell size.
c*
c* AUTHOR
c*  Thomas M. Smith
c*
c* CREATION DATE
c*  December 2006 (presumed)
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* PROGRAM HISTORY
c*  2009 Sep: V. Banzon added comments as part of documentation effort
c*
c* INPUTS
c*	sstp	= monthly data on 2 deg grid to average 
c*	ndp 	= monthly counts of valid data on 2 deg grid
c*	mask	= 2 deg mask land/ice is 0; water is 1
c*
c  RETURNED  VALUE
c*	sa	= 2 deg grid with spatially smoothed annual averages
c*
c* PARAMETERS
c*	im,jum	= xsize, ysize of 2 deg grid
c*	cj		=areal weights that vary with latitude
c*
c* VARIABLES
c*	js,je	= indices for start and end of y-average spatial window
c*	ww	= weighted sea pixel (0 for land pixel)
c*	sus	= sum of weights for ocean grid cells with data present
c*	sut	= sum of weights for all ocean grid cells
c*	rn	= relative number of data (max=10)
c*	su1	= sum of area and count weighted data
c*	su2	= sum of area- relative count weights
c*
c* TODO
c*	Put this in a subroutine library
c*
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****

c* To first compute the 26-deg spatial average, then the annual average.
      subroutine avgsa(sstp,ndp,mask,sa)
      parameter(im=180,jm=89)
      real sstp(im,jm,12),cj(jm),sa(im,jm)
      integer mask(im,jm),ndp(im,jm,12)
      data cj/0.03,0.07,0.10,0.14,0.17,0.21,0.24,0.28,0.31,0.34,
     2        0.37,0.41,0.44,0.47,0.50,0.53,0.56,0.59,0.62,0.64,
     3        0.67,0.69,0.72,0.74,0.77,0.79,0.81,0.83,0.85,0.87,
     4        0.88,0.90,0.91,0.93,0.94,0.95,0.96,0.97,0.98,0.98,
     5        0.99,0.99,1.00,1.00,1.00,1.00,1.00,0.99,0.99,0.98,
     6        0.98,0.97,0.96,0.95,0.94,0.93,0.91,0.90,0.88,0.87,
     7        0.85,0.83,0.81,0.79,0.77,0.74,0.72,0.69,0.67,0.64,
     8        0.62,0.59,0.56,0.53,0.50,0.47,0.44,0.41,0.37,0.34,
     9        0.31,0.28,0.24,0.21,0.17,0.14,0.10,0.07,0.03/
c* First the spatial averaging over moving areas.
      do mon=1,12
       do 2 j0=1,jm
       js=j0-6
       if(js.lt.1) js=1
       je=j0+6
       if(je.gt.jm)je=jm
       do 2 i0=1,im
        sa(i0,j0)=-999.9
        if(mask(i0,j0).eq.0) go to 2
        su1=0.0
        su2=0.0
        sus=0.0
        sut=0.0
        do i=i0-6,i0+6
         id=i
         if(i.lt.1) id=i+im
         if(i.gt.im)id=i-im
         do j=js,je
          ww=cj(j)*real(mask(id,j))
          sut=sut+ww
          if(ndp(id,j,mon).gt.0)then
           sus=sus+ww
           rn=real(ndp(id,j,mon))
           if(rn.gt.10.0) rn=10.0
           su1=su1+rn*ww*sstp(id,j,mon)
           su2=su2+rn*ww
          endif
         end do
        end do
c* Require 3% spatial sampling (about 5 superobs).
        fsa=sus/sut
        if(fsa.ge.0.03) sa(i0,j0)=su1/su2
 2     continue
       do 4 j=1,jm
       do 4 i=1,im
        sstp(i,j,mon)=sa(i,j)
 4     continue
      end do
c* Next annual averaging with at least 2 months defined.
      do 6 j=1,jm
      do 6 i=1,im
       su1=0.0
       su2=0.0
       do mon=1,12
        if(sstp(i,j,mon).gt.-990.)then
         su1=su1+sstp(i,j,mon)
         su2=su2+1.0
        endif
       end do
       sa(i,j)=-999.9
       if(su2.ge.1.99) sa(i,j)=su1/su2
 6    continue
      return
      end
c
c----------------------------------------------------------------------------------------------
c@***s* ERSST/ lfsst.situ.v3b.f/spsmb2
c* NAME
c*  spsmb2
c*
c* LOCATION
c*  inside $ersst_SRC_DIR/lfsst.situ.v3b.f
c*
c* WARNING
c* 
c* PURPOSE  
c* To spatially smooth 2 deg gridded data and slightly fill using a binomial filter.
c*
c* DESCRIPTION
c* Within a 3 element window, binomial weights are applied to data to compute a smoothed average, 
c* first along the x axis, then along the y-axis.  This also does some filling in if there
c* is at least 1 data value within the window.
c*
c* WARNING
c* Input data is assumed to be monthly, on a 2 deg grid  (lon: 0, 2E, ...2W)  (lat: 88S,..,0, ...88N). 
c* The area weights reflect the assumed grid points and cell size.
c*
c* AUTHOR
c*  Thomas M. Smith
c*
c* CREATION DATE
c*  December 2006 (presumed)
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* PROGRAM HISTORY
c*  2009 Sep: V. Banzon added comments as part of documentation effort
c*
c* INPUT
c*	f	= averaged anomaly field 
c*	mask	= land is 0 so marks places not to fill
c*
c* RETURNED VARIABLE:
c*	f      = filtered anomaly field
c*
c* PARAMETERS
c*	im,jm	= xsize and ysize of grid
c*
c* VARIABLES
c*	cj	= latitudinally varying areas
c* 	wgt	= binomial filter weights
c*	su1	= sum of weighted anomalies
c*	su2	= sum of weights
c*
c* TODO
c*  Put this in a subroutine library
c*  cj is NOT used in this subroutine and can be removed
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
c* To spatially smooth and slightly fill using a binomial filter.
      subroutine spsmb2(f,mask)
      parameter(im=180,jm=89)
      real f(im,jm),wk(im,jm),cj(jm),wgt(3)
      integer mask(im,jm)
      data cj/0.03,0.07,0.10,0.14,0.17,0.21,0.24,0.28,0.31,0.34,
     2        0.37,0.41,0.44,0.47,0.50,0.53,0.56,0.59,0.62,0.64,
     3        0.67,0.69,0.72,0.74,0.77,0.79,0.81,0.83,0.85,0.87,
     4        0.88,0.90,0.91,0.93,0.94,0.95,0.96,0.97,0.98,0.98,
     5        0.99,0.99,1.00,1.00,1.00,1.00,1.00,0.99,0.99,0.98,
     6        0.98,0.97,0.96,0.95,0.94,0.93,0.91,0.90,0.88,0.87,
     7        0.85,0.83,0.81,0.79,0.77,0.74,0.72,0.69,0.67,0.64,
     8        0.62,0.59,0.56,0.53,0.50,0.47,0.44,0.41,0.37,0.34,
     9        0.31,0.28,0.24,0.21,0.17,0.14,0.10,0.07,0.03/
      data wgt/0.25,0.50,0.25/

      do j=1,jm
       do i0=1,im
        wk(i0,j)=-999.9
        if(mask(i0,j).eq.1)then
         kn=0
         su1=0.0
         su2=0.0
         do i=i0-1,i0+1
          kn=kn+1
          id=i
          if(i.lt.1) id=im
          if(i.gt.im)id=1
          if(f(id,j).gt.-990.)then
           su1=su1+wgt(kn)*f(id,j)
           su2=su2+wgt(kn)
          endif
         end do
         if(su2.gt.0.0) wk(i0,j)=su1/su2
        endif
       end do
      end do
c
      do i=1,im
       do j0=1,jm
        f(i,j0)=-999.9
        if(mask(i,j0).eq.1)then
         kn=0
         su1=0.0
         su2=0.0
         do 2 j=j0-1,j0+1
          kn=kn+1
          if(j.lt.1) go to 2
          if(j.gt.jm)go to 2
          if(wk(i,j).gt.-990.)then
           su1=su1+wgt(kn)*wk(i,j)
           su2=su2+wgt(kn)
          endif
 2       continue
         if(su2.gt.0.0) f(i,j0)=su1/su2
        endif
       end do
      end do
      return
      end
c----------------------------------------------------------------
c@***s* ERSST/ lfsst.situ.v3b.f/median
c*
c* NAME
c*  median
c*
c* LOCATION
c*  inside $ersst_SRC_DIR/lfsst.situ.v3b.f
c*
c* WARNING
c* Input data is assumed to contain 15 years of data
c*
c*PURPOSE  
c* To select the Median value within a 15-element window if  2 or more data values are available
c*
c* DESCRIPTION
c* The input array contains 15 elements (annual means) for a gridpoint. The number of values
c* are counted and if less than 2, returns a missing value.  Data are rearranged from largest
c* to smallest.  If the number of valid data is odd, then the center value is selected.  If the number
c* of valid data is even, the two center values are averaged.
c*
c* AUTHOR
c*  Thomas M. Smith
c*
c* CREATION DATE
c*  December 2006 (presumed)
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* PROGRAM HISTORY
c*  2009 Sep: V. Banzon added comments as part of documentation effort
c*
c* INPUTS
c*	ts	= 15 years data for single point 
c*	ntm	= number of years with data
c*
c* OUTPUTS 
c*	sm	= median
c*
c* PARAMETERS
c*   None
c*
c* VARIABLES
c*	ktm	= counter for valid data
c*	wk	= holds data, before and during reordering by magnitude
c*	tmp	= temporarily holds relatively smaller value during re-ordering
c*	sm	= median value 
c*
c* TODO
c* Put this in a subroutine library
c* cj is NOT used in this subroutine and can be removed
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
      subroutine median(ts,ntm,sm)
      real ts(*),wk(100)
      ktm=0
      do k=1,ntm
       if(ts(k).gt.-990.)then
        ktm=ktm+1
        wk(ktm)=ts(k)
       endif
      end do
      sm=-999.9
      if(ktm.lt.2) return
c if there are enough data (>2)
c compare each value with the highest of the previous comparison
c so the data is re-ordered from lowest to highest
c and k2 is the last index where the highest value is 
      do k1=1,ktm-1
       do k2=k1+1,ktm
        if(wk(k2).gt.wk(k1))then
         tmp=wk(k1)
         wk(k1)=wk(k2)
         wk(k2)=tmp
        endif
       end do
      end do
c k is middle position in wk array (and therefore the index of the median)
c this is computed based on number of data available is odd or even
c sm is set to the single or the average central values
      if(mod(ktm,2).eq.0)then
       k1=ktm/2
       k2=k1+1
       sm=(wk(k1)+wk(k2))/2.0
      else
       k=(ktm+1)/2
       sm=wk(k)
      endif
      return
      end
c-----------------------------------------------------------------------------
c@***s* ERSST/ lfsst.situ.v3b.f/damphl
c*
c* NAME
c*  damphl
c*
c* LOCATION
c*  inside $ersst_SRC_DIR/lfsst.situ.v3b.f
c*
c* WARNING
c* Assumes data is on a  Grid: lon 0E-358E by 2 deg, lat 88S-88N by 2 deg (180 by 89).
c*
c* PURPOSE  
c* To damp high-lat anoms to go to 0 at the poles.
c* 
c* DESCRIPTION
c* The data is forced to go to 0 at the poles, but this is done differently for S and N.
c* For the Antarctic, values at latitudes below 50 S decay to zero and are explicitly set to 0 below 80S.
c* For the Arctic, values at latitudes above 60N decay to zero, and effectivey reach 0 at 80 N.
c*
c* AUTHOR
c*  Thomas M. Smith
c*
c* CREATION DATE
c*  December 2006 (presumed)
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* PROGRAM HISTORY
c*  2009 Sep: V. Banzon added comments as part of documentation effort
c*
c* INPUTS
c*	a2	= data on 2 deg grid to be damped
c*	mask	= sea mask (land=0, sea=1)
c* 
c* OUTPUTS 
c*	a2	= damped data on 2 deg grid
c*
c* PARAMETERS
c*	im,jum	= xsize, ysize of 2 deg grid
c*
c* VARIABLES
c*	lat	= latitude
c*	fd	= damping factor
c*
c* TODO
c* Put this in a subroutine library
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
      subroutine damphl(a2,mask)
      parameter(im=180,jm=89)
      real a2(im,jm)
      integer mask(im,jm)
      do 2 j=1,jm
      lat=2*j-90
      fd=1.0
      if(lat.lt.-50.)then
       if(lat.le.-80.)then
        fd=0.0
       else
        fd=0.033333*real(lat+80)
       endif
      endif
      if(lat.gt.60.) fd=0.033333*real(90-lat)
      do 2 i=1,im
       if(mask(i,j).eq.1)then
        if(a2(i,j).gt.-990.)then
         a2(i,j)=fd*a2(i,j)
        else
         if(lat.gt.80) a2(i,j)=0.0
        endif
       endif
 2    continue
      return
      end
