c    67  1         2         3         4         5         6         7 2
c@***h*  ERSST/sst2d.situ.v3b.f
c*
c* NAME
c*  sst2d.situ.v3b.f
c*
c* LOCATION
c*  $ersst_SRC_DIR/sst2d.situ.v3b.f
c*
c* WARNING
c*  Change destination directories if not running operationally.
c*  Operationally, only the more recent years (from 1985) are updated
c*  and the historical period is left alone.
c* 
c* PURPOSE
c* To combine the updated low- and high-frequency SST analysis 
c* and add back on the climatology (taking into account updated sea ice) 
c* to reform the reconstructed SST to the most recent month.
c*
c* DESCIRPTION
c* The LF analysis and HF mode weights are read in.  Missing values in 
c* the HF mode weights are first filled using the lag-1 autocorrelation
c* to filter out the missing values using the surrounding defined values.
c* If no values are defined for a long time the missing weight values are
c* damped to zero.  The HF anomaly is then defined from the weighted sum
c* of the EOT modes.  The LF anomaly is added onto the HF anomaly.  
c* The climatology (centered on the 1971-2000 base period) used to compute
c* the original SST anomaly is added back on. Both the full and anomaly 
c* SST are written out after a linear adjustment to take into account 
c*	ci=ice concentrations
c* 	af= pivot point where ice begins affecting temp
c* 	tf = freezing temperature
c* As described in Smith and reynolds (2004), when:
c* 	ci=<af, then no SST adjustment
c*	ci>=0.9, then SST'=tf 
c* 	af<ci<0.9, then linear adjustment: SST' = SST + (ci-af)*(tf-SST)/(0.9-af).
c* 
c* AUTHOR
c*  Programmer: Thomas M Smith
c*
c* CREATION DATE
c*  December 2007
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* PROGRAM HISTORY
c*  Mar 2010 - C. Liu hardcoding
c*  Jan 2008 - C. Liu added ntm for operational run to avoid processing other years
c*  2009 Sep: V. Banzon added comments as part of documentation effort
c*  July 2006  - Chunying Liu modified the code to run at NCDC 
c*
c* INPUTS
c*  passed interactively or by script
c*	mondo, iydo ... the month and year to do
c*	unit 20 .... $ersst_DAT_DIR/input/static/clim.even.1971.2000.dat 
c*                  (2-deg monthly climatology: even gridpoints from T. Smith)
c*	unit 20 .... $ersst_DAT_DIR/inter/analysis/lfsst.1978.situ.last.v3b.dat 
c*		    (2-deg LF analysis SST anomalies from 1985 center)
c*	unit 21 .... $ersst_DAT_DIR/input/static/eot6.ev130.ano.dat 
c*                  (EOTs modes for HF analysis from T. Smith)
c*  	unit 22 .... $ersst_DAT_DIR/inter/analysis/hfsst.1985.situ.last.wgt.v3b.dat
c*                  (2-deg HF analysis weights from 1985)
c*  	unit 23 .... $ersst_DAT_DIR/input/static/ice_mask2d.dat
c*                  (2-deg ice mask to exclude caspian Sea and Great Lakes from T.Smith)
c* program replaces cyr
c*  	unit 22 .... $ersst_DAT_DIR/inter/ice2d/sice.avg2.mon.cyr.dat
c*
c* OUTPUTS
c* 2 deg grids (updated 1985-latest month)
c*      unit 51 ......... $ersst_DAT_DIR/output/sst2d/sst2d.situ.YYYYMM.v3b.dat
c*                          (analysis SST)
c*      unit 52 ......... $ersst_DAT_DIR/output/anom2d/sst2d.ano.situ.YYYYMM.v3b.dat
c*                         (SST anomaly)
c*
c* PARAMETERS
c*    im, jm	= x,y size of 2 deg grid
c*	iy1	= year to begin updating
c*	ntm2	= number of months for analysis period
c*	iy2	= end year to estimate ntm2
c*	nmo	= numer of EOT modes
c*	af	= minimum ice conc. when ice begins affecting temp
c*	afd	= difference bet 0.9 and actual ice concentration
c*
c* VARIABLES
c*	ntm	= total number of months to reconstruct from 1985
c*  2-D arrays (2 deg grids):
c*	a2	= in situ avg SST
c*		= reused to hold in situ SSTs (weighted)
c*		= reused to hold SST anomaly that is written to output
c*	t2	= in situ data counts
c*		= reused to hold full reconstructed SST
c*		= and then ice-adjusted SST that is written to output
c*	ci	= monthly sea ice for each year
c*	mask2d	= ice mask
c*	mask	= land/sea mask (land=0, sea=1)
c*	alf	= LF SST anomalies
c*  3-D arrays (2 deg grids, by 12 months):
c* 	clim	= climatology for 1971-2000 to be added back
c*  3-D arrays (2 deg grids, by 130 modes):
c*	gx	= eot modes
c*  2-D arrays (130 modes by number of months analyzed)
c*	wt	= HF weights per mode per month
c*
c* SUBPROGRAMS
c*	intts
c*	$ersst_LIB_DIR/maxmin.f
c*
c* TODO
c*  put range in subroutine library
c*  pass environment variables
c*  cadj declared but not used
c*
c* LANGUAGE
c*  Fortran
c*
c@*****
c=============================END OF HEADER======================================
      parameter(im=180,jm=89,iy1=1985,iy2=2010,nmo=130)
      parameter(ntm2=12*(1+iy2-iy1))
      parameter(af=0.6,afd=0.9-af)
c
      real a2(im,jm),t2(im,jm),clim(im,jm,12),ci(im,jm),cadj(im,jm,12)
      real gx(im,jm,nmo),wt(nmo),alf(im,jm),mask2d(im,jm),wts(nmo,ntm2)
      real ts1(ntm2)
      integer mask(im,jm)
      character fm*11,st*3,cyr*4,date*6
      data fm/'unformatted'/,st/'old'/
      CHARACTER(LEN=80) :: Dir_data,fname

c
      print*, 'Input the year to do:'
      read(5,*) iydo
      print*, 'Input the month to do:'
      read(5,*) mondo
      
c   caculate ntm   Liu added for the operational run

       ntm=mondo+12*(iydo-iy1)
      
       print*,'ntm=',ntm

       CALL getenv("ersst_DAT_DIR", Dir_data)      
      

c* Read in the 2-deg climatology files: adjusted OI and recentered.
c* The adj OI is to reconstruct full SST, the recentered to ensure
c* that computed anomalies have a zero mean for the base period.
      
       fname=adjustl(adjustr(Dir_data) //
     &   '/input/static/clim.even.1971.2000.dat')
       open(20,status=st,form=fm,iostat=ierr,file=fname) 
       IF (ierr.NE.0) PRINT*,'Error opening:'//fname

      
       do mon=1,12
          read(20,end=100,err=100) a2
          do 2 j=1,jm
          do 2 i=1,im
            clim(i,j,mon)=a2(i,j)
 2        continue
       end do
       close(20)
       print*, ' Got the climatology.'

c* Set up the 2-d mask (land = 0, sea = 1).
       do j=1,jm
          do i=1,im
            mask(i,j)=0
            if(a2(i,j).gt.-990.) mask(i,j)=1
          end do
       end do

c* Read in the EOT modes.

       fname=adjustl(adjustr(Dir_data) //
     &   '/input/static/eot6.ev130.ano.dat')
       open(21,status=st,form=fm,iostat=ierr,file=fname) 
       IF (ierr.NE.0) PRINT*,'Error opening:'//fname

       do m=1,nmo
         read(21,end=100,err=100) a2
         do j=1,jm
            do i=1,im
              gx(i,j,m)=a2(i,j)
            end do
         end do
       end do
       close(21)
       print*, 'Completed reading'//fname 
       call maxmin(gx(1:im,1:jm,1),im,jm,-999.9,'gx      ')
       call maxmin(gx(1:im,1:jm,100),im,jm,-999.9,'gx      ')
       call maxmin(gx(1:im,1:jm,130),im,jm,-999.9,'gx      ')
      
c* Read in the HF weights

       fname=adjustl(adjustr(Dir_data) //
     &   '/inter/analysis/hfsst.1985.situ.last.wgt.v3b.dat')
 
      open(21,status=st,form=fm,iostat=ierr,file=fname)
       IF (ierr.NE.0) PRINT*,'Error opening:'//fname
      
      
       do iy=iy1,iydo
          mme=12
          if(iy.eq.iydo)mme=mondo
          do mon=1,mme
             nt=mon+12*(iy-iy1)
             read(21,end=100,err=100) kmo,kyr,nfit,wt
c             if(iy.eq.1985 .and.mon.eq.1)then
c        print*,'************',iy, mon,kmo,kyr,nfit,wt(1:10),wt(125:129)
c             endif
	
cliu        do nm=1,nmo
cliu           if(wt(nm).lt.-990.) wt(nm)=0.0
cliu        end do
	
	     do nm=1,nmo
                wts(nm,nt)=wt(nm)
             end do
          end do
       end do
       close(21)
      print*, 'Completed reading'//fname 

c* fill missing values with damped interpolation.      
       do nm=1,nmo
         do nt=1,ntm
            ts1(nt)=wts(nm,nt)
         end do
         call intts(ts1,ntm,nm)
         do nt=1,ntm
            wts(nm,nt)=ts1(nt)
         end do
       end do 
      
ccccccc  nt=mon+12*(iy-iy1) = 1 for 1985/1, =145 for 1997/1
       print*,'wts=',wts(130,1),wts(130,145)
       call maxmin(wts,nm,nt,undef,'wts     ')     
c
c* Open the file of LF analysis.
c  skip to 1985
       fname=adjustl(adjustr(Dir_data) //
     &     '/inter/analysis/lfsst.1978.situ.last.v3b.dat')
       open(20,status=st,form=fm,iostat=ierr,file=fname)
       IF (ierr.NE.0) PRINT*,'Error opening:'//fname

       do iy=1978,1984
          read(20,end=100,err=100) alf
       end do

c* compute the SST & adjust with sea ice data.
cliu  open the ice mask to mask the Great lakes and Caspian sea


       fname=adjustl(adjustr(Dir_data) //
     &     '/input/static/ice_mask2d.dat')
       open(23,status=st,form=fm,iostat=ierr,file=fname)
       IF (ierr.NE.0) PRINT*,'Error opening:'//fname

       read(23) mask2d
       undef = -999.9
       call maxmin(mask2d,im,jm,undef,'mask2d  ')

       do iy=iy1,iydo
          write(cyr,'(i4)') iy
       
c read monthly sea ice
          fname=adjustl(adjustr(Dir_data) //
     &       '/inter/ice2d/sice.avg2.mon.'//cyr//'.dat')
          open(22,status=st,form=fm,iostat=ierr,file=fname)
          IF (ierr.NE.0) PRINT*,'Error opening:'//fname

          if(iy.lt.iydo)then
            read(20,end=100,err=100) alf
          else
            if(iy.eq.iydo.and.mondo.eq.12) read(20,end=100,err=100) alf
          endif
       
c          if(iy.eq.1985)then
c             print*,'************alf=',iy, alf(72,64)
c          endif
       
          mme=12
          if(iy.eq.iydo)mme=mondo
          do mon=1,mme
             nt=12*(iy-iy1)+mon
       
             write (date,'(i4.4,i2.2)')  iy,mon

             fname=adjustl(adjustr(Dir_data) //
     &       '/output/sst2d/sst2d.situ.'//date//'.v3b.dat')
             open(51,form=fm,iostat=ierr,file=fname)
             IF (ierr.NE.0) PRINT*,'Error opening:'//fname


            fname=adjustl(adjustr(Dir_data) //
     &       '/output/anom2d/sst2d.ano.situ.'//date//'.v3b.dat')
            open(52,form=fm,iostat=ierr,file=fname)
            IF (ierr.NE.0) PRINT*,'Error opening:'//fname


c* Read ice data
            read(22) ci
	
	    if(iy.eq.1997.and.mon.eq.12)then
	       call maxmin(ci,im,jm,-999.9,'ice     ')
	    endif
	
	    do j=1,jm
               do i=1,im
        if(mask2d(i,j).ge.1.and.mask2d(i,j).le.4 ) ci(i,j)=undef
               end do
            end do
	
		
	
c	if(iy.eq.1985 .and.mon.eq.1)then
c        print*,'************ice=',iy, mon,ci(72,64)
c        endif
	
	
c        print*,iy, mon, nt,'-------wt wt wt wts=',wts(1:1,nt)
	
c* Compute the recon SST for the month.
           do 4 j=1,jm
              lat=2*j-90
           do 4 i=1,im
              t2(i,j)=-999.9
              a2(i,j)=-999.9
              if(mask(i,j).eq.0)go to 4
              lon=2*i-2
c*  tf = -1.8 except for the Great Lakes, when tf=0.
              tf=-1.80	 
              if(lat.ge.42.and.lat.le.48)then
                 if(lon.ge.270.and.lon.le.284) tf=0.00
              endif
              sum=alf(i,j)
	 
c	 if(iy.eq.1985 .and.mon.eq.1.and.i.eq.72.and.j.eq.64)then
c         print*,'************SSTsum=',iy, mon,'sum=',sum
c         endif
	 
 
c         if(iy.eq.1985 .and.mon.eq.1.and.i.eq.72.and.j.eq.64)then
c	 print*,'nt=',nt
c         print*,'************=',iy, mon,'gx=',gx(i,j,1:10)
c	 print*,'************=',iy, mon,'wts=',wts(1:10,nt)
c         endif
	 
 
         do nm=1,nmo
          sum=sum+gx(i,j,nm)*wts(nm,nt)
         end do
         t2(i,j)=sum+clim(i,j,mon)
c	 
c	 
c	 if(iy.eq.1985 .and.mon.eq.1.and.i.eq.72.and.j.eq.64)then
c         print*,'***********SSTs=',iy, mon,t2(72,64),clim(72,64,mon),sum
c         endif
	 
	 
         if(t2(i,j).lt.tf) t2(i,j)=tf
         if(ci(i,j).gt.af)then
          if(ci(i,j).ge.0.9)then
           t2(i,j)=tf
          else
           tt=t2(i,j)
           t2(i,j)=tt+(ci(i,j)-af)*(tf-tt)/afd
          endif
         endif
cliu v3b         a2(i,j)=t2(i,j)-cadj(i,j,mon)

         a2(i,j)=t2(i,j)-clim(i,j,mon)
	 
c* Set all sea values from 78S to the S. Pole to land.
         if(lat.le.-78)then
          t2(i,j)=-999.9
          a2(i,j)=-999.9
         endif
	 
 4      continue
c
c* Write out the full SST.
        write(51) t2
	
c        if(iy.eq.1985 .and.mon.eq.1)then
c        print*,'************SST=',iy, mon,t2(72,64)
c        endif
	
c* Write out the anomaly SST.
        write(52) a2
        end do
c* Note: since ice concentrations begin 1870 and the cycle does not
c* change till about 1900, use the 1870 cycle of concentrations for
c* 1854-1870.
       
c       if(mod(iy,5).eq.0) print*, 'Finished year ',iy
       print*,'Finished year ',iy
       
       end do
       
      print*, 'GREEN-SUCCESSFUL! sst2d.situ.v3b.f'
      call exit(1)
 100  print*, 'RED-STOP! sst2d.situ.v3b.f data problem'
      call exit(100)
      end
c*===================END MAIN====================================
c@***s* ERSST/sst2d.situ.v3b.f/intts
c* NAME
c*  intts
c*
c* LOCATION
c*  inside $ersst_SRC_DIR//sst2d.situ.v3b.f
c*
c* WARNING
c* 
c* PURPOSE  
c* To use the temporal lag-1 autocorrelation to fill in missing
c* values of the weights.  
c*
c* DESCRIPTION
c* The weights are filled using autocorrelation in both the 
c* forwards and reverse directions, and the two are then
c* averaged.  Defined modes are not affected.  The AC3 values
c* come from T. Smith.
c*
c* AUTHOR
c*  Thomas M Smith
c*
c* CREATION DATE
c*  December 2007
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* PROGRAM HISTORY
c*  2009 Sep: V. Banzon added comments as part of documentation effort
c*
c* INPUTS
c* 	ts1	= time series of mode weights with missing values
c*	ntm	= number of months in time series
c*	nm	= mode number
c*
c* RETURNED VALUE
c*	ts1	= time series of mode weights with missing values filled in
c* 
c* PARAMETERS
c*	nmo	= number of modes
c*
c* VARIABLES
c*	ts1	= time series of mode weights
c*	w1	= weights
c*	ac1	= autocorrelation for 3m weights
c*
c* TODO
c*	Put this in a subroutine library
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
      subroutine intts(ts1,ntm,nm)
      parameter(nmo=130)
      real ts1(ntm),w1(ntm),ac1(nmo)
c* AC1 for 1m weights (130 modes):
c      data ac1/0.82,0.67,0.68,0.35,0.52,0.76,0.55,0.63,0.54,0.51,
c     2         0.66,0.27,0.56,0.59,0.58,0.62,0.43,0.50,0.52,0.54,
c     3         0.58,0.64,0.62,0.53,0.49,0.58,0.65,0.44,0.43,0.54,
c     4         0.66,0.41,0.67,0.46,0.53,0.54,0.31,0.55,0.57,0.48,
c     5         0.43,0.67,0.59,0.59,0.53,0.60,0.50,0.43,0.33,0.40,
c     6         0.53,0.35,0.56,0.34,0.53,0.38,0.49,0.45,0.54,0.58,
c     7         0.35,0.49,0.57,0.59,0.64,0.29,0.59,0.37,0.57,0.49,
c     8         0.56,0.44,0.44,0.51,0.57,0.60,0.60,0.60,0.43,0.45,
c     9         0.27,0.26,0.47,0.47,0.39,0.37,0.46,0.40,0.60,0.52,
c     1         0.40,0.39,0.65,0.31,0.60,0.40,0.43,0.46,0.38,0.56,
c     1         0.51,0.40,0.53,0.58,0.42,0.56,0.30,0.47,0.43,0.54,
c     2         0.32,0.36,0.61,0.47,0.46,0.42,0.53,0.41,0.47,0.41,
c     3         0.49,0.39,0.50,0.42,0.44,0.37,0.46,0.44,0.36,0.36/
c* AC1 for 3m weights (130 modes):
      data ac1/0.89,0.87,0.69,0.75,0.65,0.79,0.57,0.64,0.65,0.75,
     2         0.66,0.66,0.69,0.70,0.63,0.64,0.73,0.77,0.76,0.68,
     3         0.82,0.68,0.73,0.59,0.73,0.61,0.72,0.71,0.67,0.57,
     4         0.67,0.65,0.70,0.51,0.70,0.75,0.67,0.72,0.61,0.51,
     5         0.65,0.70,0.71,0.67,0.71,0.71,0.72,0.75,0.52,0.73,
     6         0.60,0.56,0.75,0.61,0.72,0.70,0.69,0.68,0.63,0.63,
     7         0.66,0.70,0.63,0.62,0.66,0.60,0.71,0.56,0.63,0.64,
     8         0.64,0.53,0.66,0.61,0.71,0.67,0.65,0.69,0.64,0.63,
     9         0.69,0.65,0.68,0.53,0.53,0.65,0.52,0.51,0.67,0.59,
     1         0.60,0.60,0.68,0.66,0.68,0.54,0.62,0.56,0.50,0.60,
     1         0.57,0.62,0.66,0.62,0.63,0.63,0.53,0.57,0.54,0.58,
     2         0.58,0.58,0.62,0.55,0.52,0.62,0.68,0.68,0.64,0.63,
     3         0.60,0.54,0.58,0.58,0.60,0.52,0.68,0.61,0.61,0.41/
c
c*  1. Forwards.
      w1(1)=ts1(1)
      if(ts1(1).lt.-990.) w1(1)=0.0
      do nt=2,ntm
       w1(nt)=ts1(nt)
       if(ts1(nt).lt.-990.) w1(nt)=w1(nt-1)*ac1(nm)
      end do
c*  2. Backwards & average.
      if(ts1(ntm).lt.-990.) ts1(ntm)=w1(ntm)
      do nt=ntm-1,1,-1
       if(ts1(nt).lt.-990.)then
        ts1(nt)=(ts1(nt+1)*ac1(nm)+w1(nt))/2.0
       endif
      end do
      return
      end
      
      include 'maxmin.f'
